@@ -1,4 +1,4 @@
- /* Copyright (C) 2001-2005 Tresys Technology, LLC
+ /* Copyright (C) 2001-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /* 
@@ -14,7 +14,7 @@
 #include <tcl.h>
 #include <tk.h>
 #include <getopt.h>
-#include "../libapol/apol_tcl.h"
+#include "../libapol/apol_tcl_other.h"
 #include "../libapol/util.h"
 
 #ifndef STARTUP_SCRIPT
