@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2005 Tresys Technology, LLC
+# Copyright (C) 2004-2006 Tresys Technology, LLC
 # see file 'COPYING' for use and warranty information 
 
 # TCL/TK GUI for SE Linux policy analysis
@@ -120,7 +120,12 @@ proc Apol_Cond_Bools::cond_bool_search_bools {} {
 	
 		$resultsbox configure -state normal
 		$resultsbox delete 0.0 end
-		$resultsbox insert end $results
+                $resultsbox insert end "BOOLEANS:\n"
+                if {$results == ""} {
+                        $resultsbox insert end "Search returned no results."
+                } else {
+                        $resultsbox insert end $results
+                }
 		ApolTop::makeTextBoxReadOnly $resultsbox 
 	} else {
 		tk_messageBox -icon error -type ok -title "Error" -message "No current policy file is opened!"
@@ -216,26 +221,6 @@ proc Apol_Cond_Bools::cond_bool_set_bool_value {bool_name} {
 	return 0	
 } 
 
-################################################################
-#  ::cond_bool_embed_buttons
-#
-proc Apol_Cond_Bools::cond_bool_embed_buttons {widget bool_name} {	
-	set rb_frame [frame $widget.rb_frame:$bool_name -bd 0 -bg white]
-	set rb_true  [radiobutton $rb_frame.rb_true:$bool_name -bg white \
-		-variable Apol_Cond_Bools::cond_bools_value_array($bool_name) \
-		-command "Apol_Cond_Bools::cond_bool_set_bool_value $bool_name" \
-		-value 1 -highlightthickness 0 -text "True"]
-	set rb_false [radiobutton $rb_frame.rb_false:$bool_name -bg white \
-		-variable Apol_Cond_Bools::cond_bools_value_array($bool_name) \
-		-command "Apol_Cond_Bools::cond_bool_set_bool_value $bool_name" \
-		-value 0 -highlightthickness 0 -text "False"]
-	
-	pack $rb_frame -side left -anchor nw
-	pack $rb_true $rb_false -side left -anchor nw -padx 2
-			
-	return $rb_frame	
-} 
-
 ################################################################
 #  ::cond_bool_init_state
 #
@@ -251,40 +236,40 @@ proc Apol_Cond_Bools::cond_bool_init_state { } {
 #  	- Method for remove all embedded check buttons.
 # 
 proc Apol_Cond_Bools::cond_bool_remove_listbox_items { } {   
-	variable cond_bools_listbox
+    variable cond_bools_listbox
 
-	foreach item [$cond_bools_listbox items] {
-		set window [$cond_bools_listbox itemcget $item -window]
-    		if { [winfo exists $window] } {
-			destroy $window
-		}
-	}
-	# Delete
-	$cond_bools_listbox delete [$cond_bools_listbox items]
-	return 0	
+    foreach w [winfo children [$cond_bools_listbox getframe]] {
+        catch {destroy $w}
+    }
+    # get rid of the scrollbars
+    [$cond_bools_listbox getframe] configure -width 1 -height 1
 }
 
 ################################################################
 #  ::cond_bool_insert_listbox_items
 #
 proc Apol_Cond_Bools::cond_bool_insert_listbox_items { } {
-	variable cond_bools_listbox 
-	variable cond_bools_list
-	
-	foreach bool_name $cond_bools_list {
-		$cond_bools_listbox insert end $bool_name -text " - $bool_name" \
-		 	 -window [Apol_Cond_Bools::cond_bool_embed_buttons \
-		 	 	$Apol_Cond_Bools::cond_bools_listbox $bool_name]  
-		  
-	}
+    variable cond_bools_listbox 
+    variable cond_bools_list
 
-    	# Display updates immediately.
-    	# Adjust the view so that no part of the canvas is off-screen to the left.
-	$cond_bools_listbox configure -redraw 1
-    	$cond_bools_listbox.c xview moveto 0			 	 
-    	update idletasks
-    	$cond_bools_listbox configure -padx [winfo reqwidth [$cond_bools_listbox itemcget [$cond_bools_listbox items 0] -window]]	
-	return 0
+    set subf [$cond_bools_listbox getframe]
+    $subf configure -width 0 -height 0
+    foreach bool_name $cond_bools_list {
+        set rb_true [radiobutton $subf.rb_true:$bool_name -bg white \
+                         -variable Apol_Cond_Bools::cond_bools_value_array($bool_name) \
+                         -command [list Apol_Cond_Bools::cond_bool_set_bool_value $bool_name] \
+                         -value 1 -highlightthickness 0 -text "True"]
+        set rb_false [radiobutton $subf.rb_false:$bool_name -bg white \
+                          -variable Apol_Cond_Bools::cond_bools_value_array($bool_name) \
+                          -command [list Apol_Cond_Bools::cond_bool_set_bool_value $bool_name] \
+                          -value 0 -highlightthickness 0 -text "False"]
+        set rb_label [label $subf.rb_label:$bool_name \
+                          -bg white -text "- $bool_name"]
+        grid $rb_true $rb_false $rb_label -padx 2 -pady 5 -sticky w
+    }
+    $cond_bools_listbox configure -areaheight 0 -areawidth 0
+    $cond_bools_listbox xview moveto 0
+    $cond_bools_listbox yview moveto 0
 } 
 
 ################################################################
@@ -408,49 +393,38 @@ proc Apol_Cond_Bools::create {nb} {
 	variable cond_bools_listbox 
 	variable resultsbox 
 	variable cb_RegExp
-	
+
 	# Layout frames
 	set frame [$nb insert end $ApolTop::cond_bools_tab -text "Booleans"]
-	set topf  [frame $frame.topf]
-	set pw1   [PanedWindow $topf.pw -side top]
-	set pane  [$pw1 add ]
-	set spane [$pw1 add -weight 5]
-	set pw2   [PanedWindow $pane.pw -side left]
-	set rpane [$pw2 add -weight 3]
-	
+        set pw [PanedWindow $frame.pw -side top]
+        set left_pane [$pw add -weight 0]
+        set right_pane [$pw add -weight 1]
+        pack $pw -expand 1 -fill both
+
 	# Title frames
-	set cond_bools_box [TitleFrame $rpane.cond_bools_box -text "Booleans"]
-	set s_optionsbox   [TitleFrame $spane.obox -text "Search Options"]
-	set rslts_frame	   [TitleFrame $spane.rbox -text "Search Results"]
-	
-	# Placing layout
-	pack $topf -fill both -expand yes 
-	pack $pw1 -fill both -expand yes
-	pack $pw2 -fill both -expand yes
-	
-	# Placing title frames
-	pack $s_optionsbox -padx 2 -fill both
-	pack $cond_bools_box -padx 2 -side left -fill both -expand yes
-	pack $rslts_frame -pady 2 -padx 2 -fill both -anchor n -side bottom -expand yes
-	
-	# Roles listbox widget
-	set sw_r [ScrolledWindow [$cond_bools_box getframe].sw -auto both]
-	set cond_bools_listbox [ListBox [$cond_bools_box getframe].cond_bools_listbox \
-	          -relief sunken -borderwidth 2 -bg white  \
-	          -selectmode none -deltay 25 \
-	          -width 25 -highlightthickness 0 \
-	          -redraw 0]
-	$sw_r setwidget $cond_bools_listbox 
-	
-	set button_defaults [button [$cond_bools_box getframe].button_defaults \
+	set cond_bools_box [TitleFrame $left_pane.cond_bools_box -text "Booleans"]
+	set s_optionsbox   [TitleFrame $right_pane.obox -text "Search Options"]
+	set rslts_frame	   [TitleFrame $right_pane.rbox -text "Search Results"]
+	pack $cond_bools_box -padx 2 -expand 1 -fill both
+	pack $s_optionsbox -padx 2 -fill x -expand 0 -side top
+	pack $rslts_frame -pady 2 -padx 2 -fill both -side bottom -expand yes
+
+	# Booleans listbox widget
+        set left_frame [$cond_bools_box getframe]
+        set sw_b [ScrolledWindow $left_frame.sw -auto both]
+        set cond_bools_listbox [ScrollableFrame $sw_b.cond_bools_listbox -bg white -width 200]
+        set subf [$cond_bools_listbox getframe]
+	$sw_b setwidget $cond_bools_listbox
+	set button_defaults [button $left_frame.button_defaults \
 		-text "Reset to policy defaults" \
 		-command {Apol_Cond_Bools::cond_bool_set_bool_values_to_policy_defaults}]
-	    	
+        pack $sw_b -side top -expand 1 -fill both
+        pack $button_defaults -side bottom -expand 0 -fill x
+
 	# Search options subframes
 	set ofm [$s_optionsbox getframe]
 	set l_innerFrame [LabelFrame $ofm.l_innerFrame]
 	set c_innerFrame [LabelFrame $ofm.c_innerFrame]
-	set buttons_f    [LabelFrame $ofm.buttons_f]
 	
 	set cb_bools_default_state [checkbutton $c_innerFrame.default_state \
 		-variable Apol_Cond_Bools::search_opts(default_state) \
@@ -484,8 +458,8 @@ proc Apol_Cond_Bools::create {nb} {
 	bind bool_vars_combo_box_Tag <KeyPress> { ApolTop::_create_popup $Apol_Cond_Bools::bool_combo_box %W %K }
 			
 	# Action Buttons
-	set ok_button [button [$buttons_f getframe].ok -text "Search for Booleans" -width 15 -command {Apol_Cond_Bools::cond_bool_search_bools}]
-	#button $rfm.print -text Print -width 6 -command {ApolTop::unimplemented}
+	set ok_button [button $ofm.ok -text "OK" -width 6 \
+                           -command Apol_Cond_Bools::cond_bool_search_bools]
 	
 	# Display results window
 	set sw_d [ScrolledWindow [$rslts_frame getframe].sw -auto none]
@@ -494,14 +468,12 @@ proc Apol_Cond_Bools::create {nb} {
 	
 	# Placing all widget items
 	pack $button_defaults -side bottom -pady 2 -anchor center
-	pack $ok_button -side top -anchor e -pady 5 -padx 5
-	pack $buttons_f -side right -expand yes -fill both -anchor nw -padx 4 -pady 4
+	pack $ok_button -side right -anchor ne -padx 5 -pady 5
 	pack $l_innerFrame $c_innerFrame -side left -fill y -anchor nw -padx 4 -pady 4
 	
 	pack $cb_enable_bool_combo_box $bool_combo_box -side top -anchor nw -fill x
 	pack $cb_RegExp -side top -anchor nw 
 	pack $cb_bools_default_state $cb_bools_curr_state -side top -anchor nw 
-	pack $sw_r -fill both -expand yes
 	pack $sw_d -side left -expand yes -fill both 
 	Apol_Cond_Bools::cond_bool_init_state
 	
