@@ -1,7 +1,7 @@
 #############################################################
 #  dirflow_module.tcl  
 # -----------------------------------------------------------
-#  Copyright (C) 2003-2005 Tresys Technology, LLC
+#  Copyright (C) 2003-2006 Tresys Technology, LLC
 #  see file 'COPYING' for use and warranty information 
 #
 #  Requires tcl and tk 8.3+, with BWidgets
@@ -83,6 +83,9 @@ namespace eval Apol_Analysis_dirflow {
     	Apol_Analysis::register_analysis_modules "Apol_Analysis_dirflow" "Direct Information Flow"
 }
 
+proc Apol_Analysis_dirflow::get_short_name {} {
+    return "Direct Flow"
+}
 
 
 ## Apol_Analysis_dirflow::initialize is called when the tool first starts up.  The
