@@ -1,7 +1,7 @@
 #############################################################
 #  foo_module.tcl  
 # -----------------------------------------------------------
-#  Copyright (C) 2003-2005 Tresys Technology, LLC
+#  Copyright (C) 2003-2006 Tresys Technology, LLC
 #  see file 'COPYING' for use and warranty information 
 #
 #  Requires tcl and tk 8.3+, with BWidgets
@@ -268,3 +268,7 @@ proc Apol_Analysis_foo::create_options { options_frame } {
 	pack $sLabel $sEntry -side top -anchor nw
 	return 0	
 }
+
+proc Apol_Analysis_foo::get_short_name {} {
+    return "Foo"
+}
