@@ -1,4 +1,4 @@
-# Copyright (C) 2003 Tresys Technology, LLC
+# Copyright (C) 2003-2005 Tresys Technology, LLC
 # see file 'COPYING' for use and warranty information 
  
 # TCL/TK GUI for SE Linux policy analysis
@@ -87,9 +87,11 @@ proc Apol_Perms_Map::determine_loaded_pmap { } {
 	set Apol_Perms_Map::edit_flag	0
 	if {[string equal $loaded_pmap $user_default_pmap]} {
 		set dflt_pmap_flg 1
+		set system_dflt_flg 0
 		set title_display $Apol_Perms_Map::dflt_pmap_display
 	} elseif {[string equal [file dirname $loaded_pmap] $sys_dflt_pmap_dir]} {
 		set system_dflt_flg 1
+		set dflt_pmap_flg 0
 		set title_display $Apol_Perms_Map::sys_dflt_pmap_display
 	} elseif {$is_mls_loaded} {
 		set dflt_pmap_flg 0
@@ -507,11 +509,11 @@ proc Apol_Perms_Map::embed_mls_perms_widgets {list_b class selinux_perm} {
 	
 	if {[Apol_Perms_Map::is_mls_perm_undefined $Apol_Perms_Map::mls_base_perms_array($class,$selinux_perm)]} {
 		$lbl1 configure -text "$selinux_perm *" -fg red
-		$cb_read configure -state disabled
-		$cb_write configure -state disabled
-		$cb_both configure -state disabled
-		$cb_none configure -state disabled
-		$spinbox_weight configure -state disabled
+		#$cb_read configure -state disabled
+		#$cb_write configure -state disabled
+		#$cb_both configure -state disabled
+		#$cb_none configure -state disabled
+		#$spinbox_weight configure -state disabled
 	} else {
 		$lbl1 configure -text "$selinux_perm"
 	}
@@ -657,6 +659,8 @@ proc Apol_Perms_Map::render_pmap_Dlg { } {
     	toplevel $perm_mappings_Dlg 
      	wm withdraw $perm_mappings_Dlg	
     	wm title $perm_mappings_Dlg "Edit Permissions Mappings: $Apol_Perms_Map::title_display"
+    	wm protocol $perm_mappings_Dlg WM_DELETE_WINDOW " "
+    	
         set topf  [frame $perm_mappings_Dlg.topf]
         set pw1   [PanedWindow $topf.pw -side top]
         set pane  [$pw1 add ]
@@ -701,15 +705,14 @@ proc Apol_Perms_Map::render_pmap_Dlg { } {
 	pack $sw_list -fill both -expand yes 
         pack $b_save $b_saveas $b_saveas_Dflt $b_exit -side left -padx 5 -pady 5 -anchor center -expand yes
         pack $botf -side left -expand yes -anchor center 
-        
-        wm protocol $perm_mappings_Dlg WM_DELETE_WINDOW "Apol_Perms_Map::close_Dlg"
-   	
+           	
         # Configure top-level dialog specifications
         set width 800
 	set height 600
 	wm geom $perm_mappings_Dlg ${width}x${height}
 	wm deiconify $perm_mappings_Dlg
 	focus $perm_mappings_Dlg
+	wm protocol $perm_mappings_Dlg WM_DELETE_WINDOW "Apol_Perms_Map::close_Dlg"
 	return 0
 }
 
