@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2003 Tresys Technology, LLC
+# Copyright (C) 2001-2005 Tresys Technology, LLC
 # see file 'COPYING' for use and warranty information 
 
 # TCL/TK GUI for SE Linux policy analysis
@@ -102,7 +102,11 @@ proc Apol_RBAC::searchRoles {} {
 	} else {
 	    	$resultsbox configure -state normal
 		$resultsbox delete 0.0 end
-		$resultsbox insert end $results
+		if {$results == ""} {
+			$resultsbox insert end "0 rules match the search criteria."
+		} else {
+			$resultsbox insert end $results
+		}
 		ApolTop::makeTextBoxReadOnly $resultsbox 
         }
 	
@@ -432,7 +436,7 @@ proc Apol_RBAC::create {nb} {
     set pw2 [PanedWindow $topf.pw2 -side top -weights available]
     $pw2 add -minsize 110
     $pw2 add -weight 6
-    set obox [TitleFrame [$pw2 getframe 1].obox -text "Search Options"]
+    set obox [TitleFrame [$pw2 getframe 1].obox -text "Search Criteria"]
     set tbox [TitleFrame [$pw2 getframe 0].tbox -text "Rule Selection"]
     set dbox [TitleFrame $bottomf.dbox -text "RBAC Rules Display"]
     #set okbox [frame $topf.okbox]
