@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2005 Tresys Technology, LLC
+# Copyright (C) 2001-2006 Tresys Technology, LLC
 # see file 'COPYING' for use and warranty information 
 
 # TCL/TK GUI for SE Linux policy analysis
@@ -13,19 +13,19 @@ namespace eval ApolTop {
 	# All capital letters is the convention for variables defined via the Makefile.
 	variable bwidget_version	""
 	variable status 		""
-	variable polversion 		""
+	variable policy_version_string	""
 	variable policy_type		""
-	variable binary_policy_type	"binary"
-	variable source_policy_type	"source"
+    variable policy_mls_type	""
 	variable filename 		""
 	# The following is used with opening a policy for loading all or pieces of a policy. 
 	# The option defaults to 0 (or all portions of the policy).
 	variable policy_open_option	0
 	variable policyConf_lineno	""
-	variable polstats 		""
+	variable polstats
+        variable policy_stats_summary   ""
 	# The version number is defined as a magical string here. This is later configured in the make environment.
 	variable gui_ver 		APOL_GUI_VERSION 
-	variable copyright_date		"2001-2005"
+	variable copyright_date		"2001-2006"
 	# install_dir is a magical string to be defined via the makefile!
 	variable apol_install_dir	APOL_INSTALL_DIR
 	variable recent_files
@@ -73,6 +73,8 @@ namespace eval ApolTop {
 	# Subordinate notebook widgets
 	variable components_nb
 	variable rules_nb
+
+    variable mls_tabs {}  ;# list of notebook tabs that are only for MLS
 	
 	# Search-related variables
 	variable searchString		""
@@ -86,20 +88,35 @@ namespace eval ApolTop {
 	# underscore and that that tabnames may NOT have a colon.
 	variable tabName_prefix		"Apol_"
 	variable components_tab 	"Apol_Components"
-    	variable rules_tab 		"Apol_Rules"
 	variable types_tab		"Apol_Types"
-	variable terules_tab		"Apol_TE"
-	variable roles_tab		"Apol_Roles"
-	variable rbac_tab		"Apol_RBAC"
 	variable class_perms_tab	"Apol_Class_Perms"
+	variable roles_tab		"Apol_Roles"
 	variable users_tab		"Apol_Users"
-	variable initial_sids_tab	"Apol_Initial_SIDS"
-	variable file_contexts_tab	"Apol_File_Contexts"
 	variable cond_bools_tab		"Apol_Cond_Bools"
+        variable mls_tab                "Apol_MLS"
+	variable initial_sids_tab	"Apol_Initial_SIDS"
+        variable net_contexts_tab	"Apol_NetContexts"
+        variable fs_contexts_tab	"Apol_FSContexts"
+
+    	variable rules_tab 		"Apol_Rules"
+	variable terules_tab		"Apol_TE"
 	variable cond_rules_tab		"Apol_Cond_Rules"
-	variable policy_conf_tab	"Apol_PolicyConf"
+	variable rbac_tab		"Apol_RBAC"
+	variable range_tab		"Apol_Range"
+
+	variable file_contexts_tab	"Apol_File_Contexts"
+
 	variable analysis_tab		"Apol_Analysis"
-	 
+
+        variable policy_conf_tab	"Apol_PolicyConf"
+
+        variable tab_names {
+            Types Class_Perms Roles Users Cond_Bools MLS Initial_SIDS NetContexts FSContexts
+            TE Cond_Rules RBAC Range
+            File_Contexts
+            Analysis
+            PolicyConf
+        }
 	variable tk_msgBox_Wait
 
 	# "contents" indicates which aspects of the policy are included in the current opened policy file
@@ -135,12 +152,19 @@ proc ApolTop::get_toplevel_dialog {} {
 }
 
 proc ApolTop::is_binary_policy {} {
-	if {$ApolTop::policy_type == $ApolTop::binary_policy_type} {
+	if {$ApolTop::policy_type == "binary"} {
 		return 1
 	}
 	return 0
 }
 
+proc ApolTop::is_mls_policy {} {
+    if {![is_policy_open] || $ApolTop::policy_mls_type == "mls"} {
+        return 1
+    }
+    return 0
+}
+
 proc ApolTop::load_fc_index_file {} {
 	set rt [Apol_File_Contexts::load_fc_db]
 	if {$rt == 1} {
@@ -412,12 +436,6 @@ proc ApolTop::set_Focus_to_Text { tab } {
 		$ApolTop::rules_tab {
 			ApolTop::set_Focus_to_Text [$rules_nb raise]
 		} \
-		$ApolTop::file_contexts_tab {
-			Apol_File_Contexts::set_Focus_to_Text
-		} \
-		$ApolTop::types_tab {
-			Apol_Types::set_Focus_to_Text
-		} \
 		$ApolTop::terules_tab {
 			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag normal 
 			set raisedPage [Apol_TE::get_results_raised_tab]
@@ -427,22 +445,6 @@ proc ApolTop::set_Focus_to_Text { tab } {
 				focus [$ApolTop::rules_nb getframe $ApolTop::terules_tab]
 			}
 		} \
-		$ApolTop::roles_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_Roles::set_Focus_to_Text
-		} \
-		$ApolTop::rbac_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_RBAC::set_Focus_to_Text
-		} \
-		$ApolTop::class_perms_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_Class_Perms::set_Focus_to_Text
-		} \
-		$ApolTop::users_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_Users::set_Focus_to_Text
-		} \
 		$ApolTop::analysis_tab {
 			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag normal
 			$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag disabled
@@ -451,27 +453,10 @@ proc ApolTop::set_Focus_to_Text { tab } {
 				Apol_Analysis::set_Focus_to_Text $raisedPage
 			} 
 		} \
-		$ApolTop::policy_conf_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_PolicyConf::set_Focus_to_Text
-		} \
-		$ApolTop::initial_sids_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_Initial_SIDS::set_Focus_to_Text
-		} \
-		$ApolTop::cond_bools_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_Cond_Bools::set_Focus_to_Text
-		} \
-		$ApolTop::cond_bools_tab {
-			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
-			Apol_Cond_Rules::set_Focus_to_Text
-		} \
-		default { 
-			return 
-		}
-	
-	return 0
+            default {
+                $ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
+                ${tab}::set_Focus_to_Text
+            }
 }
 
 ########################################################################
@@ -1213,8 +1198,8 @@ proc ApolTop::create { } {
 	#[$mainframe getmenu fc_index_menu] insert 0 command -label "Create Index" -command "ApolTop::create_fc_index_file"
 		
 	$mainframe addindicator -textvariable ApolTop::policyConf_lineno -width 14
-	$mainframe addindicator -textvariable ApolTop::polstats -width 88
-	$mainframe addindicator -textvariable ApolTop::polversion -width 19 
+	$mainframe addindicator -textvariable ApolTop::policy_stats_summary -width 88
+	$mainframe addindicator -textvariable ApolTop::policy_version_string -width 28
 	
 	# Disable menu items since a policy is not yet loaded.
 	$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag disabled
@@ -1241,20 +1226,28 @@ proc ApolTop::create { } {
 	# Create subordinate tab frames
 	set components_nb [NoteBook $components_frame.components_nb]
 	set rules_nb [NoteBook $rules_frame.rules_nb]
-	
+
+	variable mls_tabs
+    
 	# Subtabs for the main policy components tab.
 	Apol_Types::create $components_nb
 	Apol_Class_Perms::create $components_nb
 	Apol_Roles::create $components_nb
 	Apol_Users::create $components_nb
 	Apol_Cond_Bools::create $components_nb
+	Apol_MLS::create $components_nb
+	lappend mls_tabs [list $components_nb [$components_nb pages end]]
 	Apol_Initial_SIDS::create $components_nb
-	
+	Apol_NetContexts::create $components_nb
+	Apol_FSContexts::create $components_nb
+
 	# Subtabs for the main policy rules tab
 	Apol_TE::create $rules_nb
 	Apol_Cond_Rules::create $rules_nb
 	Apol_RBAC::create $rules_nb
-	
+	Apol_Range::create $rules_nb
+	lappend mls_tabs [list $rules_nb [$rules_nb pages end]]
+
 	$components_nb compute_size
 	pack $components_nb -fill both -expand yes -padx 4 -pady 4
 	$components_nb raise [$components_nb page 0]
@@ -1639,8 +1632,8 @@ proc ApolTop::helpDlg {title file_name} {
     set okButton [Button $hbox.okButton -text "Close" \
 		      -command "destroy $helpDlg"]
     # go to the script dir to find the help file
-    set script_dir  [apol_GetScriptDir "$file_name"]
-    set helpfile "$script_dir/$file_name"
+    set help_dir  [apol_GetHelpDir "$file_name"]
+    set helpfile "$help_dir/$file_name"
     
     # Placing display widgets
     pack $hbox -expand yes -fill both -padx 5 -pady 5
@@ -1688,157 +1681,143 @@ proc ApolTop::resetBusyCursor {} {
 }
 
 proc ApolTop::popupPolicyStats {} {
-	variable polversion
-	variable policy_type
-	variable contents
-	
-	set rt [catch {set pstats [apol_GetStats]}]
-	if {$rt != 0} {
-		tk_messageBox -icon error -type ok -title "Error" \
-			-message "No policy file currently opened"
-		return 
-	}
-	foreach item $pstats {
-		set rt [scan $item "%s %d" key val]
-		if {$rt != 2} {
-			tk_messageBox -icon error -type ok -title "Error" -message "apol_GetStats: $rt"
-			return
-		}
-		set stats($key) $val
-	}
-	
-	# Build the output based on what was collected in the policy
-	# (for now, only perms and classes are optionally collected (really a compile time option!)
-	if {$contents(classes) == 0} {
-		set classes "not collected"
-	} else {
-		set classes $stats(classes)
-	}
-	if {$contents(perms) == 0 } {
-		set perms "not collected"
-		set common_perms "not collected"
-	} else {
-		set common_perms $stats(common_perms)
-		set perms $stats(perms)
-	}
-	
-	set w .polstatsbox
-	catch {destroy $w}
-	toplevel $w
-	
-	label $w.1 -justify left \
-		-text "Policy Summary Statistics\n "
-	set labelf [frame $w.labelf]
-		
-	set left_text "\
-Policy Version:\n\
-Policy Type:\n\n\
-Number of Classes and Permissions\n\
-     \tObject Classes:\n\
-     \tCommon Perms:\n\
-     \tPermissions:\n\n\
-Number of Types and Attributes:\n\
-     \tTypes:\n\
-     \tAttributes:\n\n\
-Number of Type Enforcement Rules:\n\
-     \tallow:\n\
-     \tneverallow:\n\
-     \tclone (pre v.11):\n\
-     \ttype_transition.:\n\
-     \ttype_change:\n\
-     \ttype_member:\n\
-     \tauditallow:\n\
-     \tauditdeny:\n\
-     \tdontaudit:\n\n\
-Number of Roles:\n\
-     \tRoles:\n\n\
-Number of RBAC Rules:\n\
-     \tallow:\n\
-     \trole_transition:\n\n\
-Number of Users:\n\
-     \tusers:\n\n\
-Number of Initial SIDs:\n\
-     \tSIDs:\n\n\
-Number of Booleans:\n\
-     \tBools:\n"
-     
-     	set right_text "\
-$polversion\n\
-$policy_type\n\n\
-\n\
-$classes\n\
-$common_perms\n\
-$perms\n\n\
-\n\
-$stats(types)\n\
-$stats(attribs)\n\n\
-\n\
-$stats(teallow)\n\
-$stats(neverallow)\n\
-$stats(clone)\n\
-$stats(tetrans)\n\
-$stats(techange)\n\
-$stats(temember)\n\
-$stats(auditallow)\n\
-$stats(auditdeny)\n\
-$stats(dontaudit)\n\n\
-\n\
-$stats(roles)\n\n\
-\n\
-$stats(roleallow)\n\
-$stats(roletrans)\n\n\
-\n\
-$stats(users)\n\n\
-\n\
-$stats(sids)\n\n\
-\n\
-$stats(cond_bools)\n"
+    variable polstats
 
-	set left_label  [label $labelf.left -justify left -text $left_text]
-	set right_label [label $labelf.right -justify left -text $right_text]
-     	button $w.close -text Close -command "catch {destroy $w}" -width 10
+    set classes $polstats(classes)
+    set common_perms $polstats(common_perms)
+    set perms $polstats(perms)
+    if {![regexp -- {^([^\(]+) \(([^,]+), ([^\)]+)} $ApolTop::policy_version_string -> policy_version policy_type policy_mls_type]} {
+        set policy_version $ApolTop::policy_version_string
+        set policy_type "unknown"
+        set policy_mls_type "unknown"
+    }
+    set policy_version [string trim $policy_version]
+
+    destroy .polstatsbox
+    set dialog [Dialog .polstatsbox -separator 1 -title "Policy Summary" \
+                    -modal none -parent .]
+    $dialog add -text Close -command [list destroy $dialog]
+    
+    set w [$dialog getframe]
 	
-	pack $w.close -side bottom -anchor center 
-	pack $w.1 -side top -anchor center
-	pack $labelf -side top -anchor nw -fill both -expand yes -padx 5 -pady 5
-	pack $left_label $right_label -side left -anchor nw -fill both -expand yes
-	wm title $w "Policy Summary"
-	wm iconname $w "policy summary"
-	wm geometry $w +50+60
-    	return		
+    label $w.title -text "Policy Summary Statistics"
+    set f [frame $w.summary]
+    label $f.l -justify left -text "    Policy Version:\n    Policy Type:\n    MLS Status:"
+    label $f.r -justify left -text "$policy_version\n$policy_type\n$policy_mls_type"
+    grid $f.l $f.r -sticky w
+    grid configure $f.r -padx 30
+    grid $w.title - -sticky w -padx 8
+    grid $f - -sticky w -padx 8
+    grid [Separator $w.sep] - -sticky ew -pady 5
+    
+    set f [frame $w.left]
+    set i 0
+    foreach {title block} {
+        "Number of Classes and Permissions" {
+            "Object Classes" classes
+            "Common Perms" common_perms
+            "Permissions" perms
+        }
+        "Number of Types and Attributes" {
+            "Types" types
+            "Attributes" attribs
+        }
+        "Number of Type Enforcement Rules" {
+            "allow" teallow
+            "neverallow" neverallow
+            "clone (pre v.11)" clone
+            "type_transition" tetrans
+            "type_change" techange
+            "type_member" temember
+            "auditallow" auditallow
+            "auditdeny" auditdeny
+            "dontaudit" dontaudit
+        }
+        "Number of Roles" {
+            "Roles" roles
+        }
+        "Number of RBAC Rules" {
+            "allow" roleallow
+            "role_transition" roletrans
+        }
+    } {
+        set ltext "$title:"
+        set rtext {}
+        foreach {l r} $block {
+            append ltext "\n    $l:"
+            append rtext "\n$polstats($r)"
+        }
+        label $f.l$i -justify left -text $ltext
+        label $f.r$i -justify left -text $rtext
+        grid $f.l$i $f.r$i -sticky w -padx 4 -pady 2
+        incr i
+    }
+
+    set i 0
+    set g [frame $w.right]
+    foreach {title block} {
+        "Number of Users" {
+            "Users" users
+        }
+        "Number of Booleans" {
+            "Bools" cond_bools
+        }
+        "Number of MLS Components" {
+            "Sensitivities" sens
+            "Categories" cats
+        }
+        "Number of MLS Rules" {
+            "range_transition" rangetrans
+        }
+        "Number of Initial SIDs" {
+            "SIDs" sids
+        }
+        "Number of OContexts" {
+            "PortCons" portcons
+            "NetIfCons" netifcons
+            "NodeCons" nodecons
+            "GenFSCons" genfscons
+            "fs_use statements" fs_uses
+        }
+    } {
+        set ltext "$title:"
+        set rtext {}
+        foreach {l r} $block {
+            append ltext "\n    $l:"
+            append rtext "\n$polstats($r)"
+        }
+        label $g.l$i -justify left -text $ltext
+        label $g.r$i -justify left -text $rtext
+        grid $g.l$i $g.r$i -sticky w -padx 4 -pady 2
+        incr i
+    }
+    grid $f $g -sticky nw -padx 4
+    $dialog draw
 }
 
 proc ApolTop::showPolicyStats {} {
-	variable polstats 
-	variable contents
-	set rt [catch {set pstats [apol_GetStats]}]
-	if {$rt != 0} {
-		tk_messageBox -icon error -type ok -title \
-			-message "No policy file currently opened"
-		return 
-	}
-	foreach item $pstats {
-		set rt [scan $item "%s %d" key val]
-		if {$rt != 2} {
-			tk_messageBox -icon error -type ok -title "Error" -message "apol_GetStats: $rt"
-			return
-		}
-		set stats($key) $val
-	}
-	set polstats ""
-	if {$contents(classes) == 1} {
-		append polstats "Classes: $stats(classes)   "
-	}
-	if {$contents(perms) == 1} {
-		append polstats "Perms: $stats(perms)   "
-	}
-	append polstats "Types: $stats(types)   Attribs: $stats(attribs)   "
-	append polstats "TE rules: [expr $stats(teallow) + $stats(neverallow) + 	\
-		$stats(auditallow) + $stats(auditdeny) + $stats(clone)  +  $stats(dontaudit) +	\
-		$stats(tetrans) + $stats(temember) + $stats(techange)]   "
-	append polstats "Roles: $stats(roles)"
-	append polstats "   Users: $stats(users)"
-	return
+    variable polstats
+    variable policy_stats_summary
+    if {[catch {apol_GetStats} pstats]} {
+        tk_messageBox -icon error -type ok -title "Error" -message $pstats
+        return 
+    }
+    array unset polstats
+    array set polstats $pstats
+
+    set policy_stats_summary ""
+    append policy_stats_summary "Classes: $polstats(classes)   "
+    append policy_stats_summary "Perms: $polstats(perms)   "
+    append policy_stats_summary "Types: $polstats(types)   "
+    append policy_stats_summary "Attribs: $polstats(attribs)   "
+    set num_te_rules [expr {$polstats(teallow) + $polstats(neverallow) +
+                            $polstats(auditallow) + $polstats(auditdeny) +
+                            $polstats(clone) + $polstats(dontaudit) +
+                            $polstats(tetrans) + $polstats(temember) +
+                            $polstats(techange)}]
+    append policy_stats_summary "TE rules: $num_te_rules   "
+    append policy_stats_summary "Roles: $polstats(roles)   "
+    append policy_stats_summary "Users: $polstats(users)"
 }
 
 proc ApolTop::aboutBox {} {
@@ -1864,13 +1843,11 @@ proc ApolTop::unimplemented {} {
 proc ApolTop::closePolicy {} {
         variable contents
 	variable filename 
-	variable polstats
-	variable polversion
-	variable policy_is_open	
+	variable policy_version_string {}
+	variable policy_is_open
+	variable policy_stats_summary {}
 	
-	set polversion ""
 	set filename ""
-	set polstats ""
 	set contents(classes)	0
 	set contents(perms)	0
 	set contents(types)	0
@@ -1878,21 +1855,19 @@ proc ApolTop::closePolicy {} {
 	set contents(roles)	0
 	set contents(rbac)	0
 	set contents(users)	0
+	variable policy_mls_type ""
 	array unset contents
 	
 	wm title . "SE Linux Policy Analysis"
-	Apol_Perms_Map::close $ApolTop::mainframe 
-	Apol_Class_Perms::close
-	Apol_Types::close
-	Apol_TE::close
-	Apol_Roles::close
-        Apol_RBAC::close
-        Apol_Users::close
-        Apol_Initial_SIDS::close
-        Apol_Cond_Bools::close
-        Apol_Cond_Rules::close
-        Apol_Analysis::close 
-        Apol_PolicyConf::close    
+
+    variable tab_names
+    foreach tab $tab_names {
+        if {$tab == "Perms_Map"} {
+            Apol_Perms_Map::close $ApolTop::mainframe 
+        } else {
+            Apol_${tab}::close
+        }
+    }
         
 	ApolTop::set_Focus_to_Text [$ApolTop::notebook raise]
 	set rt [catch {apol_ClosePolicy} err]
@@ -1909,6 +1884,7 @@ proc ApolTop::closePolicy {} {
 	$ApolTop::mainframe setmenustate Disable_Summary disabled
 	ApolTop::enable_non_binary_tabs
 	ApolTop::enable_disable_conditional_widgets 1
+	set_mls_tabs_state normal
 	ApolTop::configure_edit_pmap_menu_item 0
 	#ApolTop::configure_load_index_menu_item 0
 	
@@ -1916,52 +1892,15 @@ proc ApolTop::closePolicy {} {
 }
 
 proc ApolTop::open_apol_modules {file} {
-	set rt [catch {Apol_Class_Perms::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Types::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}	
-	set rt [catch {Apol_TE::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Roles::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_RBAC::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Users::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Initial_SIDS::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Cond_Bools::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Cond_Rules::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_Analysis::open} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
-	set rt [catch {Apol_PolicyConf::open $file} err]
-	if {$rt != 0} {
-		return -code error $err
-	}
- 	return 0
- }
+    variable tab_names
+    foreach tab $tab_names {
+        if {$tab == "PolicyConf"} {
+            Apol_PolicyConf::open $file
+        } else {
+            Apol_${tab}::open
+        }
+    }
+}
  
 proc ApolTop::enable_disable_conditional_widgets {enable} {
 	set tab [$ApolTop::notebook raise] 
@@ -2017,6 +1956,22 @@ proc ApolTop::disable_non_binary_tabs {} {
 	return 0
 }
 
+# Enable/disable all of apol's tabs that deal exclusively with MLS
+# components.  If the currently raised page is one of those tabs then
+# raise the first page (which hopefully is not MLS specific).
+proc ApolTop::set_mls_tabs_state {new_state} {
+    variable mls_tabs
+
+    foreach tab $mls_tabs {
+        foreach {notebook page} $tab break
+        set current_tab [$notebook raise]
+        $notebook itemconfigure $page -state $new_state
+        if {$current_tab == $page && $new_state == "disabled"} {
+            $notebook raise [$notebook pages 0]
+        }
+    }
+}
+
 proc ApolTop::set_initial_open_policy_state {} {
 	set rt [catch {set version_num [apol_GetPolicyVersionNumber]} err]
 	if {$rt != 0} {
@@ -2049,8 +2004,11 @@ the names are not preserved in the binary policy format."
 			}
 		}
 		ApolTop::disable_non_binary_tabs
-   	}   	
-   	
+   	}
+	if {![is_mls_policy]} {
+		set_mls_tabs_state disabled
+	}
+
 	ApolTop::set_Focus_to_Text [$ApolTop::notebook raise]  
 	# Enable perm map menu items since a policy is now open.
 	$ApolTop::mainframe setmenustate Perm_Map_Tag normal
@@ -2066,8 +2024,9 @@ the names are not preserved in the binary policy format."
 # to 1 if a recently file is being opened with this proc
 proc ApolTop::openPolicyFile {file recent_flag} {
 	variable contents
-	variable polversion
+	variable policy_version_string
 	variable policy_type
+	variable policy_mls_type
 	variable policy_is_open	
 	variable filename
 	variable policy_open_option
@@ -2120,17 +2079,12 @@ proc ApolTop::openPolicyFile {file recent_flag} {
 		focus -force .
 		return -1
 	}
-	set rt [catch {set polversion [apol_GetPolicyVersionString]}]
-	if {$rt != 0} {
-		tk_messageBox -icon error -type ok -title "Error" -message "apol_GetPolicyVersionString: $rt"
+
+	if {[catch {apol_GetPolicyVersionString} policy_version_string]} {
+		tk_messageBox -icon error -type ok -title "Error" -message "apol_GetPolicyVersionString: $policy_version_string"
 		return 0
 	}
-	set rt [catch {set policy_type [apol_GetPolicyType]}]
-	if {$rt != 0} {
-		tk_messageBox -icon error -type ok -title "Error" -message "apol_GetPolicyType: $rt"
-		return 0
-	}
-	set polversion [append polversion " \($policy_type)"]
+	foreach {policy_type policy_mls_type} [apol_GetPolicyType] break;
 	# Set the contents flags to indicate what the opened policy contains
 	set rt [catch {set con [apol_GetPolicyContents]} err]
 	if {$rt != 0} {
@@ -2147,17 +2101,19 @@ proc ApolTop::openPolicyFile {file recent_flag} {
 	}
 	
 	ApolTop::showPolicyStats
+	set policy_is_open 1
 	set rt [catch {ApolTop::open_apol_modules $file} err]
  	if {$rt != 0} {
  		tk_messageBox -icon error -type ok -title "Error" -message "$err"
+        	set policy_is_open 0
 		return $rt	
  	}
  	set rt [catch {ApolTop::set_initial_open_policy_state} err]
 	if {$rt != 0} {
 		tk_messageBox -icon error -type ok -title "Error" -message "$err"
+        	set policy_is_open 0
  		return $rt
  	}
-	set policy_is_open 1
 	
 	if {$recent_flag == 1} {
 		ApolTop::addRecent $file
@@ -2173,7 +2129,6 @@ proc ApolTop::openPolicyFile {file recent_flag} {
 
 proc ApolTop::openPolicy {} {
 	variable filename 
-	variable polversion
 	
         set progressval 0
         set file ""
