@@ -1,7 +1,7 @@
 #############################################################
 #  types_relation_module.tcl  
 # -----------------------------------------------------------
-#  Copyright (C) 2004-2005 Tresys Technology, LLC
+#  Copyright (C) 2004-2006 Tresys Technology, LLC
 #  see file 'COPYING' for use and warranty information 
 #
 #  Requires tcl and tk 8.3+, with BWidgets
@@ -108,6 +108,10 @@ namespace eval Apol_Analysis_tra {
    	Apol_Analysis::register_analysis_modules "Apol_Analysis_tra" "Types Relationship Summary"	
 }
 
+proc Apol_Analysis_tra::get_short_name {} {
+    return "Types Relationship"
+}
+
 # ------------------------------------------------------------------------------
 #  Command Apol_Analysis_tra::display_dta_options
 # ------------------------------------------------------------------------------
