@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2005 Tresys Technology, LLC
+# Copyright (C) 2001-2006 Tresys Technology, LLC
 # see file 'COPYING' for use and warranty information 
 
 # TCL/TK GUI for SE Linux policy analysis
@@ -38,8 +38,6 @@ namespace eval Apol_Types {
 	variable progress_indicator	-1
 	
 	# Global Widgets
-	variable alistbox
-	variable tlistbox
 	variable resultsbox
 	variable types_select 
 	variable typeattribs 
@@ -53,12 +51,6 @@ namespace eval Apol_Types {
     	variable fc_incl_class
     	variable fc_files_select
     	variable progressDlg 		.progress_Dlg
-    	
-	# callback procedures for the listbox items menu. Each element in this list is an embedded list of 2 items.
-	# The 2 items consist of the command label and the function name. The tabname will be added as an
-	# argument to the callback procedure.
-	variable types_menu_callbacks	""
-    	variable attribs_menu_callbacks	""
 }
 
 proc Apol_Types::open { } {
@@ -91,12 +83,6 @@ proc Apol_Types::close { } {
 }
 
 proc Apol_Types::free_call_back_procs { } {
-       	variable types_menu_callbacks	
-    	variable attribs_menu_callbacks	
-	
-	set types_menu_callbacks ""
-	set attribs_menu_callbacks ""
-	return 0
 }
 
 proc Apol_Types::init_options { } {
@@ -141,12 +127,7 @@ proc Apol_Types::set_Focus_to_Text {} {
 	return 0
 }
 
-proc Apol_Types::on_show_more_info_button_clicked {which} {
-	if {$which == "type"} {
-		set lb $Apol_Types::tlistbox
-	} else {	
-		set lb $Apol_Types::alistbox
-	}
+proc Apol_Types::on_show_more_info_button_clicked {which lb} {
 	set sel [$lb curselection]
 	if {$sel != ""} {
 		Apol_Types::popupTypeInfo $which [$lb get $sel]
@@ -261,31 +242,6 @@ proc Apol_Types::popupTypeInfo {which ta} {
 	return 0
 }
 
-proc Apol_Types::destroy_progressDlg {} {
-	variable progressDlg
-	
-	if {[winfo exists $progressDlg]} {
-		destroy $progressDlg
-	}
-	ApolTop::resetBusyCursor
-     	return 0
-} 
-
-proc Apol_Types::display_progressDlg {} {
-     	variable progressDlg
-	    		
-	set Apol_Types::progressmsg "Searching...This may take a while."
-	set progressBar [ProgressDlg $Apol_Types::progressDlg \
-		-parent $ApolTop::mainframe \
-        	-textvariable Apol_Types::progressmsg \
-        	-variable Apol_Types::progress_indicator \
-        	-maximum 3 \
-        	-width 45]
-	ApolTop::setBusyCursor
-	update
-        return 0
-} 
-
 ##############################################################
 # ::search
 #  	- Search text widget for a string
@@ -300,29 +256,44 @@ proc Apol_Types::search { str case_Insensitive regExpr srch_Direction } {
 proc Apol_Types::searchTypes {} {
 	variable opts
 	variable srchstr
+	variable progressDlg
 	
+	if {[winfo exists $progressDlg]} {
+		# another search already going, so ignore this search request
+		return
+	}
 	if {$opts(usesrchstr) && $srchstr == ""} {
 		tk_messageBox -icon error -type ok -title "Error" -message "No regular expression provided!"
 		return
 	}
-	Apol_Types::display_progressDlg
+	if {$opts(types) == 0 && $opts(attribs) == 0} {
+		tk_messageBox -icon error -type ok -title "Error" -message "No search options provided!"
+		return
+	}
+    
+	set Apol_Types::progressmsg "Searching...This may take a while."
+	ProgressDlg $progressDlg \
+		-parent $ApolTop::mainframe \
+        	-textvariable Apol_Types::progressmsg \
+        	-variable Apol_Types::progress_indicator \
+        	-maximum 3 \
+        	-width 45
+	ApolTop::setBusyCursor
 	set rt [catch {set results [apol_GetTypeInfo $opts(types) $opts(typeattribs) \
 		$opts(attribs) $opts(attribtypes) $opts(attribtypeattribs) \
 		$opts(typealiases) $opts(usesrchstr) $srchstr \
-		$opts(show_files) $opts(incl_context) $opts(incl_class)]} err]	
+		$opts(show_files) $opts(incl_context) $opts(incl_class)]} err]
+	destroy $progressDlg
+	ApolTop::resetBusyCursor
 	if {$rt != 0} {	
-		Apol_Types::destroy_progressDlg
 		tk_messageBox -icon error -type ok -title "Error" \
 			-message "$err \n\nNote:If you need to load an index file, go to the File Context tab."
-		return 
 	} else {
 	    $Apol_Types::resultsbox configure -state normal
 	    $Apol_Types::resultsbox delete 0.0 end
 	    $Apol_Types::resultsbox insert end $results
 	    ApolTop::makeTextBoxReadOnly $Apol_Types::resultsbox
         }
-        Apol_Types::destroy_progressDlg
-	return 0
 }
 
 proc Apol_Types::_useSearch { entry } {
@@ -335,20 +306,20 @@ proc Apol_Types::_useSearch { entry } {
 }
 
 proc Apol_Types::enable_disable_checkbuttons { b1 b2 opt } {
-	switch $opt \
-		"1" {
+	switch -- $opt {
+		1 {
 			set status $Apol_Types::opts(types)
-		} \
-		"2" {
+		}
+		2 {
 			set status $Apol_Types::opts(attribs)
-		} \
-		"3" {
+		}
+		3 {
 			set status $Apol_Types::opts(show_files)
-		} \
+		}
 		default { 
 			puts "Invalid option for num argument: $num\n"
 		}
-        
+	}
 	if {$status} {
 	    $b1 configure -state normal
 	    $b2 configure -state normal
@@ -358,7 +329,20 @@ proc Apol_Types::enable_disable_checkbuttons { b1 b2 opt } {
 	    $b1 configure -state disabled
 	    $b2 configure -state disabled
 	}
-	return 0
+    variable sEntry
+    variable sString
+    # disable regular expression searching if both types and attribs
+    # are disabled
+    if {$Apol_Types::opts(types) == 0 && $Apol_Types::opts(attribs) == 0} {
+        set Apol_Types::opts(usesrchstr) 0
+        $sString deselect
+        $sString configure -state disabled
+        _useSearch $sEntry
+    } else {
+        variable sString
+        $sString configure -state normal
+        _useSearch $sEntry
+    }
 }
 
 proc Apol_Types::enable_disable_incl_attribs { cb } {
@@ -383,8 +367,6 @@ proc Apol_Types::goto_line { line_num } {
 }
 
 proc Apol_Types::create {nb} {
-    variable tlistbox 
-    variable alistbox 
     variable resultsbox 
     variable srchstr 
     variable types_select 
@@ -396,8 +378,6 @@ proc Apol_Types::create {nb} {
     variable sString 
     variable sEntry 
     variable opts
-    variable types_menu_callbacks
-    variable attribs_menu_callbacks
     variable fc_incl_context
     variable fc_incl_class
     variable fc_files_select
@@ -430,43 +410,20 @@ proc Apol_Types::create {nb} {
     pack $topf -fill both -expand yes 
     
     # Types listbox
-    set sw_t       [ScrolledWindow [$tbox getframe].sw -auto both]
-    set tlistbox [listbox [$sw_t getframe].lb -height 18 -width 20 -highlightthickness 0 \
-		      -listvar Apol_Types::typelist -bg white] 
-    $sw_t setwidget $tlistbox 
+    set tlistbox [Apol_Widget::makeScrolledListbox [$tbox getframe].types -height 18 -width 20 -listvar Apol_Types::typelist]
+    Apol_Widget::setListboxCallbacks $tlistbox \
+        {{"Show Type Info" {Apol_Types::popupTypeInfo type}}}
     set t_button [Button [$tbox getframe].t_button -text "Show Type Info" \
-    	-command {Apol_Types::on_show_more_info_button_clicked type} \
+                      -command [list Apol_Types::on_show_more_info_button_clicked type [Apol_Widget::getScrolledListbox $tlistbox]] \
     	-width 10]
-    # Attributes listbox 
-    set sw_a       [ScrolledWindow [$abox getframe].sw -auto both]
-    set alistbox [listbox [$sw_a getframe].lb -height 7 -width 20 -highlightthickness 0 \
-		      -listvar Apol_Types::attriblist -bg white]        
-    $sw_a setwidget $alistbox 
+
+    # Attributes listbox
+    set alistbox [Apol_Widget::makeScrolledListbox [$abox getframe].attribs -height 7 -width 20 -highlightthickness 0 -listvar Apol_Types::attriblist]
+    Apol_Widget::setListboxCallbacks $alistbox {{"Show Attribute Info" {Apol_Types::popupTypeInfo attrib}}}
     set a_button [Button [$abox getframe].a_button -text "Show Attribute Info" \
-    	-command {Apol_Types::on_show_more_info_button_clicked attrib} \
+                      -command [list Apol_Types::on_show_more_info_button_clicked attrib [Apol_Widget::getScrolledListbox $alistbox]] \
     	-width 10]
-    	
-    # Popup menu widget
-    menu .popupMenu_types
-    set types_menu_callbacks [lappend types_menu_callbacks {"Show Type Info" "Apol_Types::popupTypeInfo type"}]
-    menu .popupMenu_attribs
-    set attribs_menu_callbacks [lappend attribs_menu_callbacks {"Show Attribute Info" "Apol_Types::popupTypeInfo attrib"}]
-    
-    # Binding events to the both listboxes
-    bindtags $tlistbox [linsert [bindtags $tlistbox] 3 tlist_Tag]  
-    bindtags $alistbox [linsert [bindtags $alistbox] 3 alist_Tag]  
-    bind tlist_Tag <Double-ButtonPress-1>  { Apol_Types::popupTypeInfo "type" [$Apol_Types::tlistbox get active]}
-    bind alist_Tag <Double-ButtonPress-1> { Apol_Types::popupTypeInfo "attrib" [$Apol_Types::alistbox get active]}
-    bind tlist_Tag <Button-3> { ApolTop::popup_listbox_Menu \
-    	%W %x %y .popupMenu_types $Apol_Types::types_menu_callbacks \
-    	$Apol_Types::tlistbox}      
-    bind alist_Tag <Button-3> { ApolTop::popup_listbox_Menu \
-    	%W %x %y .popupMenu_attribs $Apol_Types::attribs_menu_callbacks \
-    	$Apol_Types::alistbox} 
-    
-    bind tlist_Tag <<ListboxSelect>> { focus -force $Apol_Types::tlistbox}
-    bind alist_Tag <<ListboxSelect>> { focus -force $Apol_Types::alistbox}
-     
+
     # Search options section      
     set ofm [$obox getframe]
     set fm_attribs_select [frame $ofm.ao -relief sunken -borderwidth 1]
@@ -488,15 +445,15 @@ proc Apol_Types::create {nb} {
     pack $fm_sString -side left -anchor n -fill both -expand yes -padx 5
     
     # Placing types and attributes listboxes frame
-    pack $sw_t -fill both -expand yes
-    pack $sw_a -fill both -expand yes
+    pack $tlistbox -fill both -expand yes
+    pack $alistbox -fill both -expand yes
 
     set typeattribs [checkbutton $fm_types_select.typeattribs -text "Include Attribs" \
 	-variable Apol_Types::opts(typeattribs) -padx 10] 
     set typealiases [checkbutton $fm_types_select.typealiases -text "Use Aliases" \
 	-variable Apol_Types::opts(typealiases) -padx 10]
     set types_select [checkbutton $fm_types_select.type -text "Show Types" -variable Apol_Types::opts(types) \
-	-command "Apol_Types::enable_disable_checkbuttons $typeattribs $typealiases 1"]
+	 -command [list Apol_Types::enable_disable_checkbuttons $typeattribs $typealiases 1]]
 
     # Attributes search section
     set a_typeattribs [checkbutton $fm_attribs_select.typeattribs -text "Include Type Attribs" \
@@ -507,7 +464,7 @@ proc Apol_Types::create {nb} {
 	-variable Apol_Types::opts(attribtypes) \
 	-padx 10 \
 	-offvalue 0 \
-	-command "Apol_Types::enable_disable_incl_attribs $a_typeattribs" \
+	-command [list Apol_Types::enable_disable_incl_attribs $a_typeattribs] \
 	-onvalue 1]
     set attribs_select [checkbutton $fm_attribs_select.type -text "Show Attributes" \
 	-variable Apol_Types::opts(attribs) \
@@ -527,7 +484,7 @@ proc Apol_Types::create {nb} {
     set sEntry [Entry $fm_sString.entry -textvariable Apol_Types::srchstr -width 40 \
 		    -helptext "Enter a regular expression string for which to search"]
     set sString [checkbutton $fm_sString.cb -variable Apol_Types::opts(usesrchstr) -text "Search Using Regular Expression" \
-		     -command "Apol_Types::_useSearch $sEntry"] 
+		     -command [list Apol_Types::_useSearch $sEntry]]
 
     # Action buttons
     button $okbox.ok -text OK -width 6 -command { Apol_Types::searchTypes }
