@@ -1,4 +1,4 @@
- /* Copyright (C) 2001-2005 Tresys Technology, LLC
+ /* Copyright (C) 2001-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /* 
@@ -12,7 +12,7 @@
 #include <stdio.h>
 #include <tcl.h>
 #include <tk.h>
-#include "../libapol/apol_tcl.h"
+#include "../libapol/apol_tcl_other.h"
 
 
 int Tcl_AppInit(Tcl_Interp *interp)
