@@ -1,4 +1,4 @@
-/* Copyright (C) 2004-2005 Tresys Technology, LLC
+/* Copyright (C) 2004-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
  
 /* 
@@ -969,8 +969,8 @@ static apol_diff_t *apol_get_pol_diffs(unsigned int opts, policy_t *p1, policy_t
 			else {
 				/* role i is in p2; make sure it has the same types assigned in p2 */
 				added = FALSE;
-				for(j = 0; j < p1->roles[i].num; j++) {
-					if (!ap_diff_is_type_in_p2role(p1->roles[i].a[j], idx2, p1, p2, renamed_types)) {
+				for(j = 0; j < p1->roles[i].num_types; j++) {
+					if (!ap_diff_is_type_in_p2role(p1->roles[i].types[j], idx2, p1, p2, renamed_types)) {
 						if(!added) {
 							/* add the role to the diff, and then note the first missing type */
 							added = TRUE;
@@ -981,7 +981,7 @@ static apol_diff_t *apol_get_pol_diffs(unsigned int opts, policy_t *p1, policy_t
 							}
 						}
 						/* note the missing type */
-						rt = add_i_to_a(p1->roles[i].a[j], &iad_node->numa, &iad_node->a);
+						rt = add_i_to_a(p1->roles[i].types[j], &iad_node->numa, &iad_node->a);
 						if(rt < 0)
 							goto err_return;
 					}
@@ -1004,8 +1004,8 @@ static apol_diff_t *apol_get_pol_diffs(unsigned int opts, policy_t *p1, policy_t
 			else {
 				/* user i is in p2; make sure it has the same roles assigned in p2 */
 				added = FALSE;
-				for(j = 0; j < p1->users[i].num; j++) {
-					rt = get_role_name(p1->users[i].a[j], &name, p1);
+				for(j = 0; j < p1->users[i].num_roles; j++) {
+					rt = get_role_name(p1->users[i].roles[j], &name, p1);
 					if(rt < 0)
 						goto err_return;
 					if(!is_role_in_user(name, idx2, p2)) {
@@ -1021,7 +1021,7 @@ static apol_diff_t *apol_get_pol_diffs(unsigned int opts, policy_t *p1, policy_t
 							}
 						}
 						/* note the missing role */
-						rt = add_i_to_a(p1->users[i].a[j], &iad_node->numa, &iad_node->a);
+						rt = add_i_to_a(p1->users[i].roles[j], &iad_node->numa, &iad_node->a);
 						if(rt < 0) {
 							free(name);
 							goto err_return;
@@ -2682,6 +2682,74 @@ void ap_single_view_diff_sort_te_rules(ap_single_view_diff_t *svd, int sort_col,
 	}
 }
 
+/* return the total number of differences in the single view diff */
+int ap_single_view_diff_get_num_diffs(ap_single_view_diff_t *svd)
+{
+	int total = 0;
+
+	if (!svd)
+		return -1;
+
+	/* types */
+	if (svd->types)
+		total += svd->types->num_add + svd->types->num_rem + svd->types->num_chg +
+			svd->types->num_chg_add + svd->types->num_chg_rem;
+	/* roles */
+	if (svd->roles)
+		total += svd->roles->num_add + svd->roles->num_rem + svd->roles->num_chg +
+			svd->roles->num_chg_add + svd->roles->num_chg_rem;
+	/* users */
+	if (svd->users)
+		total += svd->users->num_add + svd->users->num_rem + svd->users->num_chg +
+			svd->users->num_chg_add + svd->users->num_chg_rem;
+	/* attributes */
+	if (svd->attribs)
+		total += svd->attribs->num_add + svd->attribs->num_rem + svd->attribs->num_chg +
+			svd->attribs->num_chg_add + svd->attribs->num_chg_rem;
+	/* classes */
+	if (svd->classes)
+		total += svd->classes->num_add + svd->classes->num_rem + svd->classes->num_chg +
+			svd->classes->num_chg_add + svd->classes->num_chg_rem;
+	/* common perms */
+	if (svd->common_perms)
+		total += svd->common_perms->num_add + svd->common_perms->num_rem + svd->common_perms->num_chg +
+			svd->common_perms->num_chg_add + svd->common_perms->num_chg_rem;
+	/* role allows */
+	if (svd->rallows)
+		total += svd->rallows->num_add + svd->rallows->num_rem + svd->rallows->num_chg +
+			svd->rallows->num_chg_add + svd->rallows->num_chg_rem;
+	/* bools */
+	if (svd->bools)
+		total += svd->bools->num_add + svd->bools->num_rem + svd->bools->num_chg;
+	/* role trans */
+	if (svd->rtrans)
+		total += svd->rtrans->num_add + svd->rtrans->num_rem + svd->rtrans->num_chg + 
+			svd->rtrans->num_add_type + svd->rtrans->num_rem_type;
+	/* perms */
+	if (svd->perms)
+		total += svd->perms->num_add + svd->perms->num_rem;
+	/* te */
+	if (svd->te)
+		total += svd->te->num_add + svd->te->num_rem + svd->te->num_chg + svd->te->num_add_type +
+			svd->te->num_rem_type;
+	/* conds */
+	if (svd->conds)
+		total += svd->conds->num_add + svd->conds->num_rem + svd->conds->num_chg;
+
+#ifdef DEBUG
+	/* NOTE: this is not a perfect check, but we need to make sure this function gets updated 
+	 * as we provide more difference capabilities in the future */
+	if (sizeof(ap_single_view_diff_t) != sizeof(svd->types) + sizeof(svd->roles) + sizeof(svd->users) + sizeof(svd->attribs) + 
+	    sizeof(svd->classes) + sizeof(svd->perms) + sizeof(svd->common_perms) + sizeof(svd->rallows) + sizeof(svd->bools) + 
+	    sizeof(svd->rtrans) + sizeof(svd->te) + sizeof(svd->conds) + sizeof(svd->diff)) {
+		assert(FALSE);
+		return -1;
+	}
+#endif
+
+	return total;
+}
+
 /* opts are policy open options (see policy.h).  They indicate to apol_get_pol_diffs()
  * what parts of the policy to differntiate.  Policies p1 and p2 must be opened with
  * at least the same options.  If unsure you can always use POLOPT_ALL (and ensure
