@@ -401,9 +401,58 @@ int close_policy(policy_t *policy)
 	return free_policy(&policy);
 }
 
-static int read_policy(policy_t *policy)
+/* external functions from the parser to handle start state changes 
+ * for each pass (initial for passes 1 &2 and optonly for all other passes) */
+extern int yybegin_optonly(void);
+extern int yybegin_initial(void);
+
+static int resolve_optionals(policy_t *policy)
+{
+	int changed = 0;
+	ap_optional_t *opt = NULL;
+
+	for (opt=policy->optionals; opt; opt = opt->next) {
+		if (opt->status != OPTIONAL_STATUS_UNDECIDED)
+			continue;
+		if(ap_optional_check_requires(opt, policy) > 0) {
+			changed = 1;
+			opt->status = OPTIONAL_STATUS_TAKE_MAIN;
+		}
+	}
+
+	return changed;
+}
+
+static void take_else(policy_t *policy)
+{
+	ap_optional_t *opt = NULL;
+
+	for (opt = policy->optionals; opt; opt = opt->next)
+		if (opt->status == OPTIONAL_STATUS_UNDECIDED)
+			opt->status = OPTIONAL_STATUS_TAKE_ELSE;
+}
+
+static int reparse(policy_t *policy)
 {
 	int rt;
+	policydb_lineno = 1;
+		pass++;
+		rewind(yyin);
+		yyrestart(yyin);	
+		if ((rt = yyparse())) {
+			fprintf(stderr, "error(s) encountered while parsing configuration (fourth+ pass, line: %d)\n", policydb_lineno);
+			queue_destroy(id_queue);
+			rewind(yyin);
+			yyrestart(yyin);	
+			return rt;
+		}
+	return 0;
+}
+
+static int read_policy(policy_t *policy)
+{
+	int rt, i, changed = 0;
+	ap_mls_level_t *lvl = NULL;
 	
 	policy->policy_type = POL_TYPE_SOURCE;
 	/*yydebug = 1; */
@@ -417,6 +466,7 @@ static int read_policy(policy_t *policy)
 	}
 	policydb_lineno = 1;
 	pass = 1;
+	yybegin_initial();
 	if ((rt = yyparse())) {
 		fprintf(stderr, "error(s) encountered while parsing configuration (first pass, line: %d)\n", policydb_lineno);
 		queue_destroy(id_queue);
@@ -442,7 +492,31 @@ static int read_policy(policy_t *policy)
 		yyrestart(yyin);	
 		return rt;
 	}
-		
+
+	if(policy->has_optionals) {
+		policydb_lineno = 1;
+		pass = 3;
+		rewind(yyin);
+		yyrestart(yyin);	
+		yybegin_optonly();
+		if ((rt = yyparse())) {
+			fprintf(stderr, "error(s) encountered while parsing configuration (third pass, line: %d)\n", policydb_lineno);
+			queue_destroy(id_queue);
+			rewind(yyin);
+			yyrestart(yyin);	
+			return rt;
+		}
+
+		/* Pass 4+ */
+		do {
+			changed = resolve_optionals(policy);
+			if((rt=reparse(policy)))
+				return rt;
+		} while (changed);
+		take_else(policy);
+		reparse(policy);
+	}
+
 	queue_destroy(id_queue);
 	/* Kludge; now check for policy version 18 but special permission defined (i.e., if
 	 * nlmsg_write or nlmsg_write are defined as permissions, than the version is at least
@@ -456,6 +530,21 @@ static int read_policy(policy_t *policy)
 			return -1;
 		}
 	}
+
+	/* ensure all sensitivities have a level if not
+	 * do what checkpolicy does: create one with an
+	 * empty category list */
+	if (is_mls_policy(policy)) {
+		for (i = 0; i < policy->num_sensitivities; i++) {
+			if (!(lvl = ap_mls_sensitivity_get_level(i, policy))) {
+				if (add_mls_level(i, NULL, 0, policy)) {
+					fprintf(stderr, "error adding implicit level for sensitivity %s\n", policy->sensitivities[i].name);
+					return -1;
+				}
+			}
+		}
+	}
+
 	return 0;		
 }
 
@@ -477,13 +566,13 @@ unsigned int validate_policy_options(unsigned int options)
 	if(POLOPT_ROLE_RULES & opts)
 		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_CLASSES);
 	if(POLOPT_USERS & opts)
-		opts |= POLOPT_ROLES;
+		opts |= (POLOPT_ROLES|POLOPT_MLS_COMP);
 	if(POLOPT_ROLES & opts)
 		opts |= POLOPT_TYPES;
 	if(POLOPT_INITIAL_SIDS & opts)
-		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_USERS);
+		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_USERS|POLOPT_MLS_COMP);
 	if(POLOPT_OCONTEXT & opts)
-		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_USERS);
+		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_USERS|POLOPT_MLS_COMP);
 	if(POLOPT_OBJECTS & opts)
 		opts |= POLOPT_OBJECTS;
 	if(POLOPT_CONSTRAIN & opts)
@@ -503,8 +592,12 @@ int open_partial_policy(const char* filename, unsigned int options, policy_t **p
 {
 	int rt;
 	unsigned int opts;
-	
-	opts = validate_policy_options(options);
+	struct stat buf;
+
+	/* To support optionals and the transition to 3.0 disable opening 
+	 * only part of a policy .*/	
+	/*	opts = validate_policy_options(options);*/
+	opts = POLOPT_ALL;
 	
 	if(policy == NULL)
 		return -1;
@@ -515,6 +608,17 @@ int open_partial_policy(const char* filename, unsigned int options, policy_t **p
 		return -1;
 	}
 	(*policy)->opts = opts;
+	rt = stat(filename, &buf);
+	if (rt < 0) {
+		fprintf(stderr, "Could not open policy %s!\n",filename);
+		return -1;
+	} else {
+		/* ensure this is a regular file ie. not a directory, blk_file etc. */
+		if (!S_ISREG(buf.st_mode)) {
+			fprintf(stderr, "Could not open policy %s, not a regular file!\n", filename);
+			return -1;
+		}
+	}
 	yyin = fopen(filename, "r");
 	if (yyin == NULL) {
 		fprintf(stderr, "Could not open policy %s!\n", filename);
