@@ -163,23 +163,35 @@ typedef struct obj_class {
 	ta_item_t	*validatetrans;	/* list of validatetrans for this class */
 } obj_class_t;
 
+typedef struct ap_mls_level {
+	int sensitivity;
+	int *categories;
+	int num_categories;
+} ap_mls_level_t;
+
+typedef struct ap_mls_range {
+	ap_mls_level_t *low;
+	ap_mls_level_t *high;
+} ap_mls_range_t;
 
 typedef struct security_context {
 	int		user;	
 	int		role;
 	int		type;
+	ap_mls_range_t  *range;
 } security_con_t;
 
 /* file type IDs */
-#define FILETYPE_DIR  1 /* Directory */
-#define FILETYPE_CHR  2 /* Character device */
-#define FILETYPE_BLK  3 /* Block device */
-#define FILETYPE_REG  4 /* Regular file */
-#define FILETYPE_FIFO 5 /* FIFO */
-#define FILETYPE_LNK  6 /* Symbolic link */
-#define FILETYPE_SOCK 7 /* Socket */
-#define FILETYPE_ANY  8 /* any type */
-#define FILETYPE_NONE 0 /* none */
+#define FILETYPE_NONE 0  /* none */
+/* the following values must correspond to libsepol flask.h */
+#define FILETYPE_REG  6  /* Regular file */
+#define FILETYPE_DIR  7  /* Directory */
+#define FILETYPE_LNK  9  /* Symbolic link */
+#define FILETYPE_CHR  10 /* Character device */
+#define FILETYPE_BLK  11 /* Block device */
+#define FILETYPE_SOCK 12 /* Socket */
+#define FILETYPE_FIFO 13 /* FIFO */
+#define FILETYPE_ANY  14 /* any type */
 
 /* IDs of rules */
 #define RULE_TE_ALLOW		0 	/*AV rule */
@@ -187,16 +199,16 @@ typedef struct security_context {
 #define RULE_AUDITDENY		2	/*AV rule */
 #define RULE_DONTAUDIT		3	/* really the same as auditdeny */
 #define RULE_NEVERALLOW		4	/*AV rule */
-#define RULE_MAX_AV			4   	/*end of AV rule types */
+#define RULE_MAX_AV		4   	/*end of AV rule types */
 #define RULE_TE_TRANS		5   	/*TT rule (type transition|change|member) */
 #define RULE_TE_MEMBER		6   	/*TT rule */
 #define RULE_TE_CHANGE		7	/*TT rule */
-#define RULE_MAX_TE			7
-#define RULE_CLONE			8	/*clone rule */
+#define RULE_MAX_TE		7
+#define RULE_CLONE		8	/*clone rule */
 #define RULE_ROLE_ALLOW		9	/* Role allow */
 #define RULE_ROLE_TRANS		10	/* Role transition */
-#define RULE_USER			11	/* User role definition */
-#define RULE_MAX			12	/* # of rule IDs defined (1+last rule)*/
+#define RULE_USER		11	/* User role definition */
+#define RULE_MAX		12	/* # of rule IDs defined (1+last rule)*/
 #define RULE_INVALID		99
 
 
@@ -239,6 +251,14 @@ typedef struct tt_item {
 	ta_item_t	dflt_type;	/* the default type; only one of these */	
 } tt_item_t;
 
+typedef struct ap_role {
+	char *name;
+	int *types;
+	int num_types;
+	int *dom_roles;
+	int num_dom_roles;
+} ap_role_t;
+
 /* a role allow rule; dynamic arrays */
 typedef struct role_allow_item {
 	unsigned char	flags;		/* use AV* flags above, only need SRC & TGT */	
@@ -381,17 +401,6 @@ typedef struct ap_mls_sens {
 /* typedef for clarity */
 typedef struct ap_mls_sens ap_mls_cat_t;
 
-typedef struct ap_mls_level {
-	int sensitivity;
-	int *categories;
-	int num_categories;
-} ap_mls_level_t;
-
-typedef struct ap_mls_range {
-	ap_mls_level_t *low;
-	ap_mls_level_t *high;
-} ap_mls_range_t;
-
 typedef struct ap_rangetrans {
 	unsigned long lineno;
 	unsigned char flags;
@@ -400,6 +409,38 @@ typedef struct ap_rangetrans {
 	ap_mls_range_t *range;
 } ap_rangetrans_t;
 
+typedef struct ap_user {
+	char *name;
+	int *roles;
+	int num_roles;
+	ap_mls_level_t *dflt_level;
+	ap_mls_range_t *range;
+} ap_user_t;
+
+typedef struct ap_require {
+	char *name;
+	unsigned int type; /* use IDX_* defines */
+	int idx;
+	struct ap_require *perms; /* used for class requirement to include permissions */
+	struct ap_require *next;
+} ap_require_t;
+
+#define OPTIONAL_STATUS_UNDECIDED 1
+#define OPTIONAL_STATUS_TAKE_MAIN 2
+#define OPTIONAL_STATUS_TAKE_ELSE 4
+#define OPTIONAL_STATUS_FINISHED  8
+typedef struct ap_optional {
+	ap_require_t *requires;
+	name_item_t *types;
+	name_item_t *attribs;
+	name_item_t *roles;
+	name_item_t *users;
+	name_item_t *bools;
+	bool_t status;
+	unsigned long lineno;
+	struct ap_optional *next;
+} ap_optional_t;
+
 /* IDs for DYNAMIC array only  (e.g., clones is not a dynamic array, but rather a linked list*/
 #define POL_LIST_TYPE			0
 #define POL_LIST_ATTRIB			1
@@ -450,13 +491,12 @@ typedef struct ap_rangetrans {
 #define POL_VER_COND		4	/* same */
 #define POL_VER_17		5
 #define POL_VER_18		6	/* for v 18 binary */
-#define POL_VER_18_20		7	/* for source w/o MLS */
-#define POL_VER_19		8	/* for binary w/o MLS */
-#define POL_VER_19MLS		9
-#define POL_VER_MLS		9	/* minimum policy version for MLS support */
+#define POL_VER_18_20		7	/* for source v >=18 w/o MLS */
+#define POL_VER_19		8	
+#define POL_VER_MLS		8	/* minimum policy version for MLS support */
+#define POL_VER_19_20		9	/* for source v >= 19 w/ MLS */
 #define POL_VER_20		10
-#define POL_VER_20MLS		11
-#define	POL_VER_MAX		11
+#define	POL_VER_MAX		10
 
 /* Policy type macros */
 #define POL_TYPE_UNKNOWN 	0x00000000
@@ -467,6 +507,8 @@ typedef struct ap_rangetrans {
 /* This is an actual policy data base */
 typedef struct policy {
 	int	version;		/* weak indicator of policy version, see comments above */
+	bool_t mls;		/* indicates mls policy */
+	bool_t has_optionals; /* indicates that optional policy components are loaded */
 	unsigned int opts;		/* indicates which parts of the policy are included in this policy */
 	unsigned int policy_type;	/* policy type (binary or source) */
 	int	num_types;		/* array current ptr */
@@ -526,11 +568,11 @@ typedef struct policy {
 	cond_bool_t	*cond_bools;	/* conditional policy booleans (ARRAY) */
 	cond_expr_item_t *cond_exprs;	/* conditional expressions (ARRAY) */
 /* Role-based access control rules */
-	name_a_t	*roles;		/* roles (ARRAY)*/
+	ap_role_t	*roles;		/* roles (ARRAY)*/
 	role_allow_t	*role_allow;	/* role allow rules (ARRAY) */
 	rt_item_t	*role_trans;	/* role transition rules (ARRAY) */
 /* User rules */
-	name_a_t	*users;		/* users (ARRAY) */
+	ap_user_t	*users;		/* users (ARRAY) */
 /* Constraints */
 	ap_constraint_t		*constraints;	/* constraints (ARRAY) */
 	ap_validatetrans_t	*validatetrans;	/* validatetrans (ARRAY) */
@@ -540,6 +582,8 @@ typedef struct policy {
 	int		*mls_dominance;	/* MLS sensitivity dominance size num_sensitivities*/
 	ap_mls_level_t	*levels;	/* MLS levels (ARRAY) */
 	ap_rangetrans_t	*rangetrans;	/* MLS rangetrans rules (ARRAY) */
+/* Linked list of Optionals */
+	ap_optional_t *optionals; /* used during parsing */
 /* Permissions map (which is used for information flow analysis) */
 	struct classes_perm_map *pmap;	/* see perm-map.h */
 /* Semantic hash table; this structure is created when necessary (NULL ptr means its necessary)
@@ -577,10 +621,12 @@ int init_policy( policy_t **policy_ptr);
 int free_policy(policy_t **policy_ptr);
 int set_policy_version(int ver, policy_t *policy);
 const char* get_policy_version_name(int policy_version);
+char *get_policy_version_type_mls_str(policy_t *policy);
 
 #define is_binary_policy(policy) (policy != NULL ? (policy->policy_type & POL_TYPE_BINARY) : 0)
 #define is_valid_policy_version(version) (version >= POL_VER_UNKNOWN && version <= POL_VER_MAX)
 #define get_policy_version_id(policy) (policy != NULL ? policy->version : -1)
+#define is_mls_policy(policy) ((policy != NULL && policy->mls)?1:0)
 
 /* DB updates/additions/changes */
 #define add_common_perm_to_class(cls_idx, cp_idx, policy) ((is_valid_obj_class_idx(cls_idx, policy) && is_valid_common_perm_idx(cp_idx, policy)) ? policy->obj_classes[cls_idx].common_perms = cp_idx: -1 )
@@ -590,6 +636,7 @@ int add_alias(int type_idx, const char *alias, policy_t *policy);
 int add_attrib_to_type(int type_idx, char *token, policy_t *policy);
 int insert_ta_item(ta_item_t *newitem, ta_item_t **list);
 int add_name(char *name, name_item_t **list);
+int is_name_in_name_item_list(char *name, name_item_t *list);
 int free_name_list(name_item_t *list);
 int add_clone_rule(int src, int tgt,  unsigned long lineno, policy_t *policy);
 int add_attrib(bool_t with_type, int type_idx, policy_t *policy, char *attrib);
@@ -610,7 +657,9 @@ int add_path_to_genfscon(ap_genfscon_t *genfscon, char *path, int filetype, secu
 int add_constraint(bool_t is_mls, ta_item_t *classes, ta_item_t *perms, ap_constraint_expr_t *expr, unsigned long lineno, policy_t *policy);
 int add_validatetrans(bool_t is_mls, ta_item_t *classes, ap_constraint_expr_t *expr, unsigned long lineno, policy_t *policy);
 int add_sensitivity(char *name, name_item_t *aliases, policy_t *policy);
-int add_category(char *name, name_item_t *aliases, policy_t *policy);
+int add_sensitivity_alias(int sens, char *alias, policy_t *policy);
+int add_category(char *name, int idx, name_item_t *aliases, policy_t *policy);
+int add_category_alias(int cat, char *alias, policy_t *policy);
 int add_mls_level(int sens, int *cats, int num_cats, policy_t *policy);
 av_item_t *add_new_av_rule(int rule_type, policy_t *policy);
 tt_item_t *add_new_tt_rule(int rule_type, policy_t *policy);
@@ -771,27 +820,91 @@ int apol_add_perm_to_obj_perm_set_list(obj_perm_set_t **obj_options,
 				      int *num_obj_options, int obj_class, 
 				      int perm);
 
+/* contexts */
+void security_con_destroy(security_con_t *context);
+bool_t validate_security_context(const security_con_t *context, policy_t *policy);
+bool_t match_security_context(security_con_t *context1, security_con_t *context2, unsigned char range_match, policy_t *policy);
+
+
 /* genfscon */
 int ap_genfscon_get_idx(char *fstype, policy_t *policy);
 void ap_genfscon_node_destroy(ap_genfscon_node_t *node);
+int ap_genfscon_get_num_paths(policy_t *policy);
 
 /* MLS */
 int get_sensitivity_idx(const char *name, policy_t *policy);
 int get_category_idx(const char *name, policy_t *policy);
 void ap_mls_level_free(ap_mls_level_t *lvl);
 void ap_mls_range_free(ap_mls_range_t *rng);
+bool_t ap_mls_does_level_use_category(ap_mls_level_t *level, int cat);
+bool_t ap_mls_does_range_include_level(ap_mls_range_t *range, ap_mls_level_t *level, policy_t *policy);
+bool_t ap_mls_does_range_contain_subrange(ap_mls_range_t *range, ap_mls_range_t *subrange, policy_t *policy);
+int ap_mls_get_sens_dom_val(int sensitivity, policy_t *policy);
+bool_t ap_mls_validate_level(ap_mls_level_t *level, policy_t *policy);
+bool_t ap_mls_validate_range(ap_mls_range_t *range, policy_t *policy);
+
+/* the level compare function will return one of the following on success or -1 on error */
+#define AP_MLS_EQ 0
+#define AP_MLS_DOM 1
+#define AP_MLS_DOMBY 2
+#define AP_MLS_INCOMP 3
+int ap_mls_level_compare(ap_mls_level_t *l1, ap_mls_level_t *l2, policy_t *policy);
+ap_mls_level_t * ap_mls_sensitivity_get_level(int sens, policy_t *policy);
+int ap_mls_sens_get_level_cats(int sens, int **cats, int *num_cats, policy_t *policy);
+int ap_mls_category_get_sens(int cat, int **sens, int *num_sens, policy_t *policy);
+
+/* Range Trnsition Search*/
+/* return value >=0 num resutls < 0 for error (-arg idx on EINVAL -1 otherwise) */
+/* the search type flg*/
+/* types may be one of the following src, tgt, src|tgt, any (use src array) */
+#define AP_MLS_RTS_SRC_TYPE 0x01
+#define AP_MLS_RTS_TGT_TYPE 0x02
+#define AP_MLS_RTS_ANY_TYPE 0x04
+/* range matching mode */
+#define AP_MLS_RTS_RNG_SUB   0x08 /* query range is subset of rule range */
+#define AP_MLS_RTS_RNG_SUPER 0x10 /* query range is superset of rule range */
+#define AP_MLS_RTS_RNG_EXACT (AP_MLS_RTS_RNG_SUB|AP_MLS_RTS_RNG_SUPER)
+/* how to treat searches on multiple fields 
+ * by default matches criteria in all fields
+ * if set returns matches any one field */
+#define AP_MLS_RTS_MATCH_ANY 0x20
+int ap_mls_range_transition_search(int *srcs, int num_srcs, int src_type,
+	int *tgts, int num_tgts, int tgt_type, ap_mls_range_t *range, 
+	unsigned char search_type, int **rules, policy_t *policy);
 
 /* constraints */
 void ap_constraint_expr_destroy(ap_constraint_expr_t *expr);
 
+/* users */
+void ap_user_free(ap_user_t *user);
+
 /* misc */
 
 bool_t does_clone_rule_use_type(int idx, int type, unsigned char whichlist, cln_item_t *rule,
 	int *cnt, policy_t *policy);
 int get_rule_lineno(int rule_idx, int rule_type, policy_t *policy);
 int get_ta_item_name(ta_item_t *ta, char **name, policy_t *policy);
+int extract_types_from_ta_list(ta_item_t *list, bool_t compliment, bool_t allow_self, int **types, int *num_types, policy_t *policy);
+int extract_types_and_attribs_from_ta_list(ta_item_t *list, bool_t compliment, 
+		bool_t allow_self, int **types, int *num_types, int **attribs, 
+		int *num_attribs, policy_t *policy);
 int free_ta_list(ta_item_t *list);
 
+/* optionals */
+int ap_optional_init(ap_optional_t *op);
+/* free all memory used by the list optional (does not free the optional or others in the list */
+void ap_optional_free(ap_optional_t *op);
+/* free the linked list of optionals */
+void ap_optional_list_destroy(ap_optional_t **list);
+/* returns true if all requirements are met for an optional else returns 0 */
+int ap_optional_check_requires(ap_optional_t *op, policy_t *policy);
+ap_optional_t *ap_optional_get_by_lineno(unsigned long lineno, policy_t *policy);
+/* require statements */
+int ap_require_init(ap_require_t *req);
+void ap_require_destroy (ap_require_t **req);
+/* returns 1 if requirement met 0 if not or < 0 on error */
+int ap_require_check(ap_require_t *req, ap_optional_t *opt, policy_t *policy);
+
 /**************/
 /* these are INTERNAL functions only; allow direct access to type/attrib name string
  * stored within the policy db.  They are exported only for use by other internal
