@@ -19,6 +19,7 @@
 #include "policy.h"
 #include "semantic/avhash.h"
 #include "semantic/avsemantics.h"
+#include "render.h"
 
 #include <assert.h>
 #include <stdio.h>
@@ -371,13 +372,69 @@ char *re_render_tt_rule(bool_t addlineno, int idx, policy_t *policy)
 
 }
 
-/* security contexts */
-char *re_render_security_context(const security_con_t *context,
-				 policy_t *policy
-				 )
+char *re_render_mls_level(ap_mls_level_t *level, policy_t *policy)
 {
-	char *buf, *name;
+	char *rt = NULL;
+	int sz = 0, i, cur;
+
+	if (!level || !policy)
+		return NULL;
+
+	append_str(&rt, &sz, policy->sensitivities[level->sensitivity].name);
+	if (!level->categories)
+		return rt; /* no categories, simply return the sensitivity name */
+
+	append_str(&rt, &sz, ":");
+	append_str(&rt, &sz, policy->categories[level->categories[0]].name);
+	if (level->num_categories == 1)
+		return rt; /* only one category so done */
+
+	cur = 0; /* current value to compare with cat[i] */
+	for (i = 1; i < level->num_categories; i++) {
+		if (level->categories[i] == level->categories[cur] + 1) {
+			if (i + 1 == level->num_categories || level->categories[i+1] != level->categories[cur] + 2) {
+				append_str(&rt, &sz, ".");
+				append_str(&rt, &sz, policy->categories[level->categories[i]].name);
+				cur = i;
+			} else {
+				cur++;
+			}
+		} else {
+			append_str(&rt, &sz, ", ");
+			append_str(&rt, &sz, policy->categories[level->categories[i]].name);
+			cur = i;
+		}
+	}
+	return rt;
+}
+
+char *re_render_mls_range(ap_mls_range_t *range, policy_t *policy)
+{
+	char *rt = NULL;
+	char *sub_str = NULL;
+	int sz = 0;
+
+	if (!range || !policy)
+		return NULL;
+
+	sub_str = re_render_mls_level(range->low, policy);
+	append_str(&rt, &sz, sub_str);
+	free(sub_str);
+	if (range->high != range->low) {
+		append_str(&rt, &sz, " - ");
+		sub_str = re_render_mls_level(range->high, policy);
+		append_str(&rt, &sz, sub_str);
+		free(sub_str);
+	}
+	return rt;
+}
+
+/* security contexts */
+char *re_render_security_context(const security_con_t *context, policy_t *policy)
+{
+	char *buf = NULL, *name = NULL, *range = NULL;
 	int buf_sz;
+
 	if(policy == NULL )
 		return NULL;
 	
@@ -401,7 +458,8 @@ char *re_render_security_context(const security_con_t *context,
 		goto err_return;
 	if(append_str(&buf, &buf_sz, name) != 0) 
 		goto err_return;
-	free(name);
+	free(name); 
+	name = NULL;
 	if(append_str(&buf, &buf_sz, ":") != 0) 
 		goto err_return;
 	if(get_role_name(context->role, &name, policy) != 0) 
@@ -409,18 +467,35 @@ char *re_render_security_context(const security_con_t *context,
 	if(append_str(&buf, &buf_sz, name) != 0) 
 		goto err_return;
 	free(name);
+	name = NULL;
 	if(append_str(&buf, &buf_sz, ":") != 0) 
 		goto err_return;
 	if(get_type_name(context->type, &name, policy) != 0) 
 		goto err_return;
 	if(append_str(&buf, &buf_sz, name) != 0) 
 		goto err_return;
-	free(name);	
+	free(name);
+	name = NULL;
 	
+	/* render range */
+	if (context->range != NULL) {
+		if (append_str(&buf, &buf_sz, ":") != 0)
+			goto err_return;
+		range = re_render_mls_range(context->range, policy);
+		if (append_str(&buf, &buf_sz, range) != 0)
+			goto err_return;
+		free(range);
+		range = NULL;
+	}
 	return buf;
+
 err_return:
-	if(buf != NULL) 
+	if (buf != NULL) 
 		free(buf);
+	if (range != NULL)
+		free(range);
+	if (name != NULL)
+		free(name);
 	return NULL;	
 }
 
@@ -856,11 +931,11 @@ char *re_render_nodecon(ap_nodecon_t *nodecon, policy_t *policy)
 	char *addr_str = NULL;
 	char *mask_str = NULL;
 	uint32_t tmp = 0;
-	uint32_t *tmp_arr = NULL;
+	char *tmp_str = NULL;
 
 	/* max length of a string for an IP is 40 characters
 	 *  (8 fields * 4 char/field) + 7  * ':' + '\0' */
-	const size_t ip_addr_str_len_max = 40;
+	const size_t ip_addr_str_len_max = 41;
 
 	if (!nodecon || !policy)
 		return NULL;
@@ -883,15 +958,8 @@ char *re_render_nodecon(ap_nodecon_t *nodecon, policy_t *policy)
 		snprintf(mask_str, ip_addr_str_len_max - 1, "%3d.%3d.%3d.%3d", (tmp/(1<<24)), (tmp/(1<<16)%(1<<8)), (tmp/(1<<8)%(1<<8)), (tmp%(1<<8)));
 		break;
 	case AP_IPV6:
-		tmp_arr = nodecon->addr;
-		/* the math below prints the IPv6 fields in hexidecimal 2 bytes at a time */
-		snprintf(addr_str, ip_addr_str_len_max - 1, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x",
-			tmp_arr[0]/(1<<16), tmp_arr[0]%(1<<16), tmp_arr[1]/(1<<16), tmp_arr[1]%(1<<16),
-			tmp_arr[2]/(1<<16), tmp_arr[2]%(1<<16), tmp_arr[3]/(1<<16), tmp_arr[3]%(1<<16) );
-		tmp_arr = nodecon->mask;
-		snprintf(mask_str, ip_addr_str_len_max - 1, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x",
-			tmp_arr[0]/(1<<16), tmp_arr[0]%(1<<16), tmp_arr[1]/(1<<16), tmp_arr[1]%(1<<16),
-			tmp_arr[2]/(1<<16), tmp_arr[2]%(1<<16), tmp_arr[3]/(1<<16), tmp_arr[3]%(1<<16) );
+		snprintf(addr_str, ip_addr_str_len_max - 1, "%s", (tmp_str = re_render_ipv6_addr(nodecon->addr)));
+		snprintf(mask_str, ip_addr_str_len_max - 1, "%s", (tmp_str = re_render_ipv6_addr(nodecon->mask)));
 		break;
 	default:
 		break;
@@ -950,6 +1018,7 @@ char *re_render_genfscon(ap_genfscon_t *genfscon, policy_t *policy)
 		free(lines);
 		return NULL;
 	}
+
 	strcat(front_str, "genfscon ");
 	strcat(front_str, genfscon->fstype);
 	strcat(front_str, " ");
@@ -961,10 +1030,9 @@ char *re_render_genfscon(ap_genfscon_t *genfscon, policy_t *policy)
 		if (!context_str) {
 			return NULL;
 		}
-
 		switch (path->filetype) {
 		case FILETYPE_DIR:
-			type_str = strdup("-d ");
+			type_str = strdup(" -d ");
 			break;
 		case FILETYPE_CHR:
 			type_str = strdup(" -c ");
@@ -991,7 +1059,6 @@ char *re_render_genfscon(ap_genfscon_t *genfscon, policy_t *policy)
 			goto exit_err;
 			break;
 		}
-
 		/* front_str + path + type_str + context_str + '\0' */
 		lines[i] = (char*)calloc(len + strlen(path->path) + 4 + strlen(context_str) + 1 , sizeof(char));
 		if (!lines[i])
@@ -1281,65 +1348,9 @@ char *re_render_constraint(bool_t addlineno, ap_constraint_t *constraint, policy
 	return rt;
 }
 
-char *re_render_mls_level(ap_mls_level_t *level, policy_t *policy)
-{
-	char *rt = NULL;
-	int sz = 0, i, cur;
-
-	if (!level || !policy)
-		return NULL;
-
-	append_str(&rt, &sz, policy->sensitivities[level->sensitivity].name);
-
-	if (!level->categories)
-		return rt; /* no categories, simply return the sensitivity name */
-
-	append_str(&rt, &sz, ":");
-	append_str(&rt, &sz, policy->categories[level->categories[0]].name);
-
-	if (level->num_categories == 1)
-		return rt; /* only one category so done */
-
-	cur = 0; /* current value to compare with cat[i] */
-	for (i = 1; i < level->num_categories; i++) {
-		if (level->categories[i] == level->categories[cur] + 1) {
-			if (i + 1 == level->num_categories || level->categories[i+1] != level->categories[cur] + 2) {
-				append_str(&rt, &sz, ".");
-				append_str(&rt, &sz, policy->categories[level->categories[i]].name);
-				cur = i;
-			} else {
-				cur++;
-			}
-		} else {
-			append_str(&rt, &sz, ", ");
-			append_str(&rt, &sz, policy->categories[level->categories[i]].name);
-			cur = i;
-		}
-	}
-
-	return rt;
-}
-
-char *re_render_mls_range(ap_mls_range_t *range, policy_t *policy)
-{
-	char *rt = NULL;
-	int sz = 0;
-
-	if (!range || !policy)
-		return NULL;
-
-	append_str(&rt, &sz, re_render_mls_level(range->low, policy));
-	if (range->high != range->low) {
-		append_str(&rt, &sz, " - ");
-		append_str(&rt, &sz, re_render_mls_level(range->high, policy));
-	}
-
-	return rt;
-}
-
 char *re_render_rangetrans(bool_t addlineno, int idx, policy_t *policy)
 {
-	char *rt = NULL, *tmp_name = NULL;
+	char *rt = NULL, *tmp_name = NULL, *sub_str = NULL;
 	char tmp[BUF_SZ];
 	int sz = 0, retv;
 	ta_item_t *name = NULL;
@@ -1425,7 +1436,9 @@ char *re_render_rangetrans(bool_t addlineno, int idx, policy_t *policy)
 	append_str(&rt, &sz, " ");
 
 	/* render range */
-	append_str(&rt, &sz, re_render_mls_range(policy->rangetrans[idx].range, policy));
+	sub_str = re_render_mls_range(policy->rangetrans[idx].range, policy);
+	append_str(&rt, &sz, sub_str);
+	free(sub_str);
 
 	append_str(&rt, &sz, ";");
 
@@ -1519,3 +1532,144 @@ char *re_render_role_trans(bool_t addlineno, int idx, policy_t *policy)
 	return rt;
 }
 
+char *re_render_role_allow(bool_t addlineno, int idx, policy_t *policy)
+{
+	char *rt = NULL, *tmp_name = NULL;
+	char tmp[BUF_SZ];
+	int sz = 0, retv;
+	ta_item_t *name;
+
+	if (!policy || idx < 0 || idx >= policy->num_role_allow)
+		return NULL;
+
+	if (addlineno) {
+		snprintf(tmp, sizeof(tmp)-1, "[%7lu] ", policy->role_allow[idx].lineno);
+		append_str(&rt, &sz, tmp);
+	}
+
+	append_str(&rt, &sz, "allow ");
+
+	/* render source role(s) */
+	if (policy->role_allow[idx].flags & AVFLAG_SRC_STAR) {
+		append_str(&rt, &sz, "*");
+	} else if (policy->role_allow[idx].flags & AVFLAG_SRC_TILDA) {
+		append_str(&rt, &sz, "~");
+	}
+	if (policy->role_allow[idx].src_roles->next) {
+		append_str(&rt, &sz, "{");
+	}
+
+	for (name = policy->role_allow[idx].src_roles; name; name = name->next) {
+		retv = get_ta_item_name(name, &tmp_name, policy);
+		if (retv) {
+			free(rt);
+			return NULL;
+		}
+		snprintf(tmp, sizeof(tmp)-1, "%s", tmp_name);
+		append_str(&rt, &sz, tmp);
+		free(tmp_name);
+		tmp_name = NULL;
+		if (name->next) {
+			append_str(&rt, &sz, " ");
+		}
+	}
+
+	if (policy->role_allow[idx].src_roles->next) {
+		append_str(&rt, &sz, "}");
+	}
+	append_str(&rt, &sz, " ");
+
+	/* render target role(s) */
+	if (policy->role_allow[idx].flags & AVFLAG_SRC_STAR) {
+		append_str(&rt, &sz, "*");
+	} else if (policy->role_allow[idx].flags & AVFLAG_SRC_TILDA) {
+		append_str(&rt, &sz, "~");
+	}
+	if (policy->role_allow[idx].tgt_roles->next) {
+		append_str(&rt, &sz, "{");
+	}
+
+	for (name = policy->role_allow[idx].tgt_roles; name; name = name->next) {
+		retv = get_ta_item_name(name, &tmp_name, policy);
+		if (retv) {
+			free(rt);
+			return NULL;
+		}
+		snprintf(tmp, sizeof(tmp)-1, "%s", tmp_name);
+		append_str(&rt, &sz, tmp);
+		free(tmp_name);
+		tmp_name = NULL;
+		if (name->next) {
+			append_str(&rt, &sz, " ");
+		}
+	}
+
+	if (policy->role_allow[idx].tgt_roles->next) {
+		append_str(&rt, &sz, "}");
+	}
+
+	append_str(&rt, &sz, ";");
+
+	return rt;
+}
+
+/*	case AP_IPV6:
+		tmp_arr = nodecon->addr;
+		/ * the math below prints the IPv6 fields in hexidecimal 2 bytes at a time * /
+		snprintf(addr_str, ip_addr_str_len_max - 1, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x",
+			tmp_arr[0]/(1<<16), tmp_arr[0]%(1<<16), tmp_arr[1]/(1<<16), tmp_arr[1]%(1<<16),
+			tmp_arr[2]/(1<<16), tmp_arr[2]%(1<<16), tmp_arr[3]/(1<<16), tmp_arr[3]%(1<<16) );
+		tmp_arr = nodecon->mask;
+		snprintf(mask_str, ip_addr_str_len_max - 1, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x",
+			tmp_arr[0]/(1<<16), tmp_arr[0]%(1<<16), tmp_arr[1]/(1<<16), tmp_arr[1]%(1<<16),
+			tmp_arr[2]/(1<<16), tmp_arr[2]%(1<<16), tmp_arr[3]/(1<<16), tmp_arr[3]%(1<<16) );
+		break;
+*/
+char *re_render_ipv6_addr(uint32_t addr[4])
+{
+	char *str = NULL;
+	uint16_t tmp[8] = {0,0,0,0,0,0,0,0};
+	int i, sz = 0, retv;
+	char buff[40]; /* 8 * 4 hex digits + 7 * ':' + '\0' == max size of string */
+	int contract = 0, prev_contr = 0, contr_idx_end = -1; 
+	for (i = 0; i < 4; i++) {
+		tmp[2*i] = addr[i]/(1<<16);
+		tmp[2*i+1] = addr[i]%(1<<16);
+	}
+
+	for (i = 0; i < 8; i++) {
+		if (tmp[i] == 0) {
+			contract++;
+			if (i == 7 && contr_idx_end == -1)
+				contr_idx_end = 8;
+		} else {
+			if (contract > prev_contr) {
+				contr_idx_end = i;
+			}
+			prev_contr = contract;
+			contract = 0;
+		}
+	}
+
+	if (prev_contr > contract)
+		contract = prev_contr;
+
+	for (i = 0; i < 8; i++) {
+		if (i == contr_idx_end - contract) {
+			retv = snprintf(buff + sz, 40 - sz, i?":":"::");
+			sz += retv;
+		} else if (i > contr_idx_end - contract && i < contr_idx_end) {
+			continue;
+		} else {
+			retv = snprintf(buff + sz, 40 - sz, i==7?"%04x":"%04x:", tmp[i]);
+			sz += retv;
+		}
+	}
+
+	buff[sz] = '\0';
+
+	str = strdup(buff);
+
+	return str;
+}
+
