@@ -17,6 +17,7 @@
 #include <ctype.h>
 #include <assert.h>
 #include <unistd.h>
+#include <errno.h>
 
 /* convert from string to intger "bool" */
 bool_t getbool(const char *str) 
@@ -284,6 +285,48 @@ int init_rbac_bool(rbac_bool_t *b, policy_t *policy, bool_t roles)
 	return 0;
 }
 
+/* performs the logical operation b1[i] |= b2[i] */
+int rbac_bool_or_eq(rbac_bool_t *b1, rbac_bool_t *b2, policy_t *policy)
+{
+	int i;
+
+	if (!b1 || !b1->allow || !b1->trans || !b2 || !b2->allow || !b2->trans || !policy) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	for (i= 0; i < policy->num_role_allow; i++) {
+		b1->allow[i] |= b2->allow[i];
+	}
+
+	for (i = 0; i < policy->num_role_trans; i++) {
+		b1->trans[i] |= b2->trans[i];
+	}
+
+	return 0;
+}
+
+/* performs the logical operation b1[i] &= b2[i] */
+int rbac_bool_and_eq(rbac_bool_t *b1, rbac_bool_t *b2, policy_t *policy)
+{
+	int i;
+
+	if (!b1 || !b1->allow || !b1->trans || !b2 || !b2->allow || !b2->trans || !policy) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	for (i= 0; i < policy->num_role_allow; i++) {
+		b1->allow[i] &= b2->allow[i];
+	}
+
+	for (i = 0; i < policy->num_role_trans; i++) {
+		b1->trans[i] &= b2->trans[i];
+	}
+
+	return 0;
+}
+
 /* make the boolean array TRUE for all entries */
 int all_true_rules_bool(rules_bool_t *rules_b, policy_t *policy)
 {
@@ -354,6 +397,22 @@ int all_true_rbac_bool(rbac_bool_t *b, policy_t *policy)
 	return 0;
 }
 
+int all_false_rbac_bool(rbac_bool_t *b, policy_t *policy)
+{
+	if(b == NULL)
+		return -1;
+	
+	assert(b->allow != NULL);
+	memset(b->allow, 0, policy->num_role_allow* sizeof(bool_t));
+	b->a_cnt = policy->num_role_allow;
+	
+	assert(b->trans != NULL);
+	memset(b->trans, 0, policy->num_role_trans * sizeof(bool_t));
+	b->t_cnt = policy->num_role_trans;
+	
+	return 0;
+}
+
 int free_rules_bool(rules_bool_t *rules_b)
 {
 	if(rules_b == NULL)
@@ -637,7 +696,27 @@ char* find_file(const char *file_name)
 		return dir;	
 	}
 	
-	/* 4. Didn't find it! */
+	/* 4. help install directory */
+	filesz = strlen(APOL_HELP_DIR) + strlen(file_name) + 2;
+	file = (char *)malloc(filesz);
+	if(file == NULL) {
+		fprintf(stderr, "out of memory");
+		return NULL;
+	}	
+	sprintf(file, "%s/%s", APOL_HELP_DIR, file_name);
+	rt = access(file, R_OK);
+	if(rt == 0) {
+		dir = (char *)malloc(strlen(APOL_HELP_DIR) +1);
+		if(dir == NULL) {
+			fprintf(stderr, "out of memory");
+			return NULL;
+		}
+		sprintf(dir, APOL_HELP_DIR);
+		free(file);
+		return dir;	
+	}
+	
+	/* 5. Didn't find it! */
 	free(file);		
 	return NULL;		
 }
@@ -857,3 +936,92 @@ int read_file_to_buffer(const char *fname, char **buf, int *len)
 	}
 	return 0;
 }
+
+
+int str_to_internal_ip(const char *str, uint32_t ip[4])
+{
+	int len = 0, i, retv;
+	int seg = 0;
+	uint32_t val = 0; /* value of current segment of address */
+	bool_t ipv4 = FALSE;
+	bool_t ipv6 = FALSE;
+	char tmp[6] = {'0', 'x', '\0', '\0', '\0', '\0'};
+	int seg_cnt = 0; /* for counting number of segments in ipv6 addr */
+
+	if (!str || !ip) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	len = strlen(str);
+	ip[0] = ip[1] = ip[2] = ip[3] = 0;
+
+	if (strchr(str, '.'))
+		ipv4 = TRUE;
+
+	if (strchr(str, ':'))
+		ipv6 = TRUE;
+
+	if (ipv4 == ipv6) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (ipv6) {
+		for (i = 0; i < len; i++) {
+			if (str[i] == ':')
+				seg_cnt++;
+		}
+	}
+
+	for (i = 0; i <= len; i++) {
+		if (ipv4) {
+			if (str[i] == '.' || str[i] == '\0') {
+				if (val < 0 || val > 255) {
+					errno = EINVAL;
+					return -1;
+				}
+				ip[3] |= ((0x000000ff & val) << (8 * (3 - seg)));
+				seg++;
+				val = 0;
+				if (seg == 4)
+					break;
+			} else if (isdigit(str[i])) {
+				tmp[4] = str[i];
+				retv = atoi(&(tmp[4]));
+				val *= 10;
+				val += retv;
+			} else {
+				errno = EINVAL;
+				return -1;
+			}
+		} else if (ipv6) {
+			if (str[i] == ':' || str[i] == '\0') {
+				if (val < 0 || val > 0xffff) {
+					errno = EINVAL;
+					return -1;
+				}
+				ip[seg/2] |= ((0x0000ffff & val) << (16 * (1 - seg % 2)));
+				seg++;
+				val = 0;
+				if (i + 1 < len && str[i+1] == ':') {
+					seg += 8 - seg_cnt;
+					i++;
+				}
+				if (seg == 8)
+					break;
+			} else if (isxdigit(str[i])) {
+				tmp[2] = str[i];
+				retv = strtol(tmp, NULL, 16);
+				val = val << 4;
+				val += retv;
+			} else {
+				errno = EINVAL;
+				return -1;
+			}
+		}
+	}
+
+	return ipv4?AP_IPV4:AP_IPV6;
+}
+
