@@ -10,13 +10,14 @@
  * filter_criteria.c
  */
 
-#include "filter_criteria.h"
-#include "auditlog.h"
 #include <string.h>
 #include <sys/types.h>
 #include <assert.h>
 #include <fnmatch.h>
 #include <libxml/uri.h>
+#include "filter_criteria.h"
+#include "auditlog.h"
+#include "sort.h"
 
 typedef struct strs_criteria {
 	char **strs;
@@ -37,6 +38,11 @@ typedef glob_criteria_t exe_criteria_t;
 typedef glob_criteria_t path_criteria_t;
 typedef glob_criteria_t ipaddr_criteria_t;
 typedef glob_criteria_t host_criteria_t;
+typedef glob_criteria_t comm_criteria_t;
+
+typedef struct msg_criteria {
+	int val;	/* AVC_DENIED or AVC_GRANTED */
+} msg_criteria_t;
 
 typedef struct ports_criteria {
 	int val;
@@ -46,6 +52,12 @@ typedef struct netif_criteria {
 	char *netif;
 } netif_criteria_t;
 
+typedef struct date_time_criteria {
+	struct tm *start;
+	struct tm *end;
+	int option;
+} date_time_criteria_t;
+
 
 const char *netif_criteria_get_str(seaudit_criteria_t *criteria)
 {
@@ -90,6 +102,98 @@ int ports_criteria_get_val(seaudit_criteria_t *criteria)
 	return ports_criteria->val;
 }
 
+const struct tm *date_time_criteria_get_date(seaudit_criteria_t *criteria, bool_t start)
+{
+	date_time_criteria_t *dtc = NULL;
+	if (!criteria || !criteria->data)
+		return NULL;
+	dtc = (date_time_criteria_t *)criteria->data;
+	if (start)
+		return (const struct tm*)dtc->start;
+	else
+		return (const struct tm*)dtc->end;
+}
+
+int date_time_criteria_get_option(seaudit_criteria_t *criteria)
+{
+	date_time_criteria_t *dtc = NULL;
+	if (!criteria || !criteria->data)
+		return 0;
+	dtc = (date_time_criteria_t *)criteria->data;
+	return dtc->option;
+}
+
+static bool_t date_time_criteria_action(msg_t *msg, seaudit_criteria_t *criteria, audit_log_t *log)
+{
+	date_time_criteria_t *dt_criteria;
+	int rt;
+
+	if (msg == NULL || criteria == NULL || criteria->data == NULL)
+		return FALSE;
+	dt_criteria = (date_time_criteria_t *)criteria->data;
+
+	rt = date_time_compare(dt_criteria->start, msg->date_stamp);
+	if (dt_criteria->option == FILTER_CRITERIA_DT_OPTION_BEFORE) {
+		return   rt > 0 ? TRUE : FALSE;
+	} else if (dt_criteria->option == FILTER_CRITERIA_DT_OPTION_AFTER) {
+		return   rt < 0 ? TRUE : FALSE;
+	} else {
+		if (rt > 0)
+			return FALSE;
+		rt = date_time_compare(dt_criteria->end, msg->date_stamp);
+		return rt < 0 ? FALSE : TRUE;
+	}
+}
+
+static void date_time_criteria_print(seaudit_criteria_t *criteria, FILE *stream, int tabs)
+{
+	date_time_criteria_t *dt_criteria;
+	xmlChar *escaped;
+	xmlChar *str_xml;
+	int i;
+
+	if (criteria == NULL || criteria->data == NULL || stream == NULL)
+		return;
+
+	dt_criteria = (date_time_criteria_t*)criteria->data;
+	if (tabs < 0) 
+		tabs = 0;
+	for (i = 0; i < tabs; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<criteria type=\"date_time\">\n");
+	str_xml = xmlCharStrdup(asctime(dt_criteria->start));
+	escaped = xmlURIEscapeStr(str_xml, NULL);
+	for (i = 0; i < tabs+1; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<item>%s</item>\n", escaped);
+	free(escaped);
+	free(str_xml);
+	str_xml = xmlCharStrdup(asctime(dt_criteria->end));
+	escaped = xmlURIEscapeStr(str_xml, NULL);
+	for (i = 0; i < tabs+1; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<item>%s</item>\n", escaped);
+	free(escaped);
+	free(str_xml);
+	for (i = 0; i < tabs+1; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<item>%d</item>\n", dt_criteria->option);
+	for (i = 0; i < tabs; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "</criteria>\n");
+}
+
+int msg_criteria_get_val(seaudit_criteria_t *criteria)
+{
+	msg_criteria_t *msg_criteria;
+	
+	if (!criteria)
+		return -1;
+	msg_criteria = (msg_criteria_t*)criteria->data;
+	assert(msg_criteria);
+	return msg_criteria->val;
+}
+
 static bool_t netif_criteria_action(msg_t *msg, seaudit_criteria_t *criteria, audit_log_t *log)
 {
 	netif_criteria_t *netif_criteria;
@@ -174,6 +278,42 @@ static void ports_criteria_print(seaudit_criteria_t *criteria, FILE *stream, int
 	fprintf(stream, "</criteria>\n");
 }
 
+static bool_t msg_criteria_action(msg_t *msg, seaudit_criteria_t *criteria, audit_log_t *log)
+{
+	msg_criteria_t *msg_criteria;
+
+	if (msg == NULL || criteria == NULL || criteria->data == NULL)
+		return FALSE;
+
+	msg_criteria = (msg_criteria_t*)criteria->data;
+	if (msg_criteria->val == msg->msg_data.avc_msg->msg)
+		return TRUE;
+	return FALSE;
+}
+
+static void msg_criteria_print(seaudit_criteria_t *criteria, FILE *stream, int tabs)
+{
+	msg_criteria_t *msg_criteria;
+	int i;
+
+	if (criteria == NULL || criteria->data == NULL || stream == NULL)
+		return;
+
+	if (tabs < 0)
+		tabs = 0;
+	msg_criteria = (msg_criteria_t*)criteria->data;
+
+	for (i = 0; i < tabs; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<criteria type=\"msg\">\n");
+	for (i = 0; i < tabs+1; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<item>%d</item>\n", msg_criteria->val);
+	for (i = 0; i < tabs; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "</criteria>\n");
+}
+
 static bool_t ipaddr_criteria_action(msg_t *msg, seaudit_criteria_t *criteria, audit_log_t *log)
 {
 	ipaddr_criteria_t *ipaddr_criteria;
@@ -270,6 +410,47 @@ static void host_criteria_print(seaudit_criteria_t *criteria, FILE *stream, int
 	free(str_xml);
 }
 
+static bool_t comm_criteria_action(msg_t *msg, seaudit_criteria_t *criteria, audit_log_t *log)  
+{ 
+	comm_criteria_t *comm_criteria;
+
+	if (msg == NULL || criteria == NULL || criteria->data == NULL || !msg->msg_data.avc_msg->comm)
+		return FALSE;
+
+	comm_criteria = (comm_criteria_t*)criteria->data;
+	if (!comm_criteria->globex)
+		return FALSE;
+	if (fnmatch(comm_criteria->globex, msg->msg_data.avc_msg->comm, 0) == 0)
+		return TRUE;
+	return FALSE;
+} 
+
+static void comm_criteria_print(seaudit_criteria_t *criteria, FILE *stream, int tabs)
+{
+	comm_criteria_t *comm_criteria;
+	xmlChar *escaped;
+	xmlChar *str_xml;
+	int i;
+
+	if (criteria == NULL || criteria->data == NULL || stream == NULL)
+		return;
+
+	comm_criteria = (comm_criteria_t*)criteria->data;
+	str_xml = xmlCharStrdup(comm_criteria->globex);
+	escaped = xmlURIEscapeStr(str_xml, NULL);
+	for (i = 0; i < tabs; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<criteria type=\"comm\">\n");
+	for (i = 0; i < tabs+1; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "<item>%s</item>\n", escaped);
+	for (i = 0; i < tabs; i++)
+		fprintf(stream, "\t");
+	fprintf(stream, "</criteria>\n");
+	free(escaped);
+	free(str_xml);
+}
+
 static bool_t exe_criteria_action(msg_t *msg, seaudit_criteria_t *criteria, audit_log_t *log)  
 { 
 	exe_criteria_t *exe_criteria;
@@ -999,6 +1180,60 @@ seaudit_criteria_t* class_criteria_create(char **classes, int num_classes)
 	return new_criteria;
 }
 
+/*
+ * destroy the exe criteria, not the container struct */
+static void comm_criteria_destroy(seaudit_criteria_t *ftr)
+{
+	comm_criteria_t *d;
+
+	if (ftr == NULL || ftr->data == NULL)
+		return;
+	d = (comm_criteria_t*)ftr->data;
+	if (d->globex)
+		free(d->globex);
+	free(d);
+	return;
+}
+
+/*
+ * create the entire exe criteria */
+seaudit_criteria_t* comm_criteria_create(const char* comm)
+{
+        seaudit_criteria_t *new;
+	comm_criteria_t *d;
+	int i;
+
+	d = (comm_criteria_t*)malloc(sizeof(comm_criteria_t));
+	if (d == NULL) 
+		goto bad;
+	memset(d, 0, sizeof(comm_criteria_t));
+	i = strlen(comm);
+	d->globex = (char*)malloc(sizeof(char) * (i+1));
+	if (d->globex == NULL) 
+		goto bad;
+	new = criteria_create();
+	if (new == NULL) {
+		goto bad;
+	}
+	/* set container variables */
+	new->msg_types |= AVC_MSG;
+	new->criteria_act = &comm_criteria_action; 
+	new->print = &comm_criteria_print;
+	new->destroy = &comm_criteria_destroy; 
+	new->data = d; 
+	/* set criteria variables */
+	strcpy(d->globex, comm);
+	return new;
+bad:
+	fprintf(stdout, "Out of memory");
+	if (d) {
+		if (d->globex)
+			free(d->globex);
+		free(d);
+	}
+	return NULL;
+}
+
 /*
  * destroy the exe criteria, not the container struct */
 static void exe_criteria_destroy(seaudit_criteria_t *ftr)
@@ -1293,4 +1528,113 @@ bad:
 	return NULL;
 }
 
+static void msg_criteria_destroy(seaudit_criteria_t *ftr)
+{
+	msg_criteria_t *d;
+	if (ftr == NULL)
+		return;
+	d = (msg_criteria_t*)ftr->data;
+	if (d == NULL)
+		return;
+	free(d);
+	return;
+}
+
+static void date_time_criteria_destroy(seaudit_criteria_t *ftr)
+{
+	date_time_criteria_t *d;
+
+	if (ftr == NULL)
+		return;
+
+	d = (date_time_criteria_t*)ftr->data;
+	if (d == NULL)
+		return;
+
+	free(d->start);
+	free(d->end);
+	free(d);
+	return;
+}
+
+/* 
+ *  Create a date/time filter with the given start time, end time, and matching
+ *  option.
+ */
+seaudit_criteria_t* date_time_criteria_create(struct tm *start, struct tm *end, int option)
+{
+	seaudit_criteria_t *rt = NULL;
+	date_time_criteria_t *d = NULL;
+	
+	if (option != FILTER_CRITERIA_DT_OPTION_BETWEEN &&
+		option != FILTER_CRITERIA_DT_OPTION_AFTER &&
+		option != FILTER_CRITERIA_DT_OPTION_BEFORE)
+		return NULL;	/* invalid option */
+
+	/* malloc new date criteria */
+	d = (date_time_criteria_t*)calloc(1, sizeof(date_time_criteria_t));
+	if (d == NULL)
+		goto bad;
+	d->start = (struct tm *)calloc(1, sizeof(struct tm));
+	if (d->start == NULL)
+		goto bad;
+	d->end = (struct tm *)calloc(1, sizeof(struct tm));
+	if (d->end == NULL)
+		goto bad;
+
+	/* set the criteria */
+	*(d->start) = *start;
+	*(d->end) = *end;
+	d->option = option;
+
+	/* create container filter */
+	rt = criteria_create();
+	if (rt == NULL)
+		goto bad;
+	rt->msg_types = AVC_MSG | LOAD_POLICY_MSG | BOOLEAN_MSG;
+	rt->criteria_act = &date_time_criteria_action;
+	rt->print = &date_time_criteria_print;
+	rt->destroy = &date_time_criteria_destroy;
+	rt->data = d;
+	return rt;
+
+bad:
+	fprintf(stdout, "Out of memory");
+	if (d) {
+		free(d->start);
+		free(d->end);
+		free(d);
+	}
+	if (rt)
+		seaudit_criteria_destroy(rt);
+	return NULL;
+}
+
+seaudit_criteria_t* msg_criteria_create(int msg)
+{
+        seaudit_criteria_t *new;
+	msg_criteria_t *d;
+	d = (msg_criteria_t*)malloc(sizeof(msg_criteria_t));
+	if (d == NULL)
+		goto bad;
+	memset(d, 0, sizeof(msg_criteria_t));
+	new = criteria_create();
+	if (new == NULL)
+		goto bad;
+	/* set container variables */
+	new->msg_types |= AVC_MSG;
+	new->criteria_act = &msg_criteria_action; 
+	new->print = &msg_criteria_print;
+	new->destroy = &msg_criteria_destroy; 
+	new->data = d; 
+	/* set criteria variables */
+	d->val = msg;
+	return new;
+bad:
+	fprintf(stdout, "Out of memory");
+	if (d) {
+		free(d);
+	}
+	return NULL;
+}
 
