@@ -19,12 +19,13 @@
 #include <time.h>
 #include <assert.h>
 
-#define MEMORY_BLOCK_MAX_SIZE 256
+#define MEMORY_BLOCK_MAX_SIZE 2048
 #define NUM_TIME_COMPONENTS 3
 #define OLD_LOAD_POLICY_STRING "loadingpolicyconfigurationfrom"
 #define AVCMSG " avc: "
 #define LOADMSG " security: "
-#define SYSCALL_STRING "msg=audit("
+#define SYSCALL_STRING "audit("
+#define ALT_SYSCALL_STRING "msg=audit("  /* should contain SYSCALL_STRING */
 #define BOOLMSG "committed booleans"
 #define AUDITD_MSG "type="
 #define PARSE_AVC_MSG 1
@@ -192,7 +193,7 @@ static unsigned int insert_time(char **tokens, msg_t *msg, int *position, int nu
 	length += 1 + (NUM_TIME_COMPONENTS - 1); 
 	if ((time = (char*) malloc(length * (sizeof(char)))) == NULL)
 		return PARSE_RET_MEMORY_ERROR;
-	
+
 	if (*position == num_tokens)
 		return PARSE_REACHED_END_OF_MSG;
 	strcpy(time, tokens[*position]);	
@@ -200,26 +201,30 @@ static unsigned int insert_time(char **tokens, msg_t *msg, int *position, int nu
 	(*position)++;
 	if (*position == num_tokens)
 		return PARSE_REACHED_END_OF_MSG;
+	
 	time = strcat(time, tokens[*position]);
 	time = strcat(time, " " );
 	(*position)++;
 	if (*position == num_tokens)
 		return PARSE_REACHED_END_OF_MSG;
 	time = strcat(time, tokens[*position]);
-   
+
 	if (!msg->date_stamp) {
 		if ((msg->date_stamp = (struct tm*) malloc(sizeof(struct tm))) == NULL)
 			return PARSE_RET_MEMORY_ERROR;
+		memset(msg->date_stamp, 0, sizeof(sizeof(struct tm)));
 	}
-	msg->date_stamp->tm_isdst = (daylight==0 ? 0 : 1);
 
-	if (!strptime(time, "%b %d %T", msg->date_stamp)) {    
+	if (!strptime(time, "%b %d %T", msg->date_stamp)) {
 		free(time); 
 		return 0;
 	} else {
 		free(time);
-		/* random year to make mktime happy */
-		msg->date_stamp->tm_year = 2000 - 1900;
+		/* set year to 1900 since we know no valid
+		 logs were generated then this will tell us that
+		the msg does not really have a year*/
+		msg->date_stamp->tm_isdst = 0;
+		msg->date_stamp->tm_year = 0;
 		return PARSE_RET_SUCCESS;
 	}
 	
@@ -249,6 +254,12 @@ static unsigned int avc_msg_insert_syscall_info(char *token, msg_t *msg)
 		length--;
 	}
 	header_len = strlen(SYSCALL_STRING);
+
+	/* Check to see if variations on syscall header exist */
+	if (strstr(token, ALT_SYSCALL_STRING)) {
+	  header_len = strlen(ALT_SYSCALL_STRING);
+	}
+
 	time_str = &token[header_len];
 	/* Parse seconds.nanoseconds:serial */
 	while ((fields[i] = strsep(&time_str, ".:")) != NULL && i < PARSE_NUM_SYSCALL_FIELDS) {
@@ -475,6 +486,32 @@ static int avc_msg_insert_string(char **dest, char **src)
 	return MSG_INSERT_SUCCESS;
 }
 
+/* removes quotes from a string, this is currently to 
+   remove quotes from the command argument */
+static int avc_msg_remove_quotes_insert_string(char **dest, char **src)
+{
+	int i, j, l;	
+	assert(dest != NULL && src != NULL && *src != NULL);
+	l = strlen(*src) - 1;
+	/* see if there are any quotes to begin with 
+	   if there aren't just run insert string */
+	if ((*src)[0] == '"' && (*src)[l] == '"') {
+		if ((*dest = (char*)calloc((strlen(*src) + 1), sizeof(char))) == NULL)
+			return MSG_MEMORY_ERROR;
+		j = 0;
+		for (i = 0; i < strlen(*src); i++) {
+			if ((*src)[i] == '"')
+				continue;
+			(*dest)[j] = (*src)[i];
+			j++;
+		}
+		(*dest)[j] = '\0';
+		return MSG_INSERT_SUCCESS;
+	} else
+		return avc_msg_insert_string(dest, src);
+
+}
+
 static unsigned int insert_hostname(audit_log_t *log, char **tokens, msg_t *msg, int *position, int num_tokens)
 {
         int id;
@@ -581,14 +618,14 @@ static unsigned int avc_msg_insert_additional_field_data(char **tokens, msg_t *m
 			    return PARSE_RET_MEMORY_ERROR;
 		}
 
-		if (found[AVC_EXE_FIELD] == PARSE_NOT_MATCH && avc_msg_is_prefix(*(&tokens[i]), "exe=", &field_value) != FALSE) {
+		if (found[AVC_EXE_FIELD] == PARSE_NOT_MATCH && avc_msg_is_prefix(*(&tokens[i]), "exe=", &field_value) != FALSE) {			
 			found[AVC_EXE_FIELD] = avc_msg_insert_string(&msg->msg_data.avc_msg->exe, &field_value);
 			if (found[AVC_EXE_FIELD] == PARSE_RET_MEMORY_ERROR)
 			    return PARSE_RET_MEMORY_ERROR;
 		}
 		
 		if (found[AVC_COMM_FIELD] == PARSE_NOT_MATCH && avc_msg_is_prefix(*(&tokens[i]), "comm=", &field_value) != FALSE) {
-			found[AVC_COMM_FIELD] = avc_msg_insert_string(&msg->msg_data.avc_msg->comm, &field_value);
+			found[AVC_COMM_FIELD] = avc_msg_remove_quotes_insert_string(&msg->msg_data.avc_msg->comm, &field_value);
 			if (found[AVC_COMM_FIELD] == PARSE_RET_MEMORY_ERROR)
 			    return PARSE_RET_MEMORY_ERROR;
 		}
@@ -835,7 +872,7 @@ static unsigned int insert_standard_msg_header(char **tokens, msg_t *msg, audit_
 
 static unsigned int avc_msg_insert_field_data(char **tokens, msg_t *msg, audit_log_t *log, int num_tokens)
 {
-	int position = 0;
+        int position = 0;
 	unsigned int ret = 0, tmp_ret = 0;
 
 	assert(tokens != NULL && msg != NULL && log != NULL && num_tokens > 0);
@@ -845,6 +882,8 @@ static unsigned int avc_msg_insert_field_data(char **tokens, msg_t *msg, audit_l
 		position++;
 		if (position == num_tokens)
 			return PARSE_RET_INVALID_MSG_WARN;
+		 if (audit_log_get_log_type(log) != AUDITLOG_AUDITD)
+			 audit_log_set_log_type(log, AUDITLOG_AUDITD); 
 	}
 
 	/* Insert the audit header if it exists */
@@ -865,7 +904,7 @@ static unsigned int avc_msg_insert_field_data(char **tokens, msg_t *msg, audit_l
 		else if (tmp_ret & PARSE_REACHED_END_OF_MSG) 
 			return PARSE_RET_INVALID_MSG_WARN;
 		else if (!(tmp_ret & PARSE_RET_INVALID_MSG_WARN)) {
-			position++;
+			position += 2;
 			if (position == num_tokens)
 				return PARSE_RET_INVALID_MSG_WARN;
 		}
@@ -881,6 +920,24 @@ static unsigned int avc_msg_insert_field_data(char **tokens, msg_t *msg, audit_l
 			if (position == num_tokens)
 				return PARSE_RET_INVALID_MSG_WARN;
 		}
+
+		/* new style audit messages can show up in syslog files starting with
+		 * FC5. This means that both the old kernel: header and the new
+		 * audit header might be present. So, here we check again for the
+		 * audit message.
+		 */
+		if (avc_msg_is_token_new_audit_header(*(&tokens[position]))) {
+			tmp_ret |= avc_msg_insert_syscall_info(*(&tokens[position]), msg);
+			if (tmp_ret & PARSE_RET_SUCCESS) {
+			  position += 2;
+			  if (position == num_tokens)
+					return PARSE_RET_INVALID_MSG_WARN;
+			}
+			ret |= tmp_ret;
+			/* Reset our bitmask */
+			tmp_ret = 0;
+		}
+
 	}
 		
 	/* Make sure the following token is the string "avc:" */
