@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Tresys Technology, LLC
+/* Copyright (C) 2003-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -247,7 +247,11 @@ static void log_view_store_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter
 
 	if (DATE_FIELD == column) {
 		char date[DATE_STR_SIZE];
-		strftime(date, DATE_STR_SIZE, "%b %d %H:%M:%S", store->log_view->my_log->msg_list[indx]->date_stamp);
+		/* check to see if we have been given a valid year, if so display, otherwise no year displayed */
+		if (store->log_view->my_log->msg_list[indx]->date_stamp->tm_year == 0)
+			strftime(date, DATE_STR_SIZE, "%b %d %H:%M:%S", store->log_view->my_log->msg_list[indx]->date_stamp);
+		else 
+			strftime(date, DATE_STR_SIZE, "%b %d %H:%M:%S %Y", store->log_view->my_log->msg_list[indx]->date_stamp);
 		set_utf8_return_value(value, date);
 		return;
 	}
@@ -322,6 +326,9 @@ static void log_view_store_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter
 	case AVC_EXE_FIELD:
 		set_utf8_return_value(value, cur_msg->exe);
 		break;
+	case AVC_COMM_FIELD:
+		set_utf8_return_value(value, cur_msg->comm);
+		break;
 	case AVC_PATH_FIELD:
 		set_utf8_return_value(value, cur_msg->path);
 		break;
@@ -606,6 +613,10 @@ static void seaudit_log_view_store_set_sort_column_id(GtkTreeSortable *sortable,
 		if (audit_log_view_append_sort(store->log_view, exe_sort_action_create()))
 			return;
 		break;
+	case AVC_COMM_FIELD:
+		if (audit_log_view_append_sort(store->log_view, comm_sort_action_create()))
+			return;
+		break;
 	case AVC_PATH_FIELD:
 		if (audit_log_view_append_sort(store->log_view, path_sort_action_create()))
 			return;
