@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Tresys Technology, LLC
+/* Copyright (C) 2003-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -21,6 +21,11 @@
 #include <libseaudit/auditlog.h>
 #include <libapol/policy.h>
 #include <string.h>
+#include <time.h>
+#include <ctype.h>
+#include <limits.h>
+
+#define TM_YEAR_ZERO 1900
 
 enum {
 	ITEMS_LIST_COLUMN, 
@@ -43,6 +48,22 @@ enum select_values_source_t {
 	SEAUDIT_FROM_UNION
 };
 
+enum message_types {
+	SEAUDIT_MSG_NONE,
+	SEAUDIT_MSG_AVC_DENIED,
+	SEAUDIT_MSG_AVC_GRANTED
+};
+
+/* define the date time options */
+#define SEAUDIT_DT_OPTION_NONE 0
+#define SEAUDIT_DT_OPTION_BEFORE 1
+#define SEAUDIT_DT_OPTION_AFTER 2
+#define SEAUDIT_DT_OPTION_BETWEEN 3
+
+#define SEAUDIT_MAX_U16_STRLEN 5 /* the max # of chars that can be in a string representing a 16 bit digit */
+
+const char *msg_type_strs[] = { "none", "denied", "granted" };
+
 typedef struct  seaudit_filter_list {
 	char **list;
 	int size;
@@ -60,8 +81,143 @@ typedef struct filters_select_items {
 	struct filter_window *parent;
 } filters_select_items_t;
 
+typedef struct filters_date_item {
+	struct tm *start;
+	struct tm *end;
+	unsigned int option;
+} filters_date_item_t;
+
 extern seaudit_t *seaudit_app;
 
+/* Given the year and the month set the spin button to have the correct number of 
+   days for that month */
+static void filter_window_date_set_number_days(int month, GtkSpinButton *button)
+{	
+	int days[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
+	int cur_day;
+	if (!button || month < 0)
+		return;
+	/* we have to get the current day because set_range moves the current day
+	   by the difference minus 1 day, i.e. going from jan 20 to february
+	   the day would automatically become 18 */
+	cur_day = gtk_spin_button_get_value_as_int(button);
+	gtk_spin_button_set_range(button, 1, days[month]);
+	/* return to current day, or to the max value allowed in range*/
+	gtk_spin_button_set_value(button, cur_day);	
+	return;
+}
+
+/* this is called whenever the month or year is changed in the date filter*/
+static void filter_window_date_update_days(GladeXML *xml)
+{
+	int month;
+	GtkWidget *widget;
+
+	if (!xml)
+		return;
+
+	widget = glade_xml_get_widget(xml, "DateStartMonth");	
+	/* remember that months start with 0 in this widget */
+	month = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+	widget = glade_xml_get_widget(xml, "DateStartDay");	
+	filter_window_date_set_number_days(month, GTK_SPIN_BUTTON(widget));
+
+	widget = glade_xml_get_widget(xml, "DateEndMonth");	
+	/* remember that months start with 0 in this widget */
+	month = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+	widget = glade_xml_get_widget(xml, "DateEndDay");	
+	filter_window_date_set_number_days(month, GTK_SPIN_BUTTON(widget));
+
+}
+
+/* set the frames sorrounding date options sensitive based on toggle selections */
+static void filter_window_date_set_sensitive(GtkToggleButton *tb, gpointer user_data)
+{
+	GtkWidget *startframe;
+	GtkWidget *endframe;
+	filter_window_t *fw = (filter_window_t *)user_data;
+	const char *name;
+	/* if this toggle button is the deselected one just return */
+	if (!gtk_toggle_button_get_active(tb))
+		return;
+	startframe = glade_xml_get_widget(fw->xml, "DateStartFrame");
+	endframe = glade_xml_get_widget(fw->xml, "DateEndFrame");
+	if (!startframe || !endframe)
+		return;
+	name =  gtk_widget_get_name(GTK_WIDGET(tb));
+	if (strcmp(name, "MatchNoneRadio") == 0) {
+		gtk_widget_set_sensitive(startframe, FALSE);
+		gtk_widget_set_sensitive(endframe, FALSE);
+	} else if (strcmp(name, "MatchBetweenRadio") == 0) {
+		gtk_widget_set_sensitive(startframe, TRUE);
+		gtk_widget_set_sensitive(endframe, TRUE);
+	} else {
+		gtk_widget_set_sensitive(startframe, TRUE);
+		gtk_widget_set_sensitive(endframe, FALSE);
+	}
+}
+
+/* get the option from the date filter gui */
+int filters_window_get_tm_option(GladeXML *xml)
+{
+	GtkWidget *widget;
+	widget = glade_xml_get_widget(xml, "MatchNoneRadio");	
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+		return SEAUDIT_DT_OPTION_NONE;
+	widget = glade_xml_get_widget(xml, "MatchBeforeRadio");	
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+		return SEAUDIT_DT_OPTION_BEFORE;
+	widget = glade_xml_get_widget(xml, "MatchAfterRadio");	
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+		return SEAUDIT_DT_OPTION_AFTER;
+	return SEAUDIT_DT_OPTION_BETWEEN;
+}
+
+/* 
+   get the current time stored in the window and place in parameter tm 
+   unless the year is not valid, then do not overwrite the year 
+*/
+static int filters_window_update_tm(filter_window_t *fw, bool_t start, struct tm *tm)
+{
+	GladeXML *xml;
+	GtkWidget *widget = NULL;
+
+	if (!fw || !tm)
+		return -1;
+	xml = fw->xml;
+	if (start) {
+		/* get the date */
+		widget = glade_xml_get_widget(xml, "DateStartMonth");	
+		tm->tm_mon = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+		widget = glade_xml_get_widget(xml, "DateStartDay");	
+		tm->tm_mday = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		tm->tm_year = 0;
+		/* get the hour min sec */
+		widget = glade_xml_get_widget(xml, "DateStartHour");	
+		tm->tm_hour = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		widget = glade_xml_get_widget(xml, "DateStartMinute");	
+		tm->tm_min = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		widget = glade_xml_get_widget(xml, "DateStartSecond");	
+		tm->tm_sec = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));		
+	} else {
+		/* get the date */
+		widget = glade_xml_get_widget(xml, "DateEndMonth");	
+		tm->tm_mon = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+		widget = glade_xml_get_widget(xml, "DateEndDay");	
+		tm->tm_mday = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		tm->tm_year = 0;
+		/* get the hour min sec */
+		widget = glade_xml_get_widget(xml, "DateEndHour");	
+		tm->tm_hour = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		widget = glade_xml_get_widget(xml, "DateEndMinute");	
+		tm->tm_min = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		widget = glade_xml_get_widget(xml, "DateEndSecond");	
+		tm->tm_sec = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));		
+	}
+	return 0;
+}
+
+
 /******************************************************************
  * The following are private methods for the filters_select_items *
  * object. This object is encapsulated by the filters object. 	  *
@@ -70,7 +226,7 @@ static GtkEntry* filters_select_items_get_entry(filters_select_items_t *s)
 {	
 	GtkEntry *entry;
 	GtkWidget *widget;
-	
+
 	switch (s->items_list_type) {
 	case SEAUDIT_SRC_TYPES:
 		widget = glade_xml_get_widget(s->parent->xml, "SrcTypesEntry");
@@ -605,7 +761,7 @@ static void filters_select_items_on_close_button_clicked(GtkButton *button, filt
 		filter_items_list->window = NULL;
 		filters_select_items_fill_entry(filter_items_list);
 		log_load_callback_remove(&filters_select_on_log_opened, filter_items_list);
-		policy_load_callback_remove(&filters_select_on_policy_opened, filter_items_list);
+		policy_load_callback_remove(&filters_select_on_policy_opened, filter_items_list);		
 	}
 }
 
@@ -740,7 +896,7 @@ static filters_select_items_t* filters_select_items_create(filter_window_t *pare
 	return item;
 }
 
-static void filters_select_items_display(filters_select_items_t *filter_items_list)
+static void filters_select_items_display(filters_select_items_t *filter_items_list, GtkWindow *parent)
 {
 	GladeXML *xml;
 	GtkTreeView *include_tree, *exclude_tree;
@@ -770,9 +926,16 @@ static void filters_select_items_display(filters_select_items_t *filter_items_li
 	g_assert(window);
 	filter_items_list->window = window;
 
+
+
 	/* Set this new dialog transient for the parent filters dialog so that
 	 * it will be destroyed when the parent filters dialog is destroyed */
-	gtk_window_set_transient_for(window, filter_items_list->parent->window);
+	/* set this window to be transient on the parent window, so that when it pops up it gets centered on it */
+	/* however to have it "appear" to be centered we have to hide and then show */
+	gtk_window_set_transient_for(window, parent);
+	gtk_window_set_position(window, GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_widget_hide(GTK_WIDGET(window));
+	gtk_window_present(window);
 	gtk_window_set_destroy_with_parent(window, FALSE);
 	    
 	if (!seaudit_app->cur_policy) {
@@ -901,6 +1064,59 @@ static void filters_select_items_display(filters_select_items_t *filter_items_li
 /********************************************************
  * Private methods and callbacks for the filters object *
  ********************************************************/ 
+static int filter_window_verify_filter_values(filter_window_t *filter_window)
+{
+	char *text = NULL;	
+	GtkWidget *widget = NULL;
+	int i, j;
+	struct tm *t1 = NULL, *t2 = NULL;
+
+	/* check for network port filter */
+	/* since we do not know if this is an ipv4 or ipv6 port we check against
+	   ipv6 sizes since that is the larger of the two */
+	widget = glade_xml_get_widget(filter_window->xml, "PortEntry");
+	text = (char*)gtk_entry_get_text(GTK_ENTRY(widget));
+	/* check to see if the strlen of this number is larger than 
+	   the strlen of a __16 which is the 
+	   maximum size a ipv6 port can be as defined in in6.h*/
+	if (strlen(text) > SEAUDIT_MAX_U16_STRLEN) {
+		message_display(filter_window->window, GTK_MESSAGE_ERROR, "Invalid Port Filter.\n");
+		return -1;
+	}
+	if (strcmp(text, "") != 0) {
+		for (i = 0 ; i < strlen(text); i++) {
+			if (isdigit(text[i]) == 0) {
+				message_display(filter_window->window, GTK_MESSAGE_ERROR, "Invalid Port Filter.\n");
+				return -1;
+			}
+		}
+		j = atoi(text);
+		/* now check that the number returned is within the range
+		   of #'s allowed for a ipv6 port */
+		if (j < 0 || j > (1 << 16)) {
+			message_display(filter_window->window, GTK_MESSAGE_ERROR, "Invalid Port Filter.\n");
+			return -1;
+		}
+	}	
+
+	/*check the date and time */
+	if (filters_window_get_tm_option(filter_window->xml) == SEAUDIT_DT_OPTION_BETWEEN) {
+		t1 = (struct tm *)calloc(1, sizeof(struct tm));
+		t2 = (struct tm *)calloc(1, sizeof(struct tm));
+		filters_window_update_tm(filter_window, TRUE, t1);
+		filters_window_update_tm(filter_window, FALSE, t2);
+		if (date_time_compare(t1, t2) > 0) {
+			free(t1);
+			free(t2);
+			message_display(filter_window->window, GTK_MESSAGE_ERROR, "Invalid Date Filter.\n");
+			return -1;
+		}
+		free(t1);
+		free(t2);
+	}
+	return 0;
+}
+
 static void filter_window_set_title(filter_window_t *filter_window)
 {
 	GString *title;
@@ -941,6 +1157,10 @@ static void filter_window_set_values(filter_window_t *filter_window)
 	/* set executable */
 	widget = glade_xml_get_widget(filter_window->xml, "ExeEntry");
 	gtk_entry_set_text(GTK_ENTRY(widget), filter_window->executable->str);
+
+	/* set command */
+	widget = glade_xml_get_widget(filter_window->xml, "CommEntry");
+	gtk_entry_set_text(GTK_ENTRY(widget), filter_window->comm->str);
 	
 	/* set path */
 	widget = glade_xml_get_widget(filter_window->xml, "PathEntry");
@@ -960,6 +1180,49 @@ static void filter_window_set_values(filter_window_t *filter_window)
 	widget = glade_xml_get_widget(filter_window->xml, "HostEntry");
 	gtk_entry_set_text(GTK_ENTRY(widget), filter_window->host->str);
 
+	/* next set the time*/
+	if (filter_window->dates) {
+		widget = glade_xml_get_widget(filter_window->xml, "DateStartMonth");	
+		gtk_combo_box_set_active(GTK_COMBO_BOX(widget), filter_window->dates->start->tm_mon);
+		widget = glade_xml_get_widget(filter_window->xml, "DateStartDay");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->start->tm_mday);
+		widget = glade_xml_get_widget(filter_window->xml, "DateStartHour");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->start->tm_hour);
+		widget = glade_xml_get_widget(filter_window->xml, "DateStartMinute");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->start->tm_min);
+		widget = glade_xml_get_widget(filter_window->xml, "DateStartSecond");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->start->tm_sec);
+
+		widget = glade_xml_get_widget(filter_window->xml, "DateEndMonth");	
+		gtk_combo_box_set_active(GTK_COMBO_BOX(widget), filter_window->dates->end->tm_mon);
+		widget = glade_xml_get_widget(filter_window->xml, "DateEndDay");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->end->tm_mday);
+		widget = glade_xml_get_widget(filter_window->xml, "DateEndHour");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->end->tm_hour);
+		widget = glade_xml_get_widget(filter_window->xml, "DateEndMinute");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->end->tm_min);
+		widget = glade_xml_get_widget(filter_window->xml, "DateEndSecond");	
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), filter_window->dates->end->tm_sec);
+		
+		switch (filter_window->dates->option) {
+		case SEAUDIT_DT_OPTION_NONE:
+			widget = glade_xml_get_widget(filter_window->xml, "MatchNoneRadio");
+			break;
+		case SEAUDIT_DT_OPTION_BEFORE:
+			widget = glade_xml_get_widget(filter_window->xml, "MatchBeforeRadio");
+			break;
+		case SEAUDIT_DT_OPTION_AFTER:
+			widget = glade_xml_get_widget(filter_window->xml, "MatchAfterRadio");
+			break;
+		default :
+			widget = glade_xml_get_widget(filter_window->xml, "MatchBetweenRadio");
+			break;
+		}
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
+	}
+	widget = glade_xml_get_widget(filter_window->xml, "MsgCombo");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), filter_window->msg);
+
 }
 
 static void filter_window_update_values(filter_window_t *filter_window)
@@ -967,6 +1230,7 @@ static void filter_window_update_values(filter_window_t *filter_window)
 	GtkWidget *widget;
 	GtkTextBuffer *buffer;
 	GtkTextIter start, end;
+	int rt;
 	
 	/* First parse all selection enabled entry boxes, because the user may have typed in the value */
 	filters_select_items_parse_entry(filter_window->src_types_items);
@@ -992,6 +1256,10 @@ static void filter_window_update_values(filter_window_t *filter_window)
 	/* update executable value */
 	widget = glade_xml_get_widget(filter_window->xml, "ExeEntry");
 	filter_window->executable = g_string_assign(filter_window->executable, gtk_entry_get_text(GTK_ENTRY(widget)));
+
+	/* update command value */
+	widget = glade_xml_get_widget(filter_window->xml, "CommEntry");
+	filter_window->comm = g_string_assign(filter_window->comm, gtk_entry_get_text(GTK_ENTRY(widget)));
 	
 	/* update path value */
 	widget = glade_xml_get_widget(filter_window->xml, "PathEntry");
@@ -1006,14 +1274,27 @@ static void filter_window_update_values(filter_window_t *filter_window)
 	widget = glade_xml_get_widget(filter_window->xml, "HostEntry");
 	filter_window->host = g_string_assign(filter_window->host, gtk_entry_get_text(GTK_ENTRY(widget)));
 
+	/* update message value */
+	widget = glade_xml_get_widget(filter_window->xml, "MsgCombo");
+	filter_window->msg = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+
 	widget = glade_xml_get_widget(filter_window->xml, "NotesTextView");
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
 	gtk_text_buffer_get_start_iter(buffer, &start);
 	gtk_text_buffer_get_end_iter(buffer, &end);
 	filter_window->notes = g_string_assign(filter_window->notes, gtk_text_buffer_get_text(buffer, &start, &end, FALSE));
 		
-}
+	if (filter_window->dates) {		
+		rt = filters_window_update_tm(filter_window, TRUE, filter_window->dates->start);		
+		if (rt < 0)
+			return;
+		rt = filters_window_update_tm(filter_window, FALSE, filter_window->dates->end);
+		if (rt < 0)
+			return;
+		filter_window->dates->option = filters_window_get_tm_option(filter_window->xml);
+	}
 
+}
 
 static void filter_window_seaudit_filter_list_free(seaudit_filter_list_t *list)
 {
@@ -1105,11 +1386,23 @@ static void filter_window_clear_other_tab_values(filter_window_t *filter_window)
 	widget = glade_xml_get_widget(filter_window->xml, "ExeEntry");
 	g_assert(widget);
 	gtk_entry_set_text(GTK_ENTRY(widget), "");
+
+	/* clear command */
+	widget = glade_xml_get_widget(filter_window->xml, "CommEntry");
+	g_assert(widget);
+	gtk_entry_set_text(GTK_ENTRY(widget), "");
 	
 	/* clear path */
 	widget = glade_xml_get_widget(filter_window->xml, "PathEntry");
 	g_assert(widget);
 	gtk_entry_set_text(GTK_ENTRY(widget), "");
+
+	/* clear message */
+	widget = glade_xml_get_widget(filter_window->xml, "MsgCombo");
+	g_assert(widget);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), SEAUDIT_MSG_NONE);
+
+
 }
 
 static void filter_window_clear_context_tab_values(filter_window_t *filter_window)
@@ -1155,12 +1448,13 @@ static void filter_window_on_custom_clicked(GtkButton *button, filters_select_it
 	}
 	/* add anything from the entry to the list stores */
 	filters_select_items_parse_entry(filter_items_list);
-	filters_select_items_display(filter_items_list);
+	filters_select_items_display(filter_items_list, seaudit_app->window->window);
 }
 
 static void filter_window_on_close_button_pressed(GtkButton *button, filter_window_t *filter_window)
 {
-	filter_window_hide(filter_window);
+	if (filter_window_verify_filter_values(filter_window) == 0)
+		filter_window_hide(filter_window);
 }
 
 static gboolean filter_window_on_filter_window_destroy(GtkWidget *widget, GdkEvent *event, filter_window_t *filter_window)
@@ -1179,6 +1473,17 @@ static void filter_window_on_OtherClearButton_clicked(GtkButton *button, filter_
 	filter_window_clear_other_tab_values(filter_window);
 }
 
+static void filter_window_on_radio_toggled(GtkToggleButton *tb, gpointer user_data)
+{
+	filter_window_date_set_sensitive(tb, user_data);
+}
+
+static void filter_window_on_month_changed(GtkComboBox *cb, gpointer user_data)
+{
+	filter_window_t *fw = (filter_window_t *)user_data;	
+	filter_window_date_update_days(fw->xml);
+}
+
 static gboolean filter_window_name_entry_text_changed(GtkWidget *widget, GdkEventKey *event, filter_window_t *filter_window)
 {
 	const gchar *name;
@@ -1196,6 +1501,7 @@ static gboolean filter_window_name_entry_text_changed(GtkWidget *widget, GdkEven
 filter_window_t* filter_window_create(multifilter_window_t *parent, gint parent_index, const char *name)
 {
 	filter_window_t *filter_window;
+	time_t t;
 	
 	if (!parent || !name)
 		return NULL;
@@ -1219,10 +1525,13 @@ filter_window_t* filter_window_create(multifilter_window_t *parent, gint parent_
 	filter_window->port = g_string_new("");
 	filter_window->interface = g_string_new("");
 	filter_window->executable = g_string_new("");
+	filter_window->comm = g_string_new("");
 	filter_window->path = g_string_new("");
 	filter_window->match = g_string_new("All");
 	filter_window->notes = g_string_new("");
 	filter_window->host = g_string_new("");
+
+	filter_window->msg = SEAUDIT_MSG_NONE;
 	
 	filter_window->window = NULL;
 	filter_window->xml = NULL;
@@ -1230,6 +1539,22 @@ filter_window_t* filter_window_create(multifilter_window_t *parent, gint parent_
 	filter_window->parent = parent;
 	filter_window->parent_index = parent_index;
 
+	filter_window->dates = (filters_date_item_t *)calloc(1, sizeof(filters_date_item_t));
+	t = time(NULL);
+	filter_window->dates->start = (struct tm *)calloc(1, sizeof(struct tm));
+	filter_window->dates->start = localtime_r(&t, filter_window->dates->start);       
+	filter_window->dates->end = (struct tm *)calloc(1, sizeof(struct tm));
+	filter_window->dates->end = localtime_r(&t, filter_window->dates->end);
+	
+	/* set the hour/min/secs up so that we have one full day */
+	filter_window->dates->start->tm_hour = 0;
+	filter_window->dates->start->tm_min = 0;
+	filter_window->dates->start->tm_sec = 0;
+
+	filter_window->dates->end->tm_hour = 23;
+	filter_window->dates->end->tm_min = 59;
+	filter_window->dates->end->tm_sec = 59;
+	
 	return filter_window;
 }
 
@@ -1256,6 +1581,8 @@ void filter_window_destroy(filter_window_t *filter_window)
 		g_string_free(filter_window->interface, TRUE);
 	if (filter_window->executable)
 		g_string_free(filter_window->executable, TRUE);
+	if (filter_window->comm)
+		g_string_free(filter_window->comm, TRUE);
 	if (filter_window->path)
 		g_string_free(filter_window->path, TRUE);
 	if (filter_window->name)
@@ -1272,6 +1599,8 @@ void filter_window_destroy(filter_window_t *filter_window)
 	if (filter_window->xml != NULL)
 		g_object_unref(G_OBJECT(filter_window->xml));
 
+	free(filter_window->dates);
+
 	free(filter_window);
 }
 
@@ -1311,6 +1640,7 @@ void filter_window_hide(filter_window_t *filter_window)
 		filter_window->obj_class_items->window = NULL;
 	}
 	filter_window_update_values(filter_window);
+	
 	/* if there is an idle function for this window
 	 * then we must remove it to avoid that function
 	 * being executed after we delete the window.  This
@@ -1320,8 +1650,7 @@ void filter_window_hide(filter_window_t *filter_window)
 	gtk_widget_destroy(GTK_WIDGET(filter_window->window));
 	filter_window->window = NULL;
 }
-
-void filter_window_display(filter_window_t* filter_window)
+void filter_window_display(filter_window_t* filter_window, GtkWindow *parent)
 {
 	GladeXML *xml;
 	GtkWindow *window;
@@ -1329,7 +1658,7 @@ void filter_window_display(filter_window_t* filter_window)
 	GString *path;
 	char *dir;
 
-	if (!filter_window)
+	if (!filter_window || !parent)
 		return;
 
 	if (filter_window->window) {
@@ -1350,6 +1679,13 @@ void filter_window_display(filter_window_t* filter_window)
 	
 	window = GTK_WINDOW(glade_xml_get_widget(xml, "FilterWindow"));
 	g_assert(window);
+	/* set this window to be transient on the parent window, so that when it pops up it gets centered on it */
+	/* however to have it "appear" to be centered we have to hide and then show */
+	gtk_window_set_transient_for(window, parent);
+	gtk_window_set_position(window, GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_widget_hide(GTK_WIDGET(window));
+	gtk_window_present(window);
+
 		
 	filter_window->window = window;
 	filter_window->xml = xml;
@@ -1394,8 +1730,42 @@ void filter_window_display(filter_window_t* filter_window)
 				      G_CALLBACK(filter_window_on_OtherClearButton_clicked),
 				      filter_window);
 
+	/* rename vars, or comment to make clear for dates */
+	glade_xml_signal_connect_data(xml, "filters_on_MatchNoneRadio_toggled",
+				      G_CALLBACK(filter_window_on_radio_toggled),
+				      filter_window);
+	glade_xml_signal_connect_data(xml, "filters_on_MatchBeforeRadio_toggled",
+				      G_CALLBACK(filter_window_on_radio_toggled),
+				      filter_window);
+	glade_xml_signal_connect_data(xml, "filters_on_MatchAfterRadio_toggled",
+				      G_CALLBACK(filter_window_on_radio_toggled),
+				      filter_window);
+	glade_xml_signal_connect_data(xml, "filters_on_MatchBetweenRadio_toggled",
+				      G_CALLBACK(filter_window_on_radio_toggled),
+				      filter_window);
+
+	glade_xml_signal_connect_data(xml, "filters_on_DateEndMonth_changed",
+				      G_CALLBACK(filter_window_on_month_changed),
+				      filter_window);
+
+	glade_xml_signal_connect_data(xml, "filters_on_DateStartMonth_changed",
+				      G_CALLBACK(filter_window_on_month_changed),
+				      filter_window);
+
+	widget = glade_xml_get_widget(filter_window->xml, "MsgCombo");
+	g_assert(widget);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), msg_type_strs[SEAUDIT_MSG_NONE]);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), msg_type_strs[SEAUDIT_MSG_AVC_DENIED]);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), msg_type_strs[SEAUDIT_MSG_AVC_GRANTED]);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), filter_window->msg);
+
 	/* Restore previous values and selections for the filter dialog */
 	filter_window_set_values(filter_window);
+
+
+
+
+
 }
 
 seaudit_filter_t* filter_window_get_filter(filter_window_t *filter_window)
@@ -1408,6 +1778,7 @@ seaudit_filter_t* filter_window_get_filter(filter_window_t *filter_window)
 	GtkTextIter start, end;
 	char *text;
 	int int_val;
+	struct tm *dt_start, *dt_end;
 
 	rt = seaudit_filter_create();
 
@@ -1527,6 +1898,16 @@ seaudit_filter_t* filter_window_get_filter(filter_window_t *filter_window)
 		rt->exe_criteria = exe_criteria_create(text);
 	}
 	
+	/* check for command filter */
+	if (filter_window->window) {
+		widget = glade_xml_get_widget(filter_window->xml, "CommEntry");
+		text = (char*)gtk_entry_get_text(GTK_ENTRY(widget));
+	} else 
+		text = filter_window->comm->str;
+	if (strcmp(text, "") != 0) {
+		rt->comm_criteria = comm_criteria_create(text);
+	}
+
 	/* check for path filter */
 	if (filter_window->window) {
 		widget = glade_xml_get_widget(filter_window->xml, "PathEntry");
@@ -1536,6 +1917,36 @@ seaudit_filter_t* filter_window_get_filter(filter_window_t *filter_window)
 	if (strcmp(text, "") != 0) {
 		rt->path_criteria = path_criteria_create(text);
 	}	
+	
+
+	dt_start = (struct tm *)calloc(1, sizeof(struct tm));
+	dt_end = (struct tm *)calloc(1, sizeof(struct tm));
+
+	*dt_start = *(filter_window->dates->start);
+	*dt_end = *(filter_window->dates->end);
+	if (filter_window->window) {		
+		filters_window_update_tm(filter_window, TRUE, dt_start);
+		filters_window_update_tm(filter_window, FALSE, dt_end);
+		int_val = filters_window_get_tm_option(filter_window->xml);	       
+	} else {
+		int_val = filter_window->dates->option;
+	}
+	/* check for date time filter */
+	if (int_val != SEAUDIT_DT_OPTION_NONE) {
+		switch (int_val) {
+		case SEAUDIT_DT_OPTION_BEFORE:		
+			rt->date_time_criteria = date_time_criteria_create(dt_start, dt_end, FILTER_CRITERIA_DT_OPTION_BEFORE);
+			break;
+		case SEAUDIT_DT_OPTION_AFTER:
+			rt->date_time_criteria = date_time_criteria_create(dt_start, dt_end, FILTER_CRITERIA_DT_OPTION_AFTER);
+			break;
+		default:
+			rt->date_time_criteria = date_time_criteria_create(dt_start, dt_end, FILTER_CRITERIA_DT_OPTION_BETWEEN);
+			break;
+		}
+	}
+	free(dt_start);
+	free(dt_end);
 
 	if (filter_window->window) {
 		widget = glade_xml_get_widget(filter_window->xml, "MatchEntry");
@@ -1563,6 +1974,23 @@ seaudit_filter_t* filter_window_get_filter(filter_window_t *filter_window)
 	if (strcmp(text, "") != 0)
 		rt->host_criteria = host_criteria_create(text);
 
+	if (filter_window->window) {
+		widget = glade_xml_get_widget(filter_window->xml, "MsgCombo");
+		int_val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+	} else
+		int_val = filter_window->msg;
+	switch (int_val)
+	{
+	case SEAUDIT_MSG_AVC_DENIED:
+		rt->msg_criteria = msg_criteria_create(AVC_DENIED);
+		break;
+	case SEAUDIT_MSG_AVC_GRANTED:
+		rt->msg_criteria = msg_criteria_create(AVC_GRANTED);
+		break;
+	default:
+		break;
+	}
+
 	if (filter_window->window) {
 		widget = glade_xml_get_widget(filter_window->xml, "NotesTextView");
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
@@ -1640,7 +2068,45 @@ void filter_window_set_values_from_filter(filter_window_t *filter_window, seaudi
 		filter_window->path = g_string_assign(filter_window->path, path_criteria_get_str(filter->path_criteria));
 	if (filter->exe_criteria) 
 		filter_window->executable = g_string_assign(filter_window->executable, exe_criteria_get_str(filter->exe_criteria));
+	if (filter->comm_criteria) 
+		filter_window->comm = g_string_assign(filter_window->comm, comm_criteria_get_str(filter->comm_criteria));
 	if (filter->host_criteria)
 		filter_window->host = g_string_assign(filter_window->host, host_criteria_get_str(filter->host_criteria));
+	if (filter->msg_criteria) {
+		switch (msg_criteria_get_val(filter->msg_criteria))
+		{
+		case AVC_GRANTED:
+			filter_window->msg = SEAUDIT_MSG_AVC_GRANTED;
+			break;
+		case AVC_DENIED:
+			filter_window->msg = SEAUDIT_MSG_AVC_DENIED;
+			break;
+		default:
+			filter_window->msg = SEAUDIT_MSG_NONE;
+			break;
+		}
+	}
+	if (filter->date_time_criteria) {
+		if (!filter_window->dates)
+			filter_window->dates = (filters_date_item_t *)calloc(1, sizeof(filters_date_item_t));
+		*(filter_window->dates->start) = *(date_time_criteria_get_date(filter->date_time_criteria, TRUE));
+		*(filter_window->dates->end) = *(date_time_criteria_get_date(filter->date_time_criteria, FALSE));
+		i = date_time_criteria_get_option(filter->date_time_criteria);
+		switch(i) {
+		case FILTER_CRITERIA_DT_OPTION_BEFORE:		
+			filter_window->dates->option = SEAUDIT_DT_OPTION_BEFORE;
+			break;
+		case FILTER_CRITERIA_DT_OPTION_AFTER:
+			filter_window->dates->option = SEAUDIT_DT_OPTION_AFTER;
+			break;
+			
+		case FILTER_CRITERIA_DT_OPTION_BETWEEN:
+			filter_window->dates->option = SEAUDIT_DT_OPTION_BETWEEN;
+			break;
+		default:
+			filter_window->dates->option = SEAUDIT_DT_OPTION_NONE;
+			break;
+		}
+	}
 
 }
