@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Tresys Technology, LLC
+/* Copyright (C) 2003-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -21,7 +21,9 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 
+
 struct filters_select_items;
+struct filters_date_item;
 
 typedef struct filter_window {
 	struct filters_select_items *src_types_items;
@@ -35,11 +37,14 @@ typedef struct filter_window {
 	GString *port;
 	GString *interface;
 	GString *executable;
+	GString *comm;
 	GString *path;
 	GString *name;
 	GString *match;
 	GString *notes;
-	GString *host;
+	GString *host;	
+	struct filters_date_item *dates;
+	int msg;
 	GtkWindow *window;
 	GladeXML *xml;
 	multifilter_window_t *parent;
@@ -51,7 +56,7 @@ typedef struct filter_window {
  ***************************/
 filter_window_t* filter_window_create(multifilter_window_t *parent, gint parent_index, const char *name);
 void filter_window_destroy(filter_window_t* filter_window);
-void filter_window_display(filter_window_t* filter_window);
+void filter_window_display(filter_window_t* filter_window, GtkWindow *parent);
 void filter_window_hide(filter_window_t *filter_window);
 seaudit_filter_t* filter_window_get_filter(filter_window_t *filter_window);
 void filter_window_set_values_from_filter(filter_window_t *filter_window, seaudit_filter_t *filter);
