@@ -1,4 +1,4 @@
-/* Copyright (C) 2004-2005 Tresys Technology, LLC
+/* Copyright (C) 2004-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -471,9 +471,19 @@ static void display_browse_dialog_for_entry_box(GtkEntry *entry, const char *fil
 	GtkWidget *file_selector;
 	gint response;
 	const gchar *filename;
+	GtkWidget *window;
 
 	g_assert(entry);
 	file_selector = gtk_file_selection_new(title);
+	
+	/* get the top level widget, which of this widget is the prefer window */
+	window = GTK_WIDGET(entry);
+	while (gtk_widget_get_parent(window))
+		window = gtk_widget_get_parent(window);;
+	assert(window);
+
+	/* set the file selector window to be transient on the preference window, so that when it pops up it gets centered on it */
+	gtk_window_set_transient_for(GTK_WINDOW(file_selector), GTK_WINDOW(window));
 	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
 	if (file_path != NULL)
 		gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), gtk_entry_get_text(entry));
@@ -586,6 +596,10 @@ static void on_preference_toggled(GtkToggleButton *toggle, gpointer user_data)
 		seaudit_app->seaudit_conf.column_visibility[AVC_EXE_FIELD] = gtk_toggle_button_get_active(toggle);
 		seaudit_app->column_visibility_changed = TRUE;
 
+	} else if (!strcmp("CommandCheck", gtk_widget_get_name(GTK_WIDGET(toggle)))) {
+		seaudit_app->seaudit_conf.column_visibility[AVC_COMM_FIELD] = gtk_toggle_button_get_active(toggle);
+		seaudit_app->column_visibility_changed = TRUE;
+
 	} else if (!strcmp("PIDCheck", gtk_widget_get_name(GTK_WIDGET(toggle)))) {
 		seaudit_app->seaudit_conf.column_visibility[AVC_PID_FIELD] = gtk_toggle_button_get_active(toggle);
 		seaudit_app->column_visibility_changed = TRUE;
@@ -631,7 +645,15 @@ void on_preferences_activate(GtkWidget *widget, GdkEvent *event, gpointer callba
 	g_string_free(path, TRUE);
 	window = glade_xml_get_widget(xml, "PreferWindow");
 	g_assert(window);
+	/* set this window to be transient on the main window, so that when it pops up it gets centered on it */
+	/* however to have it "appear" to be centered on xml_new we have to hide and then show */
+	gtk_window_set_transient_for(GTK_WINDOW(window), seaudit_app->window->window);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_window_present(GTK_WINDOW(window));
 	
+	/* make the window modal */
+	gtk_window_set_modal(GTK_WINDOW(window), TRUE); 	
+
 	entry = GTK_ENTRY(glade_xml_get_widget(xml, "interval_lbl"));
 	g_assert(entry);
 	g_string_printf(interval, "%d", seaudit_app->seaudit_conf.real_time_interval);
@@ -743,6 +765,10 @@ void on_preferences_activate(GtkWidget *widget, GdkEvent *event, gpointer callba
 	g_assert(toggle);
 	gtk_toggle_button_set_active(toggle, seaudit_app->seaudit_conf.column_visibility[AVC_EXE_FIELD]);
 
+	toggle = GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "CommandCheck"));
+	g_assert(toggle);
+	gtk_toggle_button_set_active(toggle, seaudit_app->seaudit_conf.column_visibility[AVC_COMM_FIELD]);
+
 	toggle = GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "PIDCheck"));
 	g_assert(toggle);
 	gtk_toggle_button_set_active(toggle, seaudit_app->seaudit_conf.column_visibility[AVC_PID_FIELD]);
