@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Tresys Technology, LLC
+/* Copyright (C) 2003-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -96,22 +96,54 @@ void seaudit_destroy(seaudit_t *seaudit_app)
 	seaudit_app = NULL;
 }
 
-static void seaudit_change_report_menuitem_state(bool_t disable) 
-{
-	GtkWidget *report_menu;
 
-	report_menu = glade_xml_get_widget(seaudit_app->window->xml, "create_standard_report");
-	g_assert(report_menu);
+/* change the sensitive state of widget by name */
+static void seaudit_widget_update_sensitive(const char *name, bool_t disable)
+{
+	GtkWidget *widget;
+
+	if (!seaudit_app || !seaudit_app->window || !seaudit_app->window->xml)
+		return;
+	widget = glade_xml_get_widget(seaudit_app->window->xml, name);
+	g_assert(widget);
 	if (disable)
-		gtk_widget_set_sensitive(report_menu , 0); 
+		gtk_widget_set_sensitive(widget , 0); 
 	else 
-		gtk_widget_set_sensitive(report_menu , 1); 
+		gtk_widget_set_sensitive(widget , 1); 
+}
+
+/* this is just a public method to set this button */
+void seaudit_view_entire_selection_update_sensitive(bool_t disable)
+{
+	seaudit_widget_update_sensitive("view_entire_message1", disable);
+}
+
+/* change the state of all widgets that depend on the log being open */
+static void seaudit_log_menu_items_update(bool_t disable)
+{
+	seaudit_widget_update_sensitive("SaveView", disable);
+	seaudit_widget_update_sensitive("SaveAsView", disable);
+	seaudit_widget_update_sensitive("ExportLog", disable);
+	seaudit_widget_update_sensitive("export_selection1", disable);
+	seaudit_widget_update_sensitive("create_standard_report", disable);
+	seaudit_widget_update_sensitive("RealTimeButton", disable);
+	seaudit_widget_update_sensitive("ModifyView", disable);
+	seaudit_widget_update_sensitive("ModifyViewButton", disable);
+	seaudit_widget_update_sensitive("NewTab", disable);
+	seaudit_widget_update_sensitive("OpenView", disable);
+}
+
+/* change the state of all widgets that depend on the policy being open */
+static void seaudit_policy_menu_items_update(bool_t disable)
+{
+	seaudit_widget_update_sensitive("top_window_query_button", disable);
+	seaudit_widget_update_sensitive("QueryPolicy", disable);
 }
 
 void seaudit_update_status_bar(seaudit_t *seaudit)
 {	
 	char str[STR_SIZE];
-	char *ver_str = NULL;
+	char *ver_str = NULL, *tmp = NULL;
 	int num_log_msgs, num_filtered_log_msgs;
 	char old_time[TIME_SIZE], recent_time[TIME_SIZE];
 	seaudit_filtered_view_t *view;
@@ -126,12 +158,12 @@ void seaudit_update_status_bar(seaudit_t *seaudit)
 	if (seaudit->cur_policy == NULL) {
 		ver_str = "Policy Version: No policy";
 		gtk_label_set_text(v_status_bar, ver_str);
+		seaudit_policy_menu_items_update(TRUE);
 	} else {
-               	if(is_binary_policy(seaudit->cur_policy))
-                	snprintf(str, STR_SIZE, "Policy Version: %s (binary)", get_policy_version_name(seaudit->cur_policy->version));
-               	else
-                	snprintf(str, STR_SIZE, "Policy Version: %s (source)", get_policy_version_name(seaudit->cur_policy->version));
+		snprintf(str, STR_SIZE, "Policy Version: %s", (tmp = get_policy_version_type_mls_str(seaudit->cur_policy)));
+		free(tmp);
 		gtk_label_set_text(v_status_bar, str);
+		seaudit_policy_menu_items_update(FALSE);
 	}
 
 	if (seaudit->cur_log == NULL) {
@@ -139,6 +171,8 @@ void seaudit_update_status_bar(seaudit_t *seaudit)
 		gtk_label_set_text(l_status_bar, str);
 		snprintf(str, STR_SIZE, "Dates: No log");
 		gtk_label_set_text(d_status_bar, str);
+		seaudit_log_menu_items_update(TRUE);
+		seaudit_view_entire_selection_update_sensitive(TRUE);
 	} else {
 		view = seaudit_window_get_current_view(seaudit->window);
 		if (view)
@@ -160,7 +194,10 @@ void seaudit_update_status_bar(seaudit_t *seaudit)
 			snprintf(str, STR_SIZE, "Dates: No messages");
 			gtk_label_set_text(d_status_bar, str);
 		}
+		seaudit_log_menu_items_update(FALSE);
+		seaudit_view_entire_selection_update_sensitive(TRUE);
 	}
+
 }
 
 int seaudit_open_policy(seaudit_t *seaudit, const char *filename)
@@ -319,7 +356,6 @@ int seaudit_open_log_file(seaudit_t *seaudit, const char *filename)
 	seaudit_set_recent_logs_submenu(&(seaudit->seaudit_conf));
 	save_seaudit_conf_file(&(seaudit->seaudit_conf));
 	log_load_signal_emit();
-	seaudit_change_report_menuitem_state(FALSE);
 	clear_wait_cursor(GTK_WIDGET(seaudit->window->window));
 	return 0;
 }
@@ -409,6 +445,8 @@ void seaudit_save_log_file(bool_t selected_only)
 	} else {
 		file_selector = gtk_file_selection_new("Export View");
 	}
+	/* set up transient so that it will center on the main window */
+	gtk_window_set_transient_for(GTK_WINDOW(file_selector), seaudit_app->window->window);
 	gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), (*(seaudit_app->audit_log_file)).str);
 
 	g_signal_connect(GTK_OBJECT(file_selector), "response", 
@@ -868,7 +906,7 @@ int main(int argc, char **argv)
 	seaudit_app = seaudit_init();
 	if (!seaudit_app)
 		exit(1);
-	seaudit_change_report_menuitem_state(TRUE);
+
 	seaudit_set_recent_policys_submenu(&(seaudit_app->seaudit_conf));
 	seaudit_set_recent_logs_submenu(&(seaudit_app->seaudit_conf));
 	seaudit_set_real_time_log_button_state(seaudit_app->seaudit_conf.real_time_log);
@@ -919,7 +957,6 @@ int main(int argc, char **argv)
 	return 0;
 }
 
-
 /*
  * glade autoconnected callbacks for main window
  *
@@ -961,7 +998,10 @@ void seaudit_on_PolicyFileOpen_activate(GtkWidget *widget, GdkEvent *event, gpoi
 	gint response;
 	const gchar *filename;
 
-	file_selector = gtk_file_selection_new("Open Policy");
+	file_selector = gtk_file_selection_new("Open Policy");  
+	/* set this window to be transient window, so that when it pops up it gets centered on it */
+	gtk_window_set_transient_for(GTK_WINDOW(file_selector), seaudit_app->window->window);
+	
 	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
 	if (seaudit_app->seaudit_conf.default_policy_file != NULL)
 		gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), seaudit_app->seaudit_conf.default_policy_file);
@@ -992,7 +1032,9 @@ void seaudit_on_LogFileOpen_activate(GtkWidget *widget, GdkEvent *event, gpointe
 	const gchar *filename;
 
 	file_selector = gtk_file_selection_new("Open Log");
-	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
+	/* set this window to be transient window, so that when it pops up it gets centered on it */
+	gtk_window_set_transient_for(GTK_WINDOW(file_selector), seaudit_app->window->window);
+		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
 	if (seaudit_app->seaudit_conf.default_log_file != NULL)
 		gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), seaudit_app->seaudit_conf.default_log_file);
 
@@ -1143,7 +1185,7 @@ void seaudit_on_about_seaudit_activate(GtkWidget *widget, GdkEvent *event, gpoin
 	
 	str = g_string_new("");
 	g_string_assign(str, "Audit Log Analysis Tool for Security \nEnhanced Linux");
-        g_string_append(str, "\n\nCopyright (c) 2003-2005\nTresys Technology, LLC\nwww.tresys.com/selinux");
+        g_string_append(str, "\n\nCopyright (c) 2003-2006\nTresys Technology, LLC\nwww.tresys.com/selinux");
 	g_string_append(str, "\n\nGUI version ");
 	g_string_append(str, SEAUDIT_GUI_VERSION_STRING);
 	g_string_append(str, "\nlibseaudit version ");
@@ -1174,6 +1216,10 @@ void seaudit_on_help_activate(GtkWidget *widget, GdkEvent *event, gpointer callb
 	char *dir;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	/* set this window to be transient to main window and center it on it */
+	gtk_window_set_transient_for(GTK_WINDOW(window), seaudit_app->window->window);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
+
 	scroll = gtk_scrolled_window_new(NULL, NULL);
 	text_view = gtk_text_view_new();
 	gtk_window_set_title(GTK_WINDOW(window), "seAudit Help");
@@ -1220,7 +1266,7 @@ void seaudit_on_filter_log_button_clicked(GtkWidget *widget, GdkEvent *event, gp
 	}
 
 	view = seaudit_window_get_current_view(seaudit_app->window);
-	seaudit_filtered_view_display(view);	
+	seaudit_filtered_view_display(view, seaudit_app->window->window);	
 	return;
 }
 
