@@ -1,4 +1,4 @@
-/* Copyright (C) 2004-2005 Tresys Technology, LLC
+/* Copyright (C) 2004-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -56,13 +56,18 @@ void clear_wait_cursor(GtkWidget *widget)
 	g_idle_add(&pointer_reset, widget);
 }
 
-GString* get_filename_from_user(const char *title, const gchar *startfilename)
+GString* get_filename_from_user(const char *title, const gchar *startfilename, GtkWindow *window, gboolean overwrite)
 {
-	GtkWidget *file_selector;
+	GtkWidget *file_selector = NULL;
 	gint response;
-	GString *filename;
+	GString *filename = NULL;
+	GString *overwrite_warning = NULL;
 
 	file_selector = gtk_file_selection_new(title);
+	if (window)
+		/* set this window to be transient window, so that when it pops up it gets centered on it */
+		gtk_window_set_transient_for(GTK_WINDOW(file_selector), window);
+	
 	if (startfilename)
 		gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selector), startfilename);
 
@@ -79,7 +84,14 @@ GString* get_filename_from_user(const char *title, const gchar *startfilename)
 		 * under the Files list. */
 		if (g_file_test(filename->str, G_FILE_TEST_IS_DIR))
 			gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), filename->str);
-		else 
+		else if (overwrite && g_file_test(filename->str, G_FILE_TEST_EXISTS)) {
+			overwrite_warning = g_string_new("");
+			g_string_printf(overwrite_warning, "Overwrite File:\n%s?", filename->str);
+			response = get_user_response_to_message(GTK_WINDOW(gtk_widget_get_toplevel(file_selector)), overwrite_warning->str);	
+			g_string_free(overwrite_warning, 1);
+			if (response == GTK_RESPONSE_YES)
+				break;				
+		} else
 			break;
 	}
 	gtk_widget_destroy(file_selector);
