@@ -1,4 +1,4 @@
-/* Copyright (C) 2004-2005 Tresys Technology, LLC
+/* Copyright (C) 2004-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -16,7 +16,7 @@ void get_dialog_response(GtkDialog *dialog, gint id, gpointer response);
 void show_wait_cursor(GtkWidget *widget);
 void clear_wait_cursor(GtkWidget *widget);
 void message_display(GtkWindow *parent, GtkMessageType msg_type, const char *msg);
-GString* get_filename_from_user(const char *title, const gchar *startfilename);
+GString* get_filename_from_user(const char *title, const gchar *startfilename, GtkWindow *window, gboolean overwrite);
 gint get_user_response_to_message(GtkWindow *window, const char *message);
 
 #endif
