@@ -6,6 +6,9 @@
  *
  */
 
+#ifndef TYPES_WO_ALLOW
+#define TYPES_WO_ALLOW
+
 #include "sechecker.h"
 #include "policy.h"
 
@@ -27,3 +30,4 @@ int types_wo_allow_get_list(sechk_module_t *mod, int **array, int*size);
  * the private data storage structure for this module */
 types_wo_allow_data_t *types_wo_allow_data_new(void);
 
+#endif
