@@ -1,4 +1,4 @@
- /* Copyright (C) 2005 Tresys Technology, LLC
+ /* Copyright (C) 2005-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
  
 /* 
@@ -19,6 +19,9 @@
 #include <util.h>
 #include <dirent.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #ifdef LIBSEFS
 #include "file_contexts.h"
 #endif
@@ -542,7 +545,7 @@ int sechk_lib_check_module_dependencies(sechk_lib_t *lib)
 
 int sechk_lib_check_module_requirements(sechk_lib_t *lib)
 {
-	int i;
+	int i, retv = 0;
 	bool_t test = TRUE;
 	sechk_name_value_t *nv = NULL;
 
@@ -561,15 +564,16 @@ int sechk_lib_check_module_requirements(sechk_lib_t *lib)
 					
 				} else {
 					/* otherwise we just disable this module and keep testing */
-					printf("Error: requirements not met for %s\n", lib->modules[i].name);					
+					printf("Error: requirements not met for %s\n", lib->modules[i].name);	
 					lib->module_selection[i] = FALSE;
+					retv = -1;
 					break;
 				}
 			}
 			nv = nv->next;
 		}
 	}
-	return 0;
+	return retv;
 }
 
 int sechk_lib_init_modules(sechk_lib_t *lib)
@@ -679,6 +683,7 @@ int sechk_lib_print_modules_report(sechk_lib_t *lib)
 bool_t sechk_lib_check_requirement(sechk_name_value_t *req, sechk_lib_t *lib)
 {
 	int pol_ver = POL_VER_UNKNOWN;
+	struct stat stat_buf;
 
 	if (!req) {
 		fprintf(stderr, "Error: invalid requirement\n");
@@ -735,7 +740,8 @@ bool_t sechk_lib_check_requirement(sechk_name_value_t *req, sechk_lib_t *lib)
 				fprintf(stderr, "Error: module requires newer policy version\n");
 			return FALSE;
 		}
-	} else if (!strcmp(req->name, SECHK_PARSE_REQUIRE_SELINUX)) {
+	} 
+	else if (!strcmp(req->name, SECHK_PARSE_REQUIRE_SELINUX)) {
 #ifdef LIBSELINUX
 		if (!is_selinux_enabled()) {
 			/* as long as we're not in quiet mode print output */
@@ -750,7 +756,7 @@ bool_t sechk_lib_check_requirement(sechk_name_value_t *req, sechk_lib_t *lib)
 		return FALSE;
 #endif
 	} else if (!strcmp(req->name, SECHK_PARSE_REQUIRE_MLS_POLICY)) {
-		if (lib->policy->version != POL_VER_19MLS) {
+		if (!lib->policy->mls) {
 			/* as long as we're not in quiet mode print output */
 			if (lib->outputformat & ~(SECHK_OUT_QUIET))
 				fprintf(stderr, "Error: module requires MLS policy\n");
@@ -770,7 +776,17 @@ bool_t sechk_lib_check_requirement(sechk_name_value_t *req, sechk_lib_t *lib)
 			fprintf(stderr, "Error: module requires selinux system, but SEChecker was not built to support system checks\n");
 		return FALSE;
 #endif
-	} else {
+	} else if (!strcmp(req->name, SECHK_PARSE_REQUIRE_DEF_CTX)) {
+		if (stat(selinux_default_context_path(), &stat_buf) < 0) {
+			/* as long as we're not in quiet mode print output */
+			if (lib->outputformat & ~(SECHK_OUT_QUIET))
+				fprintf(stderr, "Error: module requires a default contexts file\n");
+			return FALSE;
+		}
+
+		return TRUE;
+	}
+	else {
 		/* as long as we're not in quiet mode print output */
 		if (lib->outputformat & ~(SECHK_OUT_QUIET))			
 			fprintf(stderr, "Error: unrecognized requirement\n");
