@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005 Tresys Technology, LLC
+ *  Copyright (C) 2003-2006 Tresys Technology, LLC
  *  see file 'COPYING' for use and warranty information
  *
  */
@@ -13,7 +13,7 @@
  */
 
 #include <fsdata.h>
-
+#include "fshash.h"
 /* SE Linux includes*/
 #include <selinux/selinux.h>
 #include <selinux/context.h>
@@ -48,13 +48,16 @@
 #define FINDCON_VERSION_NUM "UNKNOWN"
 #endif
 
-#define COPYRIGHT_INFO "Copyright (C) 2003-2005 Tresys Technology, LLC"
+#define COPYRIGHT_INFO "Copyright (C) 2003-2006 Tresys Technology, LLC"
 
+#ifndef DEBUG
 #define DEBUG 0
+#endif
 #define MAX_INPUT_SIZE 1024
 #define NFTW_FLAGS FTW_MOUNT | FTW_PHYS
 #define NFTW_DEPTH 1024
 
+#define SEFS_BIND_HASH_SIZE 50
 
 /* Data Structures */
 
@@ -62,6 +65,7 @@ typedef struct replcon_context {
 	char *user;
 	char *role;
 	char *type;
+	char *mls;
 } replcon_context_t;
 
 typedef struct replcon_context_pair {
@@ -70,6 +74,7 @@ typedef struct replcon_context_pair {
 } replcon_context_pair_t;
 
 typedef struct replcon_info {
+	unsigned char use_raw;          /* whether to use libselinux raw functions */
 	unsigned char recursive;
 	unsigned char verbose;
 	unsigned char quiet;
@@ -92,8 +97,18 @@ typedef struct replcon_info {
 replcon_info_t replcon_info;
 char **mounts;
 unsigned int num_mounts;
+sefs_hash_t *hashtab;
+
+/* use weak bindings to allow runtime checking for raw fcns */
+/* the raw functions allow access to the raw context not the context
+   returned by the translation library */
+extern int lgetfilecon_raw(const char *, security_context_t *)
+	__attribute__((weak));
+extern int lsetfilecon_raw(const char *, security_context_t)
+	__attribute__((weak));
 
 static struct option const longopts[] = {
+        {"raw", no_argument, NULL, 'a'},
 	{"recursive", no_argument, NULL, 'r'},
 	{"object", required_argument, NULL, 'o'},
 	{"context", required_argument, NULL, 'c'},
@@ -137,6 +152,10 @@ replcon_context_free(replcon_context_t *context)
 		free(context->type);
 		context->type = NULL;
 	}
+	if (context->mls != NULL) {
+		free(context->mls);
+		context->mls = NULL;
+	}
 }
 
 /*
@@ -157,67 +176,130 @@ replcon_context_destroy(replcon_context_t * context)
  *
  * Creates a new replcon_context_t object using the specified string
  *
- * NOTE: Only checks the string for the format "xxx:xxx:xxx".
+ * NOTE: Checks the string for the format "xxx:xxx:xxx[:xxx]".  Will create
+ *       a 4 part context if mls is in use.
  *       Use replcon_is_valid_context_format for more robust
- *	 constext string validation.
+ *	 context string validation.
  */
 replcon_context_t *
 replcon_context_create(const char *context_str)
 {
 	replcon_context_t *context = NULL;
 	char **parts = NULL;
-	char *tokens = NULL, *tokens_orig = NULL;
-	int i = 0;
+	char *tokens = NULL;
+	char *tokens_orig = NULL;
+	const char *str = NULL;
+	int i = 0, context_size = is_selinux_mls_enabled() ? 4 : 3;
+	int num_colon = 0;
 
 	assert(context_str != NULL);
+	str = context_str;
+
+	while(str[i]) {
+		if (str[i] == ':')
+			num_colon++;
+		i++;
+	}
+
 	if ((context = malloc(sizeof (replcon_context_t))) == NULL) {
 		fprintf(stderr, "Out of memory.\n");
 		goto err;
 	}
-	if ((parts = malloc(3 * sizeof(char*))) == NULL) {
+	memset(context, 0, sizeof(replcon_context_t));
+
+	if ((parts = malloc(context_size * sizeof(char*))) == NULL) {
 		fprintf(stderr, "Out of memory.\n");
 		goto err;
 	}
-
-	if ((tokens_orig = tokens = strdup(context_str)) == NULL) {
+	memset(parts, 0, sizeof(char *) * context_size);
+	
+	tokens_orig = tokens = strdup(context_str);
+	if (tokens == NULL) {
 		fprintf(stderr, "Out of memory.\n");
 		goto err;
 	}
 	
-        while (i < 3) {
-        	if ((parts[i] = strsep(&tokens, ":")) == NULL) {
-        		fprintf(stderr, "Invalid context format.\n");
+	i = 0;
+	while (i < 3 && (parts[i] = strsep(&tokens, ":")) != NULL) {
+		i++;
+	}
+	
+	if (!parts[0]) {
+		if ((context->user = strdup("")) == NULL) {
+			fprintf(stderr, "Out of memory.\n");
 			goto err;
 		}
-       	       	i++;
-        }
-        
-	if ((context->user = strdup(parts[0])) == NULL) {
+	} else if ((context->user = strdup(parts[0])) == NULL) {
 		fprintf(stderr, "Out of memory.\n");
 		goto err;
 	}
-	if ((context->role = strdup(parts[1])) == NULL) {
-		fprintf(stderr, "Out of memory.\n");
-		goto err;
-	}
-	if ((context->type = strdup(parts[2])) == NULL) {
+	if (!parts[1]) {
+		if ((context->role = strdup("")) == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			goto err;
+		}
+	} else if ((context->role = strdup(parts[1])) == NULL) {
 		fprintf(stderr, "Out of memory.\n");
 		goto err;
 	}
 	
-	free(tokens_orig);
-	free(parts);
+	if (!parts[2]) {
+		if ((context->type = strdup("")) == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			goto err;
+		}
+	} else if ((context->type = strdup(parts[2])) == NULL) {
+		fprintf(stderr, "Out of memory.\n");
+		goto err;
+	}
+	
+	/* if selinux is enabled then tokens should point to whatever is left 
+	   of the original string after the first 3 parts of the context */
+	if (is_selinux_mls_enabled()) {
+		if (tokens) {
+			if ((context->mls = strdup(tokens)) == NULL) {
+				fprintf(stderr, "Out of memory.\n");
+				goto err;
+			}
+		} else {
+			if ((context->mls = strdup("")) == NULL) {
+				fprintf(stderr, "Out of memory.\n");
+				goto err;
+			}
+		}
+	}
+	if (parts) free(parts);
+	if (tokens_orig) free(tokens_orig);
 
 	return context;
 
 	err:
-	  if (tokens_orig) free (tokens_orig);
 	  if (parts) free (parts);
+	  if (tokens_orig) free(tokens_orig);
 	  replcon_context_destroy(context);
 	  fprintf(stderr, "Could not create file context from %s...\n", context_str);
 	  return NULL;
 }
 
+/*
+ * replcon_context_mls_set
+ *
+ * Sets the mls member of a replcon_context_t object using the specified argument
+ */
+int replcon_context_mls_set(replcon_context_t *context, const char *mls)
+{
+	assert(context != NULL);
+	if (context->mls != NULL) {
+		free(context->mls);
+		context->mls = NULL;
+	}
+
+	if ((context->mls = strdup(mls)) == NULL)
+		return -1;
+	
+	return 0;
+}
+
 /*
  * replcon_context_user_set
  *
@@ -268,16 +350,15 @@ int replcon_context_type_set(replcon_context_t *context, const char *type)
 		free(context->type);
 		context->type = NULL;
 	}
-
 	if ((context->type = strdup(type)) == NULL)
 		return -1;
-
 	return 0;
 }
 
 /*
  * get_security_context
  *
+ *
  * Assembles a security_context_t from the information in the replcon context
  */
 security_context_t
@@ -286,18 +367,33 @@ get_security_context(const replcon_context_t *context)
 	security_context_t sec_con;
 	
 	assert(context != NULL);
-	if ((sec_con = malloc(strlen(context->user) +
-			      strlen(context->role) +
-			      strlen(context->type) +
-			      3)) == NULL)
-		return NULL;
-
+	
+	if (is_selinux_mls_enabled()) {
+		if ((sec_con = (security_context_t)calloc(1,
+							  (strlen(context->user) + 
+							   strlen(context->role) + 
+							   strlen(context->type) +
+							   strlen(context->mls) + 
+							   4))) == NULL)
+			return NULL;
+	} else {
+		if ((sec_con = (security_context_t)calloc(1,
+							  (strlen(context->user) + 
+							   strlen(context->role) + 
+							   strlen(context->type) + 
+							   3))) == NULL)
+			return NULL;
+	}
+	
 	strcpy(sec_con, (context->user));
 	strcat(sec_con, ":");
 	strcat(sec_con, (context->role));
 	strcat(sec_con, ":");
 	strcat(sec_con, (context->type));
-
+	if (is_selinux_mls_enabled()) {
+		strcat(sec_con, ":");
+		strcat(sec_con, (context->mls));
+	}
 	return sec_con;
 }
 
@@ -334,6 +430,7 @@ replcon_usage(const char *program_name, int brief)
 	printf("  -c,  --context=CONTEXT  Specify context to search for, see below.\n");
 	printf("  -o,  --object=OBJECT    Restrict search to the specified object class.\n");
 #endif
+	printf("  --raw                   Use raw contexts.\n");
 	printf("  -r,  --recursive        Recurse through directories.\n");
 	printf("  -s,  --stdin            Read FILENAMES from standard input.\n");
 	printf("  -q,  --quiet            Suppress progress output.\n");
@@ -341,13 +438,64 @@ replcon_usage(const char *program_name, int brief)
 	printf("  -v,  --version          Display version information and exit.\n");
 	printf("  -h,  --help             Display this help and exit.\n");
 	printf("\n");
-	printf("A context may be specified as a colon separated list of user, role, and type\n");
-	printf("as follows - user_u:object_r:user_t. The tool will automatically match a user,\n");
-	printf("role, or type that is not specified, with any other user, role, or type.\n");
-	printf("For example ::user_t specifies any context that has user_t as the type.\n");
+	if(is_selinux_mls_enabled()) {
+		printf("A context may be specified as a colon separated list of user, role, type, and\n");
+		printf("mls security range as follows - user_u:object_r:user_t:s0.  A single colon can\n");
+                printf("be used to match any context, so to find all contexts you only need to type : .\n");
+		printf("The tool will automatically match a user, role, type, or range that is not \n");
+		printf("specified, with any other user, role, type, or range.  The normal matching is \n");
+		printf("done using the translation library if it is enabled.  If you want the tool to \n");
+		printf("match raw contexts please use --raw.\n");
+	} else {
+		printf("A context may be specified as a colon separated list of user, role, and type\n");
+		printf("as follows - user_u:object_r:user_t. A single colon can be used to match any \n");
+		printf("context, so to find all contexts you only need to type : .  The tool will \n");
+		printf("automatically match a user, role, or type that is not specified, with any other\n");
+		printf("user, role, or type.\n");
+	}
+#ifdef FINDCON
+	if(is_selinux_mls_enabled()) {
+		printf("Search examples:\n");
+		printf("    findcon -c : .\n");
+		printf("        Find every context in the current directory\n");
+		printf("    findcon -c user_u: .\n");
+		printf("        Find every context that contains user_u in the current directory\n");
+		printf("    findcon -c :::s0 .\n");
+		printf("        Find every context that contains MLS range s0 in the current directory\n");
+	} else {
+		printf("Search examples:\n");
+		printf("    findcon -c : .\n");
+		printf("        Find every context in the current directory\n");
+		printf("    findcon -c :role_r .\n");
+		printf("        Find every context that contains role_r in the current directory\n");
+		printf("    findcon -c user_u: .\n");
+		printf("        Find every context that contains user_u in the current directory\n");
+	}
+#else
+	if(is_selinux_mls_enabled()) {
+		printf("Replacement examples:\n");
+		printf("    replcon -c : ::type_t .\n");
+		printf("        Replace every context in the current directory with type type_t\n");
+		printf("    replcon -c user_u: :role_r .\n");
+		printf("        Replace every context that contains user_u in the current directory \n");
+		printf("        with role role_r\n");
+		printf("    replcon -c ::type_t:s0 :::s0:c0\n");
+		printf("        Replace every context that contains type type_t and MLS range s0 in the\n");
+		printf("        current directory with MLS range s0:c0\n");
+	} else {
+		printf("Replacement examples:\n");
+		printf("    replcon -c : ::type_t .\n");
+		printf("        Replace every context in the current directory with type type_t\n");
+		printf("    replcon -c user_u: :role_r .\n");
+		printf("        Replace every context that contains user_u in the current directory with\n");
+		printf("        role role_r\n");
+		printf("    replcon -c :role_r :newrole_r .\n");
+		printf("        Replace every context that contains role_r in the current directory with\n");
+		printf("        newrole_r\n");
+	}
+#endif
 	printf("\nThe special string 'unlabeled' can be provided to the -c option in order\n");
 	printf("to find or replace files that have no label.\n\n");
-	
 	printf("Valid OBJECT classes to specify include: \n");
 	array = sefs_get_valid_object_classes(&size);
 	sefs_double_array_print(array,size);
@@ -364,6 +512,7 @@ replcon_usage(const char *program_name, int brief)
 void replcon_info_init(replcon_info_t *info)
 {
 	assert(info != NULL);
+	info->use_raw = 0;
 	info->recursive = 0;
 	info->quiet = 0;
 	info->verbose = 0;
@@ -466,14 +615,17 @@ int replcon_is_valid_context_format(const char *context_str)
 		if (context_str[i] == ':')
 			count++;
 	}
-
-	if (count == 2)
-		return TRUE;
-	else
-		return FALSE;
+	
+	if (is_selinux_mls_enabled()){		
+		if (count > 5)
+			return FALSE;
+	} else {
+		if (count > 2)
+			return FALSE;
+	}
+	 return TRUE;
 }
 
-
 /*
  * replcon_info_add_object_class
  *
@@ -487,6 +639,8 @@ int replcon_info_add_object_class(replcon_info_t *info, const char *str)
 	assert(info != NULL);
 	class_id = sefs_is_valid_object_class(str);
 	switch(class_id) {
+	case -1:
+		break;
 	case 0:
 		class_id = SEFS_NORM_FILE;
 		break;
@@ -568,9 +722,12 @@ int replcon_info_add_context_pair(replcon_info_t *info, const char *old, const c
 		goto err;
 	}
 
-	if (!strcasecmp("unlabeled", old))
-		context = replcon_context_create("!:!:!");
-	else
+	if (!strcasecmp("unlabeled", old)) {
+		if (is_selinux_mls_enabled()) 
+			context = replcon_context_create("!:!:!:!");
+		else 
+			context = replcon_context_create("!:!:!");		
+	} else
 		context = replcon_context_create(old);
 	if (context != NULL)
 		info->pairs[info->num_pairs].old_context = (*context);
@@ -582,7 +739,6 @@ int replcon_info_add_context_pair(replcon_info_t *info, const char *old, const c
 	/* we use free here, because calling replcon_context_destroy would
 	 * blow away the strings we just saved in info->pairs */
 	free(context);
-
 	if ((context = replcon_context_create(new)) != NULL)
 		info->pairs[info->num_pairs].new_context = (*context);
 	else {
@@ -620,9 +776,12 @@ int replcon_info_add_context(replcon_info_t *info, const char *con)
 		goto err;
 	}
 
-	if (!strcasecmp("unlabeled", con))
-		context = replcon_context_create("!:!:!");
-	else
+	if (!strcasecmp("unlabeled", con)) {
+		if (is_selinux_mls_enabled()) 
+			context = replcon_context_create("!:!:!:!");
+		else 
+			context = replcon_context_create("!:!:!");
+	} else
 		context = replcon_context_create(con);
 	if (context) {
 		info->contexts =
@@ -688,7 +847,7 @@ int replcon_info_add_filename(replcon_info_t *info, const char *file)
 unsigned char
 replcon_context_equal(const replcon_context_t *context, const replcon_context_t *patterns)
 {
-	unsigned char user_match, role_match, type_match;
+	unsigned char user_match, role_match, type_match, mls_match;
 
 	assert((context != NULL) && (patterns != NULL));
 	user_match =
@@ -703,8 +862,19 @@ replcon_context_equal(const replcon_context_t *context, const replcon_context_t
 	    ((fnmatch(patterns->type, context->type, 0) == 0) ||
 	     (strcmp(context->type, "") == 0) ||
 	     (strcmp(patterns->type, "") == 0));
+	/* set mls_match to return true, if we have mls and no match
+	   it will be set to FNM_NOMATCH 
+	   If pattern->mls is "" then we just match everything
+	   If pattern->mls and context->mls match return true
+	*/
+	mls_match = 1;
+	if (is_selinux_mls_enabled()) {
+		mls_match = ((strcmp(patterns->mls, "") == 0) ||
+			     (fnmatch(patterns->mls, context->mls, 0) == 0)
+			);
+	} 
+	return (user_match && role_match && type_match && mls_match);
 
-	return (user_match && role_match && type_match);
 }
 
 #ifndef FINDCON
@@ -714,12 +884,14 @@ replcon_context_equal(const replcon_context_t *context, const replcon_context_t
  * Change the context of the file, as long as it meets the specifications in replcon_info
  *
  * The caller must pass a valid filename and statptr
+ * If the caller wants to use raw, the passed in context will try to match the raw context
+ * of the file not the translated context returned by libsetrans
  */
 int
 replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
 			     int fileflags, struct FTW *pfwt)
 {
-	int file_class, i;
+	int file_class, i, ret;
 	unsigned char match = FALSE;
 	replcon_context_t *replacement_con = NULL, *original_con = NULL, *new_con = NULL;
 	security_context_t old_file_con, new_file_con = NULL;
@@ -730,23 +902,35 @@ replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
 	file_class = sefs_get_file_class(statptr);
 	if (!replcon_info_has_object_class(info, file_class))
 		return 0;
-
-	if (lgetfilecon(filename, &old_file_con) <= 0) {
+	if (lgetfilecon_raw && replcon_info.use_raw) 
+		ret = lgetfilecon_raw(filename, &old_file_con);
+	else
+		ret = lgetfilecon(filename, &old_file_con);
+	if (ret <= 0) {
 		if (errno == ENODATA) {
-			if ((old_file_con = strdup("!:!:!")) == NULL)
-				goto err;
+			if (is_selinux_mls_enabled()) {
+				if ((old_file_con = strdup("!:!:!:!")) == NULL)
+					goto err;
+			} else {
+				if ((old_file_con = strdup("!:!:!")) == NULL)
+					goto err;
+			}
 		} else {
 			fprintf(stderr, "Unable to get file context for %s, skipping...\n", filename);
 			return 0;
 		}
 	}
-
 	if ((original_con = replcon_context_create(old_file_con)) == NULL)
 		goto err;
 
-	if ((replacement_con = replcon_context_create("::")) == NULL)
-		goto err;
-
+	if (is_selinux_mls_enabled()) {
+		if ((replacement_con = replcon_context_create(":::")) == NULL)
+			goto err;
+	} else {
+		if ((replacement_con = replcon_context_create("::")) == NULL)
+			goto err;
+	}
+       
 	for (i = 0; i < info->num_pairs; i++) {
 		if (replcon_context_equal(original_con, &(info->pairs[i].old_context))) {
 			match = TRUE;
@@ -757,10 +941,20 @@ replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
 				replcon_context_role_set(replacement_con, new_con->role);
 			if (strcmp(new_con->type, "") != 0)
 				replcon_context_type_set(replacement_con, new_con->type);
+			if (is_selinux_mls_enabled()) {
+				if (strcmp(new_con->mls, "") != 0)
+					replcon_context_mls_set(replacement_con, new_con->mls);
+			}
 		}
 	}
 
 	if (match) {
+		/* check to see if this is a bind mount */
+		if (sefs_hash_find(hashtab, filename) == 1) {
+			if (!info->quiet)
+				fprintf(stdout, "Did not replace context for bind mount: %-40s\n", filename);
+			goto exit;
+		}
 		/* If the new context was not spcified completely, fill in the blanks */
 		if (strcmp(replacement_con->user, "") == 0)
 			replcon_context_user_set(replacement_con, original_con->user);
@@ -768,18 +962,24 @@ replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
 			replcon_context_role_set(replacement_con, original_con->role);
 		if (strcmp(replacement_con->type, "") == 0)
 			replcon_context_type_set(replacement_con, original_con->type);
-
+		if (is_selinux_mls_enabled()) {
+			if (strcmp(replacement_con->mls, "") == 0)
+				replcon_context_mls_set(replacement_con, original_con->mls);
+		}
 		new_file_con = get_security_context(replacement_con);
 		if (new_file_con == NULL) {
 			fprintf(stderr, "Unable to create new file security context.");	
 			goto err;
 		}
-		if (lsetfilecon(filename, new_file_con) != 0) {
+		if (lsetfilecon_raw && replcon_info.use_raw) 
+			ret = lsetfilecon_raw(filename, new_file_con);
+		else
+			ret = lsetfilecon(filename, new_file_con);
+		if (ret != 0) {
 			fprintf(stderr, "Error setting context %s for file %s:\n", new_file_con, filename);
 			perror("  lsetfilecon");
 			goto err;
 		}
-
 		if (!info->quiet) {
 			if (info->verbose)
 				fprintf(stdout,
@@ -790,13 +990,14 @@ replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
 		}
 		freecon(new_file_con);
 	}
-
+       
+exit:
 	replcon_context_destroy(original_con);
 	replcon_context_destroy(replacement_con);
 	freecon(old_file_con);
 	return 0;
 
-	err:
+err:
 	if (original_con) replcon_context_destroy(original_con);
 	if (replacement_con) replcon_context_destroy(replacement_con);
 	if (old_file_con) freecon(old_file_con);
@@ -810,12 +1011,14 @@ replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
  * findcon
  *
  * The caller must pass a valid filename and statptr
+ * If the caller wants to use raw, the passed in context will try to match the raw context
+ * of the file not the translated context returned by libsetrans
  */
 int
 findcon(const char *filename, const struct stat64 *statptr,
 			     int fileflags, struct FTW *pfwt)
 {
-	int file_class, i;
+	int file_class, i, ret;
 	replcon_context_t *original_con = NULL;
 	security_context_t file_con;
 	
@@ -824,10 +1027,20 @@ findcon(const char *filename, const struct stat64 *statptr,
 	if (!replcon_info_has_object_class(&replcon_info, file_class))
 		return 0;
 
-	if (lgetfilecon(filename, &file_con) <= 0) {
+	if (lgetfilecon_raw && replcon_info.use_raw) 
+		ret = lgetfilecon_raw(filename, &file_con);
+	else
+		ret = lgetfilecon(filename, &file_con);
+
+	if (ret <= 0) {
 		if (errno == ENODATA) {
-			if ((file_con = strdup("!:!:!")) == NULL)
-				goto err;
+			if (is_selinux_mls_enabled()) {
+				if ((file_con = strdup("!:!:!:!")) == NULL)
+					goto err;
+			} else {				
+				if ((file_con = strdup("!:!:!")) == NULL)
+					goto err;
+			}
 		} else {
 			fprintf(stderr, "Unable to get file context for %s\n, skipping...", filename);
 			return 0;
@@ -987,6 +1200,9 @@ replcon_parse_command_line(int argc, char **argv)
 			}
 #endif
 			break;
+		case 'a':
+			replcon_info.use_raw = TRUE;
+			break;
 		case 'r':	/* recursive directory parsing */
 			replcon_info.recursive = TRUE;
 			break;
@@ -1082,11 +1298,18 @@ main(int argc, char **argv)
 #else
 	rw = 1;
 #endif
-
-	if (find_mount_points("/", &mounts, &num_mounts, rw)) {
+	/* initialize the hash used for bind mounts */
+	hashtab = sefs_hash_new(SEFS_BIND_HASH_SIZE);
+	if (!hashtab)
+		goto err;
+	if (find_mount_points("/", &mounts, &num_mounts, hashtab, rw)) {
 		fprintf(stderr, "Could not enumerate mountpoints.\n");
 		goto err;
 	}
+	/* check to see if we are using raw and user did not ask */
+	if (replcon_info.use_raw == FALSE && lgetfilecon_raw == NULL) {
+		printf("Note: System only contains raw contexts\n");
+	}
 
 	if (replcon_info.stdin) {
 		while (fgets(stream_input, (MAX_INPUT_SIZE - 1), stdin)) {
@@ -1098,16 +1321,17 @@ main(int argc, char **argv)
 			replcon_stat_file_replace_context(replcon_info.
 							  filenames[i]);
 	}
-
 	replcon_info_free(&replcon_info);
 	for (i = 0; i < num_mounts; i++)
 		free(mounts[i]);
 	free(mounts);
-	
+	sefs_hash_destroy(hashtab);
 	return 0;
 	
 err:
 	replcon_info_free(&replcon_info);
 	if (mounts) free(mounts);
+	sefs_hash_destroy(hashtab);
 	return -1;
 }
+	
