@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 Tresys Technology, LLC
+/* Copyright (C) 2004-2006 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /* 
@@ -10,7 +10,6 @@
  
 /* libapol */
 #include "util.h"
-#include "sediff_rename_types.h"
 #include <policy.h>
 #include <policy-io.h>
 #include <policy-query.h>
@@ -31,7 +30,7 @@
 	#define SEDIFF_VERSION_NUM "UNKNOWN"
 #endif
 
-#define COPYRIGHT_INFO "Copyright (C) 2004,2005 Tresys Technology, LLC"
+#define COPYRIGHT_INFO "Copyright (C) 2004-2006 Tresys Technology, LLC"
 #define SEDIFF_GUI_PROG	"sediffx"
 
 char *p1_file, *p2_file;
@@ -63,7 +62,6 @@ void usage(const char *program_name, int brief)
 {
 	printf("%s (sediff ver. %s)\n\n", COPYRIGHT_INFO, SEDIFF_VERSION_NUM);
 	printf("Usage: %s [OPTIONS] POLICY1 POLICY2\n", program_name);
-	printf("Usage: %s -X [POLICY1 POLICY2]\n",program_name);
 	if(brief) {
 		printf("\n   Try %s --help for more help.\n\n", program_name);
 		return;
@@ -82,7 +80,6 @@ void usage(const char *program_name, int brief)
 "  -R, --roletrans   role transition rules\n"
 "  -A, --roleallows  role allow rules\n"  
 "  -C, --conds      conditionals and their rules\n\n"
-"  -X, --gui        launch the sediff gtk gui\n"
 "  -q, --quiet      only print different definitions\n"
 "  -s, --stats      print useful policy statics\n"
 "  -h, --help       display this help and exit\n"
@@ -1124,11 +1121,11 @@ int main (int argc, char **argv)
 	int classes, types, roles, users, all, stats, attributes, rallows, rtrans;
 	int optc, isids, conds, terules, rbac, bools, rt,gui, quiet;
 	policy_t *p1 = NULL, *p2 = NULL;
-	char *p1_file = NULL, *p2_file = NULL;
+	char *p1_file = NULL, *p2_file = NULL, *tmp = NULL;
 	apol_diff_result_t *diff = NULL;
 	ap_single_view_diff_t *svd = NULL;
 	unsigned int opts = POLOPT_NONE;
-	char prog_path[PATH_MAX];
+	int ret_code = 0;
 	
 	attributes = rallows = rtrans = classes = types = roles = users = bools = all = stats = isids = conds = terules = rbac = gui = quiet = 0;
 	while ((optc = getopt_long (argc, argv, "qXaActrubiTRCshv", longopts, NULL)) != -1)  {
@@ -1216,32 +1213,8 @@ int main (int argc, char **argv)
 	}
 	/* are we going to use the gui */
 	else if (gui == 1) {
-		snprintf(prog_path, PATH_MAX, "./%s", SEDIFF_GUI_PROG);
-		/* launch the gui with no arguments */
-		if (argc - optind == 0 ) {
-			rt = access(prog_path, X_OK);
-			if (rt == 0) {
-				rt = execvp(prog_path,argv);
-			} else {
-				rt = execvp(SEDIFF_GUI_PROG,argv);
-			}
-
-
-		}
-		/* launch the gui with file args */
-		else if (argc - optind == 2) {
-			rt = access(prog_path, X_OK);
-			if (rt == 0) {
-				rt = execvp(prog_path,argv);
-			} else {
-				rt = execvp(prog_path,argv);
-			}			
-		}
-		if (argc - optind != 0 && argc - optind != 2) {
-			usage(argv[0], 1);
-			exit(1);
-		}
-		exit(1);
+		printf("Use of 'sediff -X' is not supported.  Instead invoke 'sediffx' directly.\n");
+		exit(0);
 	}
 
 	else {
@@ -1277,6 +1250,7 @@ int main (int argc, char **argv)
 		exit(1);
 	}
 
+	printf("Calculating difference, this might take a while\n");
 	svd = ap_single_view_diff_new(opts, p1, p2, NULL);
 	if (svd == NULL) {
 		printf("Problem differentiating policies\n");
@@ -1285,8 +1259,10 @@ int main (int argc, char **argv)
 	diff = svd->diff;
 	
 	printf("Difference between policy 1 and policy 2: \n");
-	printf("   p1 (%6s, ver: %s): %s\n", policy_type(svd->diff->p1), get_policy_version_name(svd->diff->p1->version), p1_file);
-	printf("   p2 (%6s, ver: %s): %s\n\n", policy_type(svd->diff->p2), get_policy_version_name(svd->diff->p2->version), p2_file);
+	printf("   p1 (%s): %s\n", (tmp = get_policy_version_type_mls_str(svd->diff->p1)), p1_file);
+	free(tmp);
+	printf("   p2 (%s): %s\n\n", (tmp = get_policy_version_type_mls_str(svd->diff->p2)), p2_file);
+	free(tmp);
 	
 	if(classes || all)  {
 		if (!(quiet && (diff->diff1->num_classes == 0 && diff->diff2->num_classes == 0))) {
@@ -1349,9 +1325,11 @@ int main (int argc, char **argv)
 		printf("\n");
 	}
 
+	if (ap_single_view_diff_get_num_diffs(svd) > 0)
+		ret_code = 1;
 	ap_single_view_diff_destroy(svd);
 	close_policy(p1);
 	close_policy(p2);
-	exit(0);
+	exit(ret_code);
 }
 
